;Copyright (C) 1999 Konstantin Boldyshev <konst@voshod.com>
;Read LICENSE file for full copyright information (GNU GPL)
;
; file		: system.inc
; created	: 08-apr-1999
; modified	: 04-jul-1999
; version	: 0.03
; assembler	: nasm 0.98
; description	: system call and section macros
; author	: Konstantin Boldyshev <konst@voshod.com>
; comment	: you must include this file to do anything else
;
;0.01: 05-Jun-1999	initial release
;0.02: 17-Jun-1999	total rewrite, new macros & syscalls.
;0.03: 01-Jul-1999	fixed bugs in __setreg32, __syscall_setregs rewritten,
;			new syscalls and constants, added optimization for
;			size/speed

%ifndef __SYSTEM_INC
%define __SYSTEM_INC

%assign TRUE	0
%assign FALSE	1

%assign	NULL	0

;
;configuration parameters
;

%include "config.inc"

;------------------------------------------------------------------
;intellectual register assignment (generate smallest possible code)
;------------------------------------------------------------------

%macro __setreg32_08_lo 3
%ifnidn	%3,EMPTY
    xor	%2,%2
    mov	%1,%3
%endif
%endmacro

%macro __setreg32_08_hi 3
%ifnidn	%3,EMPTY
    xor	%2,%2
    mov	%1,(%3 / 0x100)
%endif
%endmacro

%macro __setreg32_16 3
%ifnidn	%3,EMPTY
    %if %3=0xFFFF
        xor	%2,%2
	dec	%1
    %else
	mov	%2,%3
    %endif
%endif
%endmacro

;
;here it is. long weird macro.
;

%macro __setreg32 2-3
%ifnidn	%2,EMPTY
 %if __OPTIMIZE=__O_SPEED
        %ifid %2
	    mov	%1,%2
	%elifnum %2 && %if %2=0x00000000
	    sub	%1,%1
	%else
	    mov	%1,%2
	%endif
 %else			;%if __OPTIMIZE=__O_SIZE
    %ifid %2
	mov	%1,%2
    %elifstr %2
	mov	%1,%2
    %elifnum %2
     %if %2=0x00000000
	xor	%1,%1
     %elif %2=0x00000001
    	xor	%1,%1
	inc	%1
     %elif %2=0x00000002
    	xor	%1,%1
	inc	%1
	inc	%1
     %elif %2=0xFFFFFFFF
	xor	%1,%1
	dec	%1
     %elif %2=0xFFFFFFFE
	xor	%1,%1
	dec	%1
	dec	%1
     %elif %2<0x00000100
	%ifidn %1,eax
	    __setreg32_08_lo al,%1,%2
	%elifidn %1,ebx
	    __setreg32_08_lo bl,%1,%2
	%elifidn %1,ecx
	    __setreg32_08_lo cl,%1,%2
	%elifidn %1,edx
	    __setreg32_08_lo dl,%1,%2
	%else
	    mov	%1,%2
	%endif
     %elif %2<0x00010000 
	%if (%2 % 0x100) = 0
	    %ifidn %1,eax
		__setreg32_08_hi ah,%1,%2
	    %elifidn %1,ebx
	        __setreg32_08_hi bh,%1,%2
	    %elifidn %1,ecx
		__setreg32_08_hi ch,%1,%2
	    %elifidn %1,edx
		__setreg32_08_hi dh,%1,%2
	    %else
		mov	%1,%2
	    %endif
	%else
	    %ifidn %1,eax
		__setreg32_16 ax,%1,%2
	    %elifidn %1,ebx
		__setreg32_16 bx,%1,%2
	    %elifidn %1,ecx
		__setreg32_16 cx,%1,%2
	    %elifidn %1,edx
		__setreg32_16 dx,%1,%2
	    %else
		mov	%1,%2
	    %endif
	%endif
     %endif
    %else
	mov	%1,%2
    %endif
 %endif
%endif
%endmacro

;
;
;

%assign	STDIN	0
%assign	STDOUT	1
%assign	STDERR	2

;
;
;

%assign WNOHANG	1

;
;
;

%assign O_RDONLY	0
%assign O_WRONLY	1
%assign O_RDWR		2
%assign O_ACCMODE	3
%assign O_CREAT		100q
%assign O_EXCL		200q
%assign O_NOCTTY	400q
%assign O_TRUNC		1000q
%assign O_APPEND	2000q
%assign O_NONBLOCK	4000q
%assign O_NDELAY	O_NONBLOCK
%assign O_SYNC		 10000q
%assign FASYNC		 20000q
%assign O_DIRECT	 40000q
%assign O_LARGEFILE	100000q
%assign O_DIRECTORY	200000q
%assign O_NOFOLLOW	400000q

%assign SEEK_SET	0
%assign SEEK_CUR	1
%assign SEEK_END	2

;
;
;

%assign SIG_DFL		0
%assign SIG_IGN		1
%assign SIG_ERR		-1

%assign SIGHUP		1
%assign SIGINT		2
%assign SIGQUIT		3
%assign SIGILL		4
%assign SIGTRAP		5
%assign SIGABRT		6
%assign SIGIOT		6
%assign SIGBUS		7
%assign SIGFPE		8
%assign SIGKILL		9
%assign SIGUSR1		10
%assign SIGSEGV		11
%assign SIGUSR2		12
%assign SIGPIPE		13
%assign SIGALRM		14
%assign SIGTERM		15
%assign SIGSTKFLT	16
%assign SIGCHLD		17
%assign SIGCLD		SIGCHLD
%assign SIGCONT		18
%assign SIGSTOP		19
%assign SIGTSTP		20
%assign SIGTTIN		21
%assign SIGTTOU		22
%assign SIGURG		23
%assign SIGXCPU		24
%assign SIGXFSZ		25
%assign SIGVTALRM	26
%assign SIGPROF		27
%assign SIGWINCH	28
%assign SIGIO		29
%assign SIGPOLL		SIGIO
%assign SIGPWR		30
%assign SIGUNUSED	31

;
;
;

%assign	LINUX_REBOOT_MAGIC1	0xfee1dead
%assign	LINUX_REBOOT_MAGIC2	0x28121969
%assign	LINUX_REBOOT_MAGIC2A	85072278
%assign	LINUX_REBOOT_MAGIC2B	369367448

%assign	LINUX_REBOOT_CMD_RESTART	0x01234567	;Restart system using default command and mode.
%assign	LINUX_REBOOT_CMD_HALT		0xCDEF0123	;Stop OS and give system control to ROM monitor, if any.
%assign	LINUX_REBOOT_CMD_CAD_ON		0x89ABCDEF	;Ctrl-Alt-Del sequence causes RESTART command.
%assign	LINUX_REBOOT_CMD_CAD_OFF	0x00000000	;Ctrl-Alt-Del sequence sends SIGINT to init task.
%assign	LINUX_REBOOT_CMD_POWER_OFF	0x4321FEDC	;Stop OS and remove all power from system, if possible.
%assign	LINUX_REBOOT_CMD_RESTART2	0xA1B2C3D4	;Restart system using given command string.

;
;
;

%assign PROT_READ	0x1
%assign PROT_WRITE	0x2
%assign PROT_EXEC	0x4
%assign PROT_NONE	0x0

%assign MAP_SHARED	0x01
%assign MAP_PRIVATE	0x02
%assign MAP_TYPE	0x0f
%assign MAP_FIXED	0x10
%assign MAP_ANONYMOUS	0x20

%assign MAP_GROWSDOWN	0x0100
%assign MAP_DENYWRITE	0x0800
%assign MAP_EXECUTABLE	0x1000
%assign MAP_LOCKED	0x2000
%assign MAP_NORESERVE	0x4000


;
;system calls
;

%assign __NR_exit		1
%assign __NR_fork		2
%assign __NR_read		3
%assign __NR_write		4
%assign __NR_open		5
%assign __NR_close		6
%assign __NR_waitpid		7
%assign __NR_creat		8
%assign __NR_link		9
%assign __NR_unlink		10
%assign __NR_execve		11
%assign __NR_chdir		12
%assign __NR_time		13
%assign __NR_mknod		14
%assign __NR_chmod		15
%assign __NR_lchown		16
%assign __NR_break		17
%assign __NR_oldstat		18
%assign __NR_lseek		19
%assign __NR_getpid		20
%assign __NR_mount		21
%assign __NR_umount		22
%assign __NR_setuid		23
%assign __NR_getuid		24
%assign __NR_stime		25
%assign __NR_ptrace		26
%assign __NR_alarm		27
%assign __NR_oldfstat		28
%assign __NR_pause		29
%assign __NR_utime		30
%assign __NR_stty		31
%assign __NR_gtty		32
%assign __NR_access		33
%assign __NR_nice		34
%assign __NR_ftime		35
%assign __NR_sync		36
%assign __NR_kill		37
%assign __NR_rename		38
%assign __NR_mkdir		39
%assign __NR_rmdir		40
%assign __NR_dup		41
%assign __NR_pipe		42
%assign __NR_times		43
%assign __NR_prof		44
%assign __NR_brk		45
%assign __NR_setgid		46
%assign __NR_getgid		47
%assign __NR_signal		48
%assign __NR_geteuid		49
%assign __NR_getegid		50
%assign __NR_acct		51
%assign __NR_umount2		52
%assign __NR_lock		53
%assign __NR_ioctl		54
%assign __NR_fcntl		55
%assign __NR_mpx		56
%assign __NR_setpgid		57
%assign __NR_ulimit		58
%assign __NR_oldolduname	59
%assign __NR_umask		60
%assign __NR_chroot		61
%assign __NR_ustat		62
%assign __NR_dup2		63
%assign __NR_getppid		64
%assign __NR_getpgrp		65
%assign __NR_setsid		66
%assign __NR_sigaction		67
%assign __NR_sgetmask		68
%assign __NR_ssetmask		69
%assign __NR_setreuid		70
%assign __NR_setregid		71
%assign __NR_sigsuspend		72
%assign __NR_sigpending		73
%assign __NR_sethostname	74
%assign __NR_setrlimit		75
%assign __NR_getrlimit		76
%assign __NR_getrusage		77
%assign __NR_gettimeofday	78
%assign __NR_settimeofday	79
%assign __NR_getgroups		80
%assign __NR_setgroups		81
%assign __NR_select		82
%assign __NR_symlink		83
%assign __NR_oldlstat		84
%assign __NR_readlink		85
%assign __NR_uselib		86
%assign __NR_swapon		87
%assign __NR_reboot		88
%assign __NR_readdir		89
%assign __NR_mmap		90
%assign __NR_munmap		91
%assign __NR_truncate		92
%assign __NR_ftruncate		93
%assign __NR_fchmod		94
%assign __NR_fchown		95
%assign __NR_getpriority	96
%assign __NR_setpriority	97
%assign __NR_profil		98
%assign __NR_statfs		99
%assign __NR_fstatfs		100
%assign __NR_ioperm		101
%assign __NR_socketcall		102
%assign __NR_syslog		103
%assign __NR_setitimer		104
%assign __NR_getitimer		105
%assign __NR_stat		106
%assign __NR_lstat		107
%assign __NR_fstat		108
%assign __NR_olduname		109
%assign __NR_iopl		110
%assign __NR_vhangup		111
%assign __NR_idle		112
%assign __NR_vm86old		113
%assign __NR_wait4		114
%assign __NR_swapoff		115
%assign __NR_sysinfo		116
%assign __NR_ipc		117
%assign __NR_fsync		118
%assign __NR_sigreturn		119
%assign __NR_clone		120
%assign __NR_setdomainname	121
%assign __NR_uname		122
%assign __NR_modify_ldt		123
%assign __NR_adjtimex		124
%assign __NR_mprotect		125
%assign __NR_sigprocmask	126
%assign __NR_create_module	127
%assign __NR_init_module	128
%assign __NR_delete_module	129
%assign __NR_get_kernel_syms	130
%assign __NR_quotactl		131
%assign __NR_getpgid		132
%assign __NR_fchdir		133
%assign __NR_bdflush		134
%assign __NR_sysfs		135
%assign __NR_personality	136
%assign __NR_afs_syscall	137
%assign __NR_setfsuid		138
%assign __NR_setfsgid		139
%assign __NR__llseek		140
%assign __NR_getdents		141
%assign __NR__newselect		142
%assign __NR_flock		143
%assign __NR_msync		144
%assign __NR_readv		145
%assign __NR_writev		146
%assign __NR_getsid		147
%assign __NR_fdatasync		148
%assign __NR__sysctl		149
%assign __NR_mlock		150
%assign __NR_munlock		151
%assign __NR_mlockall		152
%assign __NR_munlockall		153
%assign __NR_sched_setparam		154
%assign __NR_sched_getparam		155
%assign __NR_sched_setscheduler		156
%assign __NR_sched_getscheduler		157
%assign __NR_sched_yield		158
%assign __NR_sched_get_priority_max	159
%assign __NR_sched_get_priority_min	160
%assign __NR_sched_rr_get_interval	161
%assign __NR_nanosleep		162
%assign __NR_mremap		163
%if KERNEL=22
%assign __NR_setresuid		164
%assign __NR_getresuid		165
%assign __NR_vm86		166
%assign __NR_query_module	167
%assign __NR_poll		168
%assign __NR_nfsservctl		169
%assign __NR_setresgid		170
%assign __NR_getresgid		171
%assign __NR_prctl              172
%assign __NR_rt_sigreturn	173
%assign __NR_rt_sigaction	174
%assign __NR_rt_sigprocmask	175
%assign __NR_rt_sigpending	176
%assign __NR_rt_sigtimedwait	177
%assign __NR_rt_sigqueueinfo	178
%assign __NR_rt_sigsuspend	179
%assign __NR_pread		180
%assign __NR_pwrite		181
%assign __NR_chown		182
%assign __NR_getcwd		183
%assign __NR_capget		184
%assign __NR_capset		185
%assign __NR_sigaltstack	186
%assign __NR_sendfile		187
%endif
%assign __NR_getpmsg		188
%assign __NR_putpmsg		189
%if KERNEL=22
%assign __NR_vfork		190
%endif

;
;
;

%macro __syscall		0-1
%if %0>0
	__setreg32	eax,%{1}
%endif
	int	0x80
%endmacro

%macro	__syscall_setregs 1-6
%if %1>0
 %if %1>1
  %if %1>2
   %if %1>3
    %if %1>4
    __setreg32	edi,%6
    %endif
    __setreg32	esi,%5
   %endif
    __setreg32	edx,%4
  %endif
   __setreg32	ecx,%3
 %endif
  __setreg32	ebx,%2
%endif
%endmacro

;
;
;

%macro sys_exit 0-1
	__syscall_setregs %0,%1
	__syscall    __NR_exit
%endmacro

%macro sys_exit0 0
	xor	eax,eax
	mov	ebx,eax
	inc	eax
	__syscall
%endmacro

%macro sys_exit1 0
	xor	eax,eax
	inc	eax
	mov	ebx,eax
	__syscall
%endmacro

%macro sys_fork 0
	__syscall __NR_fork
%endmacro

%macro sys_read 0-3
	__syscall_setregs %0,%1,%2,%3
	__syscall __NR_read
%endmacro

%macro sys_write 0-3
	__syscall_setregs %0,%1,%2,%3
	__syscall __NR_write
%endmacro

%macro sys_open 0-3
	__syscall_setregs %0,%1,%2,%3
%if %0<3
	__setreg32 edx,600q
%endif
	__syscall __NR_open
%endmacro

%macro sys_close 0-1
	__syscall_setregs %0,%1
	__syscall __NR_close
%endmacro

%macro sys_execve 0-3
	__syscall_setregs %0,%1,%2,%3
	__syscall __NR_execve
%endmacro

%macro sys_lseek 0-3
	__syscall_setregs %0,%1,%2,%3
	__syscall __NR_lseek
%endmacro

%macro sys_mount 0-5
	__syscall_setregs %0,%1,%2,%3,%4,%5
	__syscall __NR_mount
%endmacro

%macro sys_umount 0-1
	__syscall_setregs %0,%1
	__syscall __NR_umount
%endmacro

%macro sys_alarm 0-1
	__syscall_setregs %0,%1
	__syscall __NR_alarm
%endmacro

%macro sys_pause 0
	__syscall __NR_pause
%endmacro

%macro sys_sync 0
	__syscall __NR_sync
%endmacro

%macro sys_kill 0-2
	__syscall_setregs %0,%1,%2
	__syscall __NR_kill
%endmacro

%macro sys_mkdir 0-2
	__syscall_setregs %0,%1,%2
	__syscall __NR_mkdir
%endmacro

%macro sys_rmdir 0-1
	__syscall_setregs %0,%1
	__syscall __NR_rmdir
%endmacro

%macro sys_signal 0-2
	__syscall_setregs %0,%1,%2
	__syscall __NR_signal
%endmacro

%macro sys_umount2 0-2
	__syscall_setregs %0,%1,%2
	__syscall __NR_umount2
%endmacro

%macro sys_sethostname 0-2
	__syscall_setregs %0,%1,%2
	__syscall __NR_sethostname
%endmacro

%macro sys_ioctl 0-3
	__syscall_setregs %0,%1,%2,%3
	__syscall __NR_ioctl
%endmacro

%macro sys_readlink 0-3
	__syscall_setregs %0,%1,%2,%3
	__syscall __NR_readlink
%endmacro

%macro sys_swapon 0-2
	__syscall_setregs %0,%1,%2
	__syscall __NR_swapon
%endmacro

%macro sys_reboot 0-4
	__syscall_setregs %0,%1,%2,%3,%4
	__syscall __NR_reboot
%endmacro

%macro sys_mmap 0-1
	__syscall_setregs %0,%1
	__syscall __NR_mmap
%endmacro

%macro sys_munmap 0-2
	__syscall_setregs %0,%1,%2
	__syscall __NR_munmap
%endmacro

%macro sys_ioperm 0-3
	__syscall_setregs %0,%1,%2,%3
	__syscall __NR_ioperm
%endmacro

%macro sys_socketcall 0-3
	__syscall_setregs %0,%1,%2,%3
        __syscall __NR_socketcall
%endmacro

%macro sys_syslog 0-3
	__syscall_setregs %0,%1,%2,%3
	__syscall __NR_syslog
%endmacro

%macro sys_stat 0-2
	__syscall_setregs %0,%1,%2
	__syscall __NR_stat
%endmacro

%macro sys_fstat 0-2
	__syscall_setregs %0,%1,%2
	__syscall __NR_fstat
%endmacro

%macro sys_lstat 0-2
	__syscall_setregs %0,%1,%2
	__syscall __NR_lstat
%endmacro

%macro sys_wait4 0-4
	__syscall_setregs %0,%1,%2,%3,%4
	__syscall __NR_wait4
%endmacro

%macro sys_swapoff 0-1
	__syscall_setregs %0,%1
	__syscall __NR_swapoff
%endmacro

%macro sys_sysinfo 0-1
	__syscall_setregs %0,%1
	__syscall __NR_sysinfo
%endmacro

%macro sys_setdomainname 0-2
	__syscall_setregs %0,%1,%2
	__syscall __NR_setdomainname
%endmacro

%macro sys_uname 0-1
	__syscall_setregs %0,%1
	__syscall __NR_uname
%endmacro

%macro sys_delete_module 0-1
	__syscall_setregs %0,%1
	__syscall __NR_delete_module
%endmacro

%macro sys_bdflush 0-2
	__syscall_setregs %0,%1,%2
	__syscall __NR_bdflush
%endmacro

%macro sys_getdents 0-3
	__syscall_setregs %0,%1,%2,%3
	__syscall __NR_getdents
%endmacro

%macro sys_nanosleep 0-2
	__syscall_setregs %0,%1,%2
	__syscall __NR_nanosleep
%endmacro

%macro sys_getcwd 0-2
	__syscall_setregs %0,%1,%2
	__syscall __NR_getcwd
%endmacro

%macro sys_sendfile 0-4
	__syscall_setregs %0,%1,%2,%3,%4
	__syscall __NR_sendfile
%endmacro


;
;
;

%define START _start

%macro CODESEG 0
%if __OPTIMIZE=__O_SPEED
    section	.text	align=16
%elif __OPTIMIZE=__O_SIZE
    section	.text	align=0
%endif
    global	START
%endmacro

%macro UDATASEG 0
section	.bss
%endmacro

%macro DATASEG 0
%if __OPTIMIZE=__O_SPEED
    section	.data	align=16
%elif __OPTIMIZE=__O_SIZE
    section	.data	align=0
%endif
%endmacro

%endif