#include <WINGs.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <cdaudio.h>

#include "pixmaps.h"

#define PROGRAM_VERSION "1.1"
#define DEFAULT_DEVICE "/dev/cdrom"

typedef struct
{
   WMWindow *win;
   WMLabel *lblBack;
   WMLabel *lblIcon;
   WMButton *btnTrack[20];
   WMButton *btnPlayAll;
   WMButton *btnPlayShuffle;
   WMButton *btnPlayTrack;
   WMButton *btnRepeat;
   WMLabel *lblTrack;
   WMLabel *lblTime;
   WMButton *btnRec;
   WMButton *btnPlay;
   WMButton *btnPause;
   WMButton *btnStop;
   WMButton *btnPrev;
   WMButton *btnNext;
   WMButton *btnEject;
   WMSlider *sldPos;
} cdplayer;

typedef struct
{
   WMWindow *win;
   WMPopUpButton *popMenu;
   WMFrame *frmEntry;
   WMFrame *frmNames;
   WMTextField *txtArtist;
   WMTextField *txtTitle;
   WMFrame *frmGenre;
   WMPopUpButton *popGenre;
   WMFrame *frmTrack;
   WMList *lstTracks;
   WMTextField *txtTrack;
   WMButton *btnLookup;
   WMButton *btnSaveEntry;
   WMFrame *frmSites;
   WMFrame *frmName;
   WMList *lstNames;
   WMTextField *txtName;
   WMButton *btnRemove;
   WMButton *btnAdd;
   WMFrame *frmProtocol;
   WMPopUpButton *popProtocol;
   WMFrame *frmPort;
   WMTextField *txtPort;
   WMFrame *frmAddressing;
   WMTextField *txtAddressing;
   WMButton *btnRefresh;
   WMButton *btnSaveSites;
} editor;

typedef struct
{
   cdplayer *cdplayer;
   editor *editor;
} app;

struct disc_info disc;
struct disc_data data;
struct cddb_host host;
struct cddb_conf conf;
struct cddb_serverlist list;
struct cddb_server proxy;

Display *dpy;
int last = -1;
int editing = 0;
int position = 0;
int play_mode = 0;
int play_track = 0;
int random_track = 0;
int end_track = 0;
int total_tracks = 0;
int track_mode = 0;
int last_mode = 0;
int time_mode = 0;
int cd_desc;
int play_list[MAX_TRACKS];

void Close(WMWidget *, cdplayer *);
void CreateDisplay(WMScreen *);
void Track(WMWidget *, cdplayer *);
void ModeClick(WMWidget *, cdplayer *);
void Play(WMWidget *, cdplayer *);
void Pause(WMWidget *, cdplayer *);
void Stop(WMWidget *, cdplayer *);
void Prev(WMWidget *, cdplayer *);
void Next(WMWidget *, cdplayer *);
void Eject(WMWidget *, cdplayer *);
void Skip(WMWidget *, cdplayer *);
void Timer(void *clientData);
void Update(void *clientData);

int cddb_get_siteslist(struct cddb_serverlist *, struct cddb_host *);

void wAbort()
{
   exit(1);
}

void usage(const char *name)
{
   const char *tmp;
   tmp = strrchr(name, '/');
   if (tmp != NULL) name = tmp+1;
   fprintf(stderr,
      "Usage: %s [-hv] [-d device]\n"
      "\n"
      "\t-h\t\tthis help.\n"
      "\t-v\t\tshow version.\n"
      "\t-d device\tuse alternate CDROM device (default %s)\n"
      "\n",
      name, DEFAULT_DEVICE);
}

void version(const char *name)
{
   const char *tmp;
   tmp = strrchr(name, '/');
   if (tmp != NULL) name = tmp+1;
   fprintf(stderr, "%s version %s\n", name, PROGRAM_VERSION);
}

int main(int argc, char **argv)
{
   int ret;
   char *device;
   WMScreen *scr;
   device = DEFAULT_DEVICE;
   while ((ret = getopt(argc, argv, "hvd:")) != EOF)
   {
      switch (ret)
      {
         case 'h':
            usage(argv[0]);
            return 1;
         case 'v':
            version(argv[0]);
            return 1;
         case 'd':
            device = optarg;
            break;
         default:
            usage(argv[0]);
            return 1;
      }
   }
   if ((cd_desc = cd_init_device(device)) < 0)
   {
      if (errno == EBUSY)
         printf("%s already appears to be mounted.  Sorry.\n", device);
      else perror(device);
   }
   WMInitializeApplication("cdplayer", &argc, argv);
   if (!(dpy = XOpenDisplay("")))
   {
      fprintf(stderr, "Can't open display.\n");
      exit(0);
   }
   scr = WMCreateScreen(dpy, DefaultScreen(dpy));
   CreatePixmaps(scr);
   CreateDisplay(scr);
   WMScreenMainLoop(scr);
   return 0;
}

char *charalloc(int i)
{
   char *c = (char *)malloc(i);
   if (!c)
   {
      fprintf(stderr, "malloc() error.\n");
      exit(1);
   }
   return c;
}

void SetButtonImages(WMWidget *self, WMPixmap *image, WMPixmap *altimage)
{
   WMSetButtonImage(self, image);
   WMSetButtonAltImage(self, altimage);
}

void ScrollName(void *clientData)
{
   WMHandlerID timer;
   cdplayer *w = (cdplayer *)clientData;
   timer = WMAddTimerHandler(200, ScrollName, w);
   if (track_mode == 1)
   {
      char *tmp;
      WMFont *font = WMBoldSystemFontOfSize(WMWidgetScreen(w->win), 12);
      tmp = charalloc(strlen(data.data_track[play_track-1].track_name)+1);
      strcpy(tmp, data.data_track[play_track-1].track_name);
      if (WMWidthOfString(font, tmp, strlen(tmp)) > 160)
      {
         char *tmp2, *tmp3;
         tmp2 = charalloc(strlen(tmp)*2+6);
         sprintf(tmp2, "%s *** %s", tmp, tmp);
         tmp3 = charalloc(strlen(tmp2)+1);
         strcpy(tmp3, &tmp2[position]);
         while (WMWidthOfString(font, tmp3, strlen(tmp3)) > 160)
            tmp3[strlen(tmp3)-1] = '\0';
         if (position < strlen(tmp)+4) position ++;
         else position = 0;
         WMSetLabelText(w->lblTrack, tmp3);
         free(tmp2);
         free(tmp3);
      }
      else WMSetLabelText(w->lblTrack, tmp);
      free(tmp);
      WMReleaseFont(font);
   }
   else WMDeleteTimerHandler(timer);
}

void TrackUpdate(WMWidget *self)
{
   if (track_mode == 0)
   {
      char tmp[20];
      sprintf(tmp, "%d%s", play_track,
         disc.disc_track[play_track-1].track_type == CDAUDIO_TRACK_DATA ?
         " (data)" : "");
      WMSetLabelText(self, tmp);
   }
}

void TimeUpdate(cdplayer *w)
{
   char tmp[25];
   int min, sec, length, pos, total, current;
   length = disc.disc_track[play_track-1].track_length.minutes*60+
      disc.disc_track[play_track-1].track_length.seconds;
   pos = disc.disc_track_time.minutes*60+disc.disc_track_time.seconds;
   total = disc.disc_length.minutes*60+disc.disc_length.seconds;
   current = disc.disc_time.minutes*60+disc.disc_time.seconds;
   switch (time_mode)
   {
      case 0:
         min = disc.disc_track_time.minutes;
         sec = disc.disc_track_time.seconds;
         sprintf(tmp, "%02d:%02d", min, sec);
         break;
      case 1:
         min = div(length-pos, 60).quot;
         sec = div(length-pos, 60).rem;
         sprintf(tmp, "-%02d:%02d", min, sec);
         break;
      case 2:
         min = disc.disc_time.minutes;
         sec = disc.disc_time.seconds;
         sprintf(tmp, "%02d:%02d", min, sec);
         break;
      case 3:
         min = div(total-current, 60).quot;
         sec = div(total-current, 60).rem;
         sprintf(tmp, "-%02d:%02d", min, sec);
         break;
   }
   WMSetLabelText(w->lblTime, tmp);
   if (WMGetSliderMaxValue(w->sldPos) != length)
      WMSetSliderMaxValue(w->sldPos, length);
   WMSetSliderValue(w->sldPos, pos);
}

void ZeroOrTotal(cdplayer *w)
{
   char tmp[25] = "00:00";
   if (time_mode == 1)
      sprintf(tmp, "-%02d:%02d",
         disc.disc_track[play_track-1].track_length.minutes,
         disc.disc_track[play_track-1].track_length.seconds);
   else if (time_mode == 3)
      sprintf(tmp, "-%02d:%02d",
         disc.disc_length.minutes, disc.disc_length.seconds);
   WMSetLabelText(w->lblTime, tmp);
   WMSetSliderValue(w->sldPos, 0);
}

void Reset(cdplayer *w)
{
   TrackUpdate(w->lblTrack);
   ZeroOrTotal(w);
   SetButtonImages(w->btnPlay, playbtn, playbtn);
   SetButtonImages(w->btnPause, pausebtnd, pausebtnd);
   SetButtonImages(w->btnStop, stopbtnd, stopbtnd);
}

void PlayOrStop(cdplayer *w)
{
   if (disc.disc_mode == CDAUDIO_PLAYING)
   {
      if (play_mode == 0) end_track = total_tracks;
      else end_track = play_track;
      cd_play_track(cd_desc, play_track, end_track);
   }
   else
   {
      cd_stop(cd_desc);
      SetButtonImages(w->btnPlay, playbtn, playbtn);
      SetButtonImages(w->btnPause, pausebtnd, pausebtnd);
      SetButtonImages(w->btnStop, stopbtnd, stopbtnd);
   }
   TrackUpdate(w->lblTrack);
   position = 0;
   ZeroOrTotal(w);
}

void randomTrack(void)
{
   if (play_mode == 1)
   {
      int i;
      for (i = 0; i != total_tracks; i++)
         if (play_list[i] == play_track) random_track = i;
   }
}

void UpdateEntry(editor *e, struct disc_data *d)
{
   int i;
   char *tmp;
   WMSetTextFieldText(e->txtArtist, d->data_artist);
   WMSetTextFieldText(e->txtTitle, d->data_title);
   WMSetPopUpButtonSelectedItem(e->popGenre, d->data_genre-1);
   WMClearList(e->lstTracks);
   for (i = 0; i != total_tracks; i++)
   {
      tmp = charalloc(strlen(d->data_track[i].track_name)+15);
      sprintf(tmp, "%d. %s", i+1, d->data_track[i].track_name);
      WMAddListItem(e->lstTracks, tmp);
      free(tmp);
   }
}

void UpdateSites(editor *e)
{
   int i;
   WMClearList(e->lstNames);
   for (i = 0; i != list.list_len; i++)
      WMAddListItem(e->lstNames, list.list_host[i].host_server.server_name);
   WMSetTextFieldText(e->txtName, "");
   WMSetPopUpButtonSelectedItem(e->popProtocol, 0);
   WMSetTextFieldText(e->txtPort, "");
   WMSetTextFieldText(e->txtAddressing, "");
   WMSetButtonEnabled(e->btnRemove, False);
   last = -1;
}

void trackClick(WMWidget *self, editor *e)
{
   char *tmp = strchr(WMGetListSelectedItem(self)->text, ' ');
   WMSetTextFieldText(e->txtTrack, tmp+1);
}

void trackDoubleClick(WMWidget *self, cdplayer *w)
{
   play_track = WMGetListSelectedItemRow(self)+1;
   randomTrack();
   PlayOrStop(w);
}

void saveLast(editor *e)
{
   if (last > -1)
   {
      strcpy(list.list_host[last].host_server.server_name,
         WMGetTextFieldText(e->txtName));
      list.list_host[last].host_protocol =
         WMGetPopUpButtonSelectedItem(e->popProtocol);
      list.list_host[last].host_server.server_port =
         atoi(WMGetTextFieldText(e->txtPort));
      strcpy(list.list_host[last].host_addressing,
         WMGetTextFieldText(e->txtAddressing));
   }
}

void nameClick(WMWidget *self, editor *e)
{
   char tmp[15];
   int i = WMGetListSelectedItemRow(self);
   saveLast(e);
   WMSetTextFieldText(e->txtName, WMGetListItem(self, i)->text);
   if (list.list_host[i].host_protocol)
   {
      if (list.list_host[i].host_protocol == CDINDEX_MODE_HTTP)
         WMSetPopUpButtonSelectedItem(e->popProtocol, 2);
      else WMSetPopUpButtonSelectedItem(e->popProtocol, 1);
   }
   else WMSetPopUpButtonSelectedItem(e->popProtocol, 0);
   sprintf(tmp, "%d", list.list_host[i].host_server.server_port);
   WMSetTextFieldText(e->txtPort, tmp);
   WMSetTextFieldText(e->txtAddressing, list.list_host[i].host_addressing);
   WMSetButtonEnabled(e->btnRemove, True);
   last = i;
}

void CloseEditor(WMWidget *self, app *a)
{
   cdplayer *w = a->cdplayer;
   editor *e = a->editor;
   WMUnmapWidget(self);
   WMDestroyWidget(self);
   SetButtonImages(w->btnEject, ejectbtn, ejectbtnh);
   free(e);
   free(a);
   editing = 0;
}

void LookupButton(WMWidget *self, editor *e)
{
   struct disc_data tmp;
   WMSetButtonEnabled(self, False);
   if (cddb_read_data(cd_desc, &tmp) < 0)
      WMRunAlertPanel(WMWidgetScreen(self), e->win, "Error",
         "Could not download disc data from server", "OK", NULL, NULL);
   else UpdateEntry(e, &tmp);
   WMSetButtonEnabled(self, True);
}

void SaveEntryButton(WMWidget *self, editor *e)
{
   int i;
   char *tmp;
   cddb_erase_entry(&data);
   strcpy(data.data_artist, WMGetTextFieldText(e->txtArtist));
   strcpy(data.data_title, WMGetTextFieldText(e->txtTitle));
   data.data_genre = WMGetPopUpButtonSelectedItem(e->popGenre)+1;
   for (i = 0; i != total_tracks; i++)
   {
      tmp = strchr(WMGetListItem(e->lstTracks, i)->text, ' ');
      strcpy(data.data_track[i].track_name, tmp+1);
   }
   cddb_write_disc_data(cd_desc, &data);
}

void RemoveButton(WMWidget *self, editor *e)
{
   int i, pos;
   pos = WMGetListSelectedItemRow(e->lstNames);
   for (i = pos; i != list.list_len-1; i++)
      memcpy(&list.list_host[i], &list.list_host[i+1],
         sizeof(struct cddb_host));
   list.list_len --;
   WMRemoveListItem(e->lstNames, pos);
   WMSetTextFieldText(e->txtName, "");
   WMSetPopUpButtonSelectedItem(e->popProtocol, 0);
   WMSetTextFieldText(e->txtPort, "");
   WMSetTextFieldText(e->txtAddressing, "");
   WMSetButtonEnabled(self, False);
   last = -1;
}

void AddButton(WMWidget *self, editor *e)
{
   if (list.list_len < CDDB_MAX_SERVERS)
   {
      WMInsertListItem(e->lstNames, list.list_len, "new site");
      strcpy(list.list_host[list.list_len].host_server.server_name,
         "new site");
      list.list_host[list.list_len].host_protocol = 0;
      list.list_host[list.list_len].host_server.server_port = 0;
      strcpy(list.list_host[list.list_len].host_addressing, "");
      list.list_len ++;
   }
}

void RefreshButton(WMWidget *self, editor *e)
{
   char *server;
   WMSetButtonEnabled(self, False);
   server = WMRunInputPanel(WMWidgetScreen(self), e->win, "Refresh",
      "Enter server URL:", "cddbp://freedb.freedb.org:888", "OK", "Cancel");
   if (server != NULL)
   {
      cddb_process_url(&host, server);
      if (cddb_get_siteslist(&list, &host) < 0)
         WMRunAlertPanel(WMWidgetScreen(self), e->win, "Error",
            "Could not download sites list from server", "OK", NULL, NULL);
      else UpdateSites(e);
      free(server);
   }
   WMSetButtonEnabled(self, True);
}

void SaveSitesButton(WMWidget *self, editor *e)
{
   saveLast(e);
   conf.conf_access = CDDB_ACCESS_REMOTE;
   cddb_write_serverlist(&conf, &list, &proxy);
}

void changePage(WMWidget *self, editor *e)
{
   int page = WMGetPopUpButtonSelectedItem(self);
   if (page == 0)
   {
      WMMapWidget(e->frmEntry);
      WMUnmapWidget(e->frmSites);
   }
   else
   {
      WMMapWidget(e->frmSites);
      WMUnmapWidget(e->frmEntry);
   }
}

static void changedTrack(void *observerData, WMNotification *notification)
{
   editor *e = (editor *)observerData;
   int i = WMGetListSelectedItemRow(e->lstTracks);
   if (i > -1)
   {
      WMTextField *t = (WMTextField *)WMGetNotificationObject(notification);
      WMListItem *item = WMGetListSelectedItem(e->lstTracks);
      char *tmp = charalloc(strlen(WMGetTextFieldText(t))+15);
      sprintf(tmp, "%d. %s", i+1, WMGetTextFieldText(t));
      free(item->text);
      item->text = tmp;
      WMRedisplayWidget(e->lstTracks);
   }
}

static void changedName(void *observerData, WMNotification *notification)
{
   editor *e = (editor *)observerData;
   if (WMGetListSelectedItemRow(e->lstNames) > -1)
   {
      WMTextField *t = (WMTextField *)WMGetNotificationObject(notification);
      WMListItem *item = WMGetListSelectedItem(e->lstNames);
      free(item->text);
      item->text = WMGetTextFieldText(t);
      WMRedisplayWidget(e->lstNames);
   }
}

void OpenEditor(WMScreen *scr, cdplayer *w)
{
   int i;
   app *a;
   editor *e;
   a = (app *)malloc(sizeof(app));
   e = (editor *)malloc(sizeof(editor));
   if (!e || !a)
   {
      fprintf(stderr, "Couldn't malloc enough for display.\n");
      wAbort(False);
   }
   a->cdplayer = w;
   a->editor = e;

   e->win = WMCreateWindow(scr, "Editor");
   WMSetWindowTitle(e->win, "CDDB Editor");
   WMSetWindowMiniwindowTitle(e->win, "Editor");
   WMSetWindowCloseAction(e->win, (WMAction *)CloseEditor, a);
   WMSetWindowMinSize(e->win, 300, 450);
   WMSetWindowMaxSize(e->win, 300, 450);
   WMResizeWidget(e->win, 300, 450);

   e->popMenu = WMCreatePopUpButton(e->win);
   WMResizeWidget(e->popMenu, 160, 20);
   WMMoveWidget(e->popMenu, 70, 10);
   WMSetPopUpButtonAction(e->popMenu, (WMAction *)changePage, e);
   WMAddPopUpButtonItem(e->popMenu, "CDDB Entry");
   WMAddPopUpButtonItem(e->popMenu, "CDDB Sites");
   WMSetPopUpButtonSelectedItem(e->popMenu, 0);

   e->frmEntry = WMCreateFrame(e->win);
   WMResizeWidget(e->frmEntry, 300, 420);
   WMMoveWidget(e->frmEntry, 0, 30);
   WMSetFrameRelief(e->frmEntry, WRFlat);

   e->frmNames = WMCreateFrame(e->frmEntry);
   WMResizeWidget(e->frmNames, 280, 80);
   WMMoveWidget(e->frmNames, 10, 10);
   WMSetFrameTitle(e->frmNames, "Artist / Title");

   e->txtArtist = WMCreateTextField(e->frmNames);
   WMResizeWidget(e->txtArtist, 260, 20);
   WMMoveWidget(e->txtArtist, 10, 20);

   e->txtTitle = WMCreateTextField(e->frmNames);
   WMResizeWidget(e->txtTitle, 260, 20);
   WMMoveWidget(e->txtTitle, 10, 50);

   WMMapSubwidgets(e->frmNames);

   e->frmGenre = WMCreateFrame(e->frmEntry);
   WMResizeWidget(e->frmGenre, 280, 50);
   WMMoveWidget(e->frmGenre, 10, 100);
   WMSetFrameTitle(e->frmGenre, "Genre");

   e->popGenre = WMCreatePopUpButton(e->frmGenre);
   WMResizeWidget(e->popGenre, 260, 20);
   WMMoveWidget(e->popGenre, 10, 20);
   for (i = 1; i != 12; i++)
      WMAddPopUpButtonItem(e->popGenre, cddb_genre(i));

   WMMapSubwidgets(e->frmGenre);

   e->frmTrack = WMCreateFrame(e->frmEntry);
   WMResizeWidget(e->frmTrack, 280, 214);
   WMMoveWidget(e->frmTrack, 10, 160);
   WMSetFrameTitle(e->frmTrack, "Tracks");

   e->lstTracks = WMCreateList(e->frmTrack);
   WMResizeWidget(e->lstTracks, 260, 154);
   WMMoveWidget(e->lstTracks, 10, 20);
   WMSetListAction(e->lstTracks, (WMAction *)trackClick, e);
   WMSetListDoubleAction(e->lstTracks, (WMAction *)trackDoubleClick, w);

   e->txtTrack = WMCreateTextField(e->frmTrack);
   WMResizeWidget(e->txtTrack, 260, 20);
   WMMoveWidget(e->txtTrack, 10, 184);
   WMAddNotificationObserver(changedTrack, e, WMTextDidChangeNotification,
      e->txtTrack);

   WMMapSubwidgets(e->frmTrack);

   e->btnLookup = WMCreateCommandButton(e->frmEntry);
   WMResizeWidget(e->btnLookup, 80, 26);
   WMMoveWidget(e->btnLookup, 120, 384);
   WMSetButtonText(e->btnLookup, "Lookup");
   WMSetButtonAction(e->btnLookup, (WMAction *)LookupButton, e);

   e->btnSaveEntry = WMCreateCommandButton(e->frmEntry);
   WMResizeWidget(e->btnSaveEntry, 80, 26);
   WMMoveWidget(e->btnSaveEntry, 210, 384);
   WMSetButtonText(e->btnSaveEntry, "Save");
   WMSetButtonAction(e->btnSaveEntry, (WMAction *)SaveEntryButton, e);

   WMMapSubwidgets(e->frmEntry);

   e->frmSites = WMCreateFrame(e->win);
   WMResizeWidget(e->frmSites, 300, 420);
   WMMoveWidget(e->frmSites, 0, 30);
   WMSetFrameRelief(e->frmSites, WRFlat);

   e->frmName = WMCreateFrame(e->frmSites);
   WMResizeWidget(e->frmName, 280, 184);
   WMMoveWidget(e->frmName, 10, 10);
   WMSetFrameTitle(e->frmName, "Name");

   e->lstNames = WMCreateList(e->frmName);
   WMResizeWidget(e->lstNames, 260, 94);
   WMMoveWidget(e->lstNames, 10, 20);
   WMSetListAction(e->lstNames, (WMAction *)nameClick, e);

   e->txtName = WMCreateTextField(e->frmName);
   WMResizeWidget(e->txtName, 260, 20);
   WMMoveWidget(e->txtName, 10, 124);
   WMAddNotificationObserver(changedName, e, WMTextDidChangeNotification,
      e->txtName);

   e->btnRemove = WMCreateCommandButton(e->frmName);
   WMResizeWidget(e->btnRemove, 70, 20);
   WMMoveWidget(e->btnRemove, 120, 154);
   WMSetButtonText(e->btnRemove, "Remove");
   WMSetButtonAction(e->btnRemove, (WMAction *)RemoveButton, e);

   e->btnAdd = WMCreateCommandButton(e->frmName);
   WMResizeWidget(e->btnAdd, 70, 20);
   WMMoveWidget(e->btnAdd, 200, 154);
   WMSetButtonText(e->btnAdd, "Add");
   WMSetButtonAction(e->btnAdd, (WMAction *)AddButton, e);

   WMMapSubwidgets(e->frmName);

   e->frmProtocol = WMCreateFrame(e->frmSites);
   WMResizeWidget(e->frmProtocol, 280, 50);
   WMMoveWidget(e->frmProtocol, 10, 204);
   WMSetFrameTitle(e->frmProtocol, "Protocol");

   e->popProtocol = WMCreatePopUpButton(e->frmProtocol);
   WMResizeWidget(e->popProtocol, 260, 20);
   WMMoveWidget(e->popProtocol, 10, 20);
   WMAddPopUpButtonItem(e->popProtocol, "CDDB Mode CDDBP");
   WMAddPopUpButtonItem(e->popProtocol, "CDDB Mode HTTP");
   WMAddPopUpButtonItem(e->popProtocol, "CDINDEX Mode HTTP");

   WMMapSubwidgets(e->frmProtocol);

   e->frmPort = WMCreateFrame(e->frmSites);
   WMResizeWidget(e->frmPort, 280, 50);
   WMMoveWidget(e->frmPort, 10, 264);
   WMSetFrameTitle(e->frmPort, "Port");

   e->txtPort = WMCreateTextField(e->frmPort);
   WMResizeWidget(e->txtPort, 260, 20);
   WMMoveWidget(e->txtPort, 10, 20);

   WMMapSubwidgets(e->frmPort);

   e->frmAddressing = WMCreateFrame(e->frmSites);
   WMResizeWidget(e->frmAddressing, 280, 50);
   WMMoveWidget(e->frmAddressing, 10, 324);
   WMSetFrameTitle(e->frmAddressing, "Addressing");

   e->txtAddressing = WMCreateTextField(e->frmAddressing);
   WMResizeWidget(e->txtAddressing, 260, 20);
   WMMoveWidget(e->txtAddressing, 10, 20);

   WMMapSubwidgets(e->frmAddressing);

   e->btnRefresh = WMCreateCommandButton(e->frmSites);
   WMResizeWidget(e->btnRefresh, 80, 26);
   WMMoveWidget(e->btnRefresh, 120, 384);
   WMSetButtonText(e->btnRefresh, "Refresh");
   WMSetButtonAction(e->btnRefresh, (WMAction *)RefreshButton, e);

   e->btnSaveSites = WMCreateCommandButton(e->frmSites);
   WMResizeWidget(e->btnSaveSites, 80, 26);
   WMMoveWidget(e->btnSaveSites, 210, 384);
   WMSetButtonText(e->btnSaveSites, "Save");
   WMSetButtonAction(e->btnSaveSites, (WMAction *)SaveSitesButton, e);

   WMMapSubwidgets(e->frmSites);

   WMRealizeWidget(e->win);
   WMMapSubwidgets(e->win);

   changePage(e->popMenu, e);

   WMMapWidget(e->win);

   UpdateEntry(e, &data);
   UpdateSites(e);
}

static void IconClick(XEvent *event, void *clientData)
{
   if (disc_present(cd_desc, &disc) && !editing)
   {
      cdplayer *w = (cdplayer *)clientData;
      editing = 1;
      SetButtonImages(w->btnEject, ejectbtnd, ejectbtnd);
      cddb_read_serverlist(&conf, &list, &proxy);
      OpenEditor(WMWidgetScreen(w->win), w);
   }
}

void Close(WMWidget *self, cdplayer *w)
{
   close(cd_desc);
   ReleasePixmaps();
   free(w);
   exit(0);
}

static void TrackClick(XEvent *event, void *clientData)
{
   if (disc_present(cd_desc, &disc))
   {
      cdplayer *w = (cdplayer *)clientData;
      if (track_mode == 0)
      {
         track_mode = 1;
         position = 0;
         ScrollName(w);
      }
      else
      {
         track_mode = 0;
         TrackUpdate(w->lblTrack);
      }
   }
}

static void TimeClick(XEvent *event, void *clientData)
{
   if (disc_present(cd_desc, &disc))
   {
      cdplayer *w = (cdplayer *)clientData;
      if (time_mode < 3) time_mode ++;
      else time_mode = 0;
      if (disc.disc_mode == CDAUDIO_NOSTATUS) ZeroOrTotal(w);
      else TimeUpdate(w);
   }
}

void CreateDisplay(WMScreen *scr)
{
   int i;
   cdplayer *w;
   w = (cdplayer *)malloc(sizeof(cdplayer));
   if (!w)
   {
      fprintf(stderr, "Couldn't malloc enough for display.\n");
      wAbort(False);
   }

   w->win = WMCreateWindow(scr, "CDPlayer");
   WMSetWindowTitle(w->win, "CDPlayer");
   WMSetWindowMiniwindowTitle(w->win, "CDPlayer");
   WMSetWindowCloseAction(w->win, (WMAction *)Close, NULL);
   WMSetWindowMinSize(w->win, 236, 164);
   WMSetWindowMaxSize(w->win, 236, 164);
   WMResizeWidget(w->win, 236, 164);
   WMSetApplicationIconImage(scr,
      WMCreatePixmapFromXPMData(scr, appicon_xpm));

   w->lblBack = WMCreateLabel(w->win);
   WMResizeWidget(w->lblBack, 236, 164);
   WMMoveWidget(w->lblBack, 0, 0);
   WMSetLabelRelief(w->lblBack, WRFlat);
   WMSetLabelImagePosition(w->lblBack, WIPImageOnly);
   WMSetLabelImage(w->lblBack,
      WMCreatePixmapFromXPMData(scr, panelgradient_xpm));

   w->lblIcon = WMCreateLabel(w->lblBack);
   WMResizeWidget(w->lblIcon, 48, 48);
   WMMoveWidget(w->lblIcon, 7, 5);
   WMSetLabelRelief(w->lblIcon, WRFlat);
   WMSetLabelImagePosition(w->lblIcon, WIPImageOnly);
   WMSetLabelImage(w->lblIcon, WMCreatePixmapFromXPMData(scr, appicon_xpm));
   WMCreateEventHandler(WMWidgetView(w->lblIcon),
      ButtonPressMask, IconClick, w);

   for (i = 0; i != 10; i++)
   {
      w->btnTrack[i] = WMCreateCustomButton(w->lblBack, WBBPushChangeMask);
      WMResizeWidget(w->btnTrack[i], 17, 24);
      WMMoveWidget(w->btnTrack[i], 63+i*17, 5);
      WMSetButtonBordered(w->btnTrack[i], 0);
      WMSetButtonImagePosition(w->btnTrack[i], WIPImageOnly);
      SetButtonImages(w->btnTrack[i], numberbtn, numberbtnh);
      WMSetButtonAction(w->btnTrack[i], (WMAction *)Track, w);
   }
   for (i = 10; i != 20; i++)
   {
      w->btnTrack[i] = WMCreateCustomButton(w->lblBack, WBBPushChangeMask);
      WMResizeWidget(w->btnTrack[i], 17, 24);
      WMMoveWidget(w->btnTrack[i], 63+(i-10)*17, 29);
      WMSetButtonBordered(w->btnTrack[i], 0);
      WMSetButtonImagePosition(w->btnTrack[i], WIPImageOnly);
      SetButtonImages(w->btnTrack[i], numberbtn, numberbtnh);
      WMSetButtonAction(w->btnTrack[i], (WMAction *)Track, w);
   }

   w->btnPlayAll = WMCreateCustomButton(w->lblBack, WBBStateChangeMask);
   WMResizeWidget(w->btnPlayAll, 39, 25);
   WMMoveWidget(w->btnPlayAll, 3, 59);
   WMSetButtonBordered(w->btnPlayAll, 0);
   WMSetButtonImagePosition(w->btnPlayAll, WIPImageOnly);
   WMSetButtonImage(w->btnPlayAll,
      WMCreatePixmapFromXPMData(scr, modeplayall_xpm));
   WMSetButtonAltImage(w->btnPlayAll,
      WMCreatePixmapFromXPMData(scr, modeplayallh_xpm));
   WMSetButtonAction(w->btnPlayAll, (WMAction *)ModeClick, w);

   w->btnPlayShuffle = WMCreateCustomButton(w->lblBack, WBBStateChangeMask);
   WMResizeWidget(w->btnPlayShuffle, 39, 25);
   WMMoveWidget(w->btnPlayShuffle, 42, 59);
   WMSetButtonBordered(w->btnPlayShuffle, 0);
   WMSetButtonImagePosition(w->btnPlayShuffle, WIPImageOnly);
   WMSetButtonImage(w->btnPlayShuffle,
      WMCreatePixmapFromXPMData(scr, modeplayshuffle_xpm));
   WMSetButtonAltImage(w->btnPlayShuffle,
      WMCreatePixmapFromXPMData(scr, modeplayshuffleh_xpm));
   WMSetButtonAction(w->btnPlayShuffle, (WMAction *)ModeClick, w);

   WMGroupButtons(w->btnPlayAll, w->btnPlayShuffle);

   w->btnPlayTrack = WMCreateCustomButton(w->lblBack, WBBStateChangeMask);
   WMResizeWidget(w->btnPlayTrack, 39, 25);
   WMMoveWidget(w->btnPlayTrack, 81, 59);
   WMSetButtonBordered(w->btnPlayTrack, 0);
   WMSetButtonImagePosition(w->btnPlayTrack, WIPImageOnly);
   WMSetButtonImage(w->btnPlayTrack,
      WMCreatePixmapFromXPMData(scr, modeplaytrack_xpm));
   WMSetButtonAltImage(w->btnPlayTrack,
      WMCreatePixmapFromXPMData(scr, modeplaytrackh_xpm));
   WMSetButtonAction(w->btnPlayTrack, (WMAction *)ModeClick, w);

   WMGroupButtons(w->btnPlayShuffle, w->btnPlayTrack);
   WMSetButtonSelected(w->btnPlayAll, 1);

   w->btnRepeat = WMCreateCustomButton(w->lblBack, WBBStateChangeMask);
   WMResizeWidget(w->btnRepeat, 39, 25);
   WMMoveWidget(w->btnRepeat, 194, 59);
   WMSetButtonBordered(w->btnRepeat, 0);
   WMSetButtonImagePosition(w->btnRepeat, WIPImageOnly);
   WMSetButtonImage(w->btnRepeat,
      WMCreatePixmapFromXPMData(scr, moderepeat_xpm));
   WMSetButtonAltImage(w->btnRepeat,
      WMCreatePixmapFromXPMData(scr, moderepeath_xpm));

   w->lblTrack = WMCreateLabel(w->lblBack);
   WMResizeWidget(w->lblTrack, 170, 21);
   WMMoveWidget(w->lblTrack, 3, 88);
   WMSetLabelRelief(w->lblTrack, WRSunken);
   WMSetLabelTextAlignment(w->lblTrack, WACenter);
   WMSetWidgetBackgroundColor(w->lblTrack, WMDarkGrayColor(scr));
   WMSetLabelTextColor(w->lblTrack, WMWhiteColor(scr));
   WMSetLabelFont(w->lblTrack, WMBoldSystemFontOfSize(scr, 12));
   WMSetLabelText(w->lblTrack, "no disc");
   WMCreateEventHandler(WMWidgetView(w->lblTrack),
      ButtonPressMask, TrackClick, w);

   w->lblTime = WMCreateLabel(w->lblBack);
   WMResizeWidget(w->lblTime, 57, 21);
   WMMoveWidget(w->lblTime, 176, 88);
   WMSetLabelRelief(w->lblTime, WRSunken);
   WMSetLabelTextAlignment(w->lblTime, WACenter);
   WMSetWidgetBackgroundColor(w->lblTime, WMDarkGrayColor(scr));
   WMSetLabelTextColor(w->lblTime, WMWhiteColor(scr));
   WMSetLabelFont(w->lblTime, WMBoldSystemFontOfSize(scr, 12));
   WMSetLabelText(w->lblTime, "00:00");
   WMCreateEventHandler(WMWidgetView(w->lblTime),
      ButtonPressMask, TimeClick, w);

   w->btnRec = WMCreateCustomButton(w->lblBack, WBBPushChangeMask);
   WMResizeWidget(w->btnRec, 25, 25);
   WMMoveWidget(w->btnRec, 3, 114);
   WMSetButtonBordered(w->btnRec, 0);
   WMSetButtonImagePosition(w->btnRec, WIPImageOnly);
   SetButtonImages(w->btnRec, recordbtnd, recordbtnd);

   w->btnPlay = WMCreateCustomButton(w->lblBack, WBBPushChangeMask);
   WMResizeWidget(w->btnPlay, 25, 25);
   WMMoveWidget(w->btnPlay, 28, 114);
   WMSetButtonBordered(w->btnPlay, 0);
   WMSetButtonImagePosition(w->btnPlay, WIPImageOnly);
   SetButtonImages(w->btnPlay, playbtnd, playbtnd);
   WMSetButtonAction(w->btnPlay, (WMAction *)Play, w);

   w->btnPause = WMCreateCustomButton(w->lblBack, WBBPushChangeMask);
   WMResizeWidget(w->btnPause, 25, 25);
   WMMoveWidget(w->btnPause, 53, 114);
   WMSetButtonBordered(w->btnPause, 0);
   WMSetButtonImagePosition(w->btnPause, WIPImageOnly);
   SetButtonImages(w->btnPause, pausebtnd, pausebtnd);
   WMSetButtonAction(w->btnPause, (WMAction *)Pause, w);

   w->btnStop = WMCreateCustomButton(w->lblBack, WBBPushChangeMask);
   WMResizeWidget(w->btnStop, 25, 25);
   WMMoveWidget(w->btnStop, 78, 114);
   WMSetButtonBordered(w->btnStop, 0);
   WMSetButtonImagePosition(w->btnStop, WIPImageOnly);
   SetButtonImages(w->btnStop, stopbtnd, stopbtnd);
   WMSetButtonAction(w->btnStop, (WMAction *)Stop, w);

   w->btnPrev = WMCreateCustomButton(w->lblBack, WBBPushChangeMask);
   WMResizeWidget(w->btnPrev, 25, 25);
   WMMoveWidget(w->btnPrev, 118, 114);
   WMSetButtonBordered(w->btnPrev, 0);
   WMSetButtonImagePosition(w->btnPrev, WIPImageOnly);
   SetButtonImages(w->btnPrev, backbtnd, backbtnd);
   WMSetButtonAction(w->btnPrev, (WMAction *)Prev, w);

   w->btnNext = WMCreateCustomButton(w->lblBack, WBBPushChangeMask);
   WMResizeWidget(w->btnNext, 25, 25);
   WMMoveWidget(w->btnNext, 143, 114);
   WMSetButtonBordered(w->btnNext, 0);
   WMSetButtonImagePosition(w->btnNext, WIPImageOnly);
   SetButtonImages(w->btnNext, nextbtnd, nextbtnd);
   WMSetButtonAction(w->btnNext, (WMAction *)Next, w);

   w->btnEject = WMCreateCustomButton(w->lblBack, WBBPushChangeMask);
   WMResizeWidget(w->btnEject, 25, 25);
   WMMoveWidget(w->btnEject, 207, 114);
   WMSetButtonBordered(w->btnEject, 0);
   WMSetButtonImagePosition(w->btnEject, WIPImageOnly);
   SetButtonImages(w->btnEject, ejectbtnd, ejectbtnd);
   WMSetButtonAction(w->btnEject, (WMAction *)Eject, w);

   w->sldPos = WMCreateSlider(w->lblBack);
   WMResizeWidget(w->sldPos, 229, 17);
   WMMoveWidget(w->sldPos, 3, 142);
   WMSetSliderMinValue(w->sldPos, 0);
   WMSetSliderValue(w->sldPos, 0);
   WMSetSliderKnobThickness(w->sldPos, 2);
   WMSetSliderImage(w->sldPos, WMCreatePixmapFromXPMData(scr, posbar_xpm));
   WMSetSliderAction(w->sldPos, (WMAction *)Skip, w);

   WMMapSubwidgets(w->lblBack);

   WMRealizeWidget(w->win);
   WMMapSubwidgets(w->win);
   WMMapWidget(w->win);

   Update(w);
}

void ModeClick(WMWidget *self, cdplayer *w)
{
   if (WMGetButtonSelected(w->btnPlayAll)) play_mode = 0;
   else if (WMGetButtonSelected(w->btnPlayShuffle))
   {
      int i, j, k;
      play_mode = 1;
      random_track = 0;
      srand((unsigned int)time(NULL));
      for (i = 0; i != total_tracks; i++)
         play_list[i] = i+1;
      for (i = 0; i != total_tracks; i++)
      {
         j = rand()%total_tracks;
         if (j != 0)
         {
            k = play_list[0];
            play_list[0] = play_list[j];
            play_list[j] = k;
         }
      }
      if (total_tracks > 0)
         play_track = play_list[0];
   }
   else if (WMGetButtonSelected(w->btnPlayTrack)) play_mode = 2;
   if (disc_present(cd_desc, &disc)) PlayOrStop(w);
}

void Track(WMWidget *self, cdplayer *w)
{
   if (disc_present(cd_desc, &disc))
   {
      int i, track;
      track = total_tracks+1;
      for (i = 0; i != total_tracks; i++)
         if (self == w->btnTrack[i]) track = i;
      if (track < total_tracks)
      {
         play_track = track+1;
         randomTrack();
         PlayOrStop(w);
      }
   }
}

void Play(WMWidget *self, cdplayer *w)
{
   if (disc_present(cd_desc, &disc))
   {
      if (disc.disc_mode == CDAUDIO_PAUSED)
      {
         SetButtonImages(w->btnPause, pausebtn, pausebtn);
         SetButtonImages(self, playbtnh, playbtnh);
         cd_resume(cd_desc);
      }
      else
      {
         if (play_mode == 0) end_track = total_tracks;
         else end_track = play_track;
         cd_play_track(cd_desc, play_track, end_track);
      }
   }
}

void Pause(WMWidget *self, cdplayer *w)
{
   if (disc_present(cd_desc, &disc))
   {
      if (disc.disc_mode == CDAUDIO_PLAYING)
      {
         SetButtonImages(w->btnPlay, playbtn, playbtn);
         SetButtonImages(self, pausebtnh, pausebtnh);
         cd_pause(cd_desc);
      }
      else if (disc.disc_mode == CDAUDIO_PAUSED)
      {
         SetButtonImages(self, pausebtn, pausebtn);
         SetButtonImages(w->btnPlay, playbtnh, playbtnh);
         cd_resume(cd_desc);
      }
   }
}

void Stop(WMWidget *self, cdplayer *w)
{
   if (disc_present(cd_desc, &disc))
   {
      cd_stop(cd_desc);
      Reset(w);
   }
}

void Prev(WMWidget *self, cdplayer *w)
{
   if (disc_present(cd_desc, &disc) && total_tracks > 1)
   {
      if (play_mode == 1)
      {
         if (random_track == 0) random_track = total_tracks-1;
         else random_track --;
         play_track = play_list[random_track];
      }
      else
      {
         if (play_track == 1) play_track = total_tracks;
         else play_track --;
      }
      PlayOrStop(w);
   }
}

void Next(WMWidget *self, cdplayer *w)
{
   if (disc_present(cd_desc, &disc) && total_tracks > 1)
   {
      if (play_mode == 1)
      {
         if (random_track == total_tracks-1) random_track = 0;
         else random_track ++;
         play_track = play_list[random_track];
      }
      else
      {
         if (play_track == total_tracks) play_track = 1;
         else play_track ++;
      }
      PlayOrStop(w);
   }
}

void Eject(WMWidget *self, cdplayer *w)
{
   if (disc_present(cd_desc, &disc) && !editing)
   {
      int i;
      cd_stop(cd_desc);
      cd_eject(cd_desc);
      play_track = 0;
      total_tracks = 0;
      last_mode = track_mode;
      track_mode = 0;
      for (i = 0; i != 20; i++)
         SetButtonImages(w->btnTrack[i], numberbtn, numberbtnh);
      WMSetLabelText(w->lblTrack, "no disc");
      WMSetLabelText(w->lblTime, "00:00");
      SetButtonImages(w->btnPlay, playbtnd, playbtnd);
      SetButtonImages(w->btnPause, pausebtnd, pausebtnd);
      SetButtonImages(w->btnStop, stopbtnd, stopbtnd);
      SetButtonImages(w->btnPrev, backbtnd, backbtnd);
      SetButtonImages(w->btnNext, nextbtnd, nextbtnd);
      SetButtonImages(self, ejectbtnd, ejectbtnd);
      WMSetSliderValue(w->sldPos, 0);
      Update(w);
   }
}

void Skip(WMWidget *self, cdplayer *w)
{
   if (disc_present(cd_desc, &disc)) if (disc.disc_mode == CDAUDIO_PLAYING)
   {
      int pos;
      struct disc_timeval skip_time;
      pos = disc.disc_track_time.minutes*60+disc.disc_track_time.seconds;
      skip_time.minutes = 0;
      if (WMGetSliderValue(self) < pos) skip_time.seconds = -4;
      else skip_time.seconds = 4;
      cd_track_advance(cd_desc, end_track, &skip_time);
   }
}

int disc_present(int cd_desc, struct disc_info *disc)
{
   cd_stat(cd_desc, disc);
   if (disc->disc_present) return 1;
   else return 0;
}

void currentInfo(cdplayer *w)
{
   if (play_track != disc.disc_current_track && disc.disc_current_track > 0)
   {
      play_track = disc.disc_current_track;
      randomTrack();
   }
   TrackUpdate(w->lblTrack);
   TimeUpdate(w);
}

void Timer(void *clientData)
{
   WMHandlerID timer;
   cdplayer *w = (cdplayer *)clientData;
   timer = WMAddTimerHandler(1000, Timer, w);
   if (disc_present(cd_desc, &disc))
      switch (disc.disc_mode)
      {
         case CDAUDIO_PLAYING:
            currentInfo(w);
            SetButtonImages(w->btnPlay, playbtnh, playbtnh);
            SetButtonImages(w->btnPause, pausebtn, pausebtn);
            SetButtonImages(w->btnStop, stopbtn, stopbtnh);
            break;
         case CDAUDIO_PAUSED:
            currentInfo(w);
            SetButtonImages(w->btnPlay, playbtn, playbtn);
            SetButtonImages(w->btnPause, pausebtnh, pausebtnh);
            SetButtonImages(w->btnStop, stopbtn, stopbtnh);
            break;
         case CDAUDIO_COMPLETED:
            cd_stop(cd_desc);
            switch (play_mode)
            {
               case 0:
                  play_track = 1;
                  end_track = total_tracks;
                  if (WMGetButtonSelected(w->btnRepeat))
                     cd_play_track(cd_desc, play_track, end_track);
                  else Reset(w);
                  break;
               case 1:
                  if (random_track == total_tracks-1)
                  {
                     random_track = 0;
                     play_track = play_list[random_track];
                     end_track = play_track;
                     if (WMGetButtonSelected(w->btnRepeat))
                        cd_play_track(cd_desc, play_track, end_track);
                     else Reset(w);
                  }
                  else
                  {
                     random_track ++;
                     play_track = play_list[random_track];
                     end_track = play_track;
                     cd_play_track(cd_desc, play_track, end_track);
                  }
                  break;
               case 2:
                  if (WMGetButtonSelected(w->btnRepeat))
                     cd_play_track(cd_desc, play_track, end_track);
                  else Reset(w);
                  break;
            }
            break;
         case CDAUDIO_NOSTATUS:
            Reset(w);
            break;
      }
   else WMDeleteTimerHandler(timer);
}

void Update(void *clientData)
{
   WMHandlerID timer;
   cdplayer *w = (cdplayer *)clientData;
   timer = WMAddTimerHandler(1000, Update, w);
   if (disc_present(cd_desc, &disc)) if (disc.disc_total_tracks > 0)
   {
      int i;
      WMDeleteTimerHandler(timer);
      cddb_read_disc_data(cd_desc, &data);
      play_track = 1;
      total_tracks = disc.disc_total_tracks;
      for (i = 0; i != total_tracks; i++) if (i < 20)
         SetButtonImages(w->btnTrack[i], trackbtn[i], trackbtnh[i]);
      if (play_mode == 1) ModeClick(w->btnPlayShuffle, w);
      if (last_mode == 1)
      {
         track_mode = 1;
         position = 0;
         ScrollName(w);
      }
      if (total_tracks > 1)
      {
         SetButtonImages(w->btnPrev, backbtn, backbtnh);
         SetButtonImages(w->btnNext, nextbtn, nextbtnh);
      }
      SetButtonImages(w->btnEject, ejectbtn, ejectbtnh);
      Timer(w);
   }
}

int cddb_get_siteslist(struct cddb_serverlist *list, struct cddb_host *host)
{
   int sock;
   char http_string[512];
   struct cddb_hello hello;
   strcpy(hello.hello_program, "cdplayer.app");
   strcpy(hello.hello_version, PROGRAM_VERSION);
   if (host->host_protocol == CDDB_MODE_CDDBP)
   {
      if ((sock = cddb_connect_server(host, NULL, &hello)) < 0) return -1;
      if (cddb_sites(sock, CDDB_MODE_CDDBP, list) < 0) return -1;
   }
   else if (host->host_protocol == CDDB_MODE_HTTP)
   {
      if ((sock = cddb_connect_server(host, NULL, &hello,
         http_string, 512)) < 0) return -1;
      if (cddb_sites(sock, CDDB_MODE_HTTP, list, http_string) < 0)
         return -1;
   }
   return 0;
}
