/*
 * MAIN.C source file for extace
 * 
 /GDK/GNOME sound (esd) system output display program
 * 
 * Copyright (C) 1998 by Michael Fullbright
 * Re-hacked to look good by The Rasterman
 * 
 * Hacked some more by Dave J. Andruczyk <dave@techdev.buffalostate.edu>
 * to be fully scalable with lots of new options, for tilting the axis's
 * and various other cool stuff.
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include <config.h>
#include <globals.h>
#include <protos.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <time.h>
#include <math.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <gdk_imlib.h>


// See globals.h for variable declarations and DEFINES

void draw(void)
{
    gint i,j,k;
    static double plevels[MAXBANDS];
    static gint lin_x_axis[MAXBANDS];
    static gint log_x_axis[MAXBANDS];
    double val;
    int sum;
    int count;
    GdkPoint pt[4];
    GdkColor cl;
    gint br,lvl;
    gint lwidth = width/bands;

    if (!GetFFT()) return;

    if (!win) return;

    // Gives a "log-like" x axis (for 2D (EQ like analyzer))
    // hacking to make it work better with the various displays
    // the bin[i] deteremines how much data is summed together for the 
    // display since the display can't fit a 1024 point fft nicely.
    // 
    for(i=0;i<bands;i++)
    {
	// Linear x axis
	lin_x_axis[i]=(NSAMP/2)/bands; 
    }
    if (scalefixed == 0)
    {
	scalefactor = 10.0;		//reset it

	count = 0;
recalc:
	count++;
	scale = bands/scalefactor;
	sum = 0;
	for (i=0; i < bands ; i++)
	{
	    log_x_axis[i] = exp((i/scale));
	    sum = sum + log_x_axis[i];
	}
	    // reduction algorithm, to get scalefactor so that ALL datapoints
	    // in the FFT get onto the display
	if (sum > NSAMP/2)
	{
	    if (sum > (2.0*(NSAMP/2)))
		scalefactor = scalefactor - .80;
	    if (sum > (1.25*(NSAMP/2)))
		scalefactor = scalefactor - .150;
	    if (sum > (1.15*(NSAMP/2)))
		scalefactor = scalefactor - .025;
	    if (sum > (1.05*(NSAMP/2)))
		scalefactor = scalefactor - .009;
	    if (sum < (1.05*(NSAMP/2)))
		scalefactor = scalefactor - .001;
	    goto recalc;
	}
#ifdef SCALEDEBUG
	g_print("It took %i iterations to get the scalefactor\n",count);
	g_print("Number of points in display is %i\n",sum);
#endif
    }
    scalefixed = 1;

    for (i=1; i<NSAMP/2; i++)
	auval[i-1] = (aufft[i]/20)*(double)sensitivity;
    auval[(NSAMP/2)-1]=0;


    // this function groups together the data from the FFT so that it can
    // fit into the smaller division displayed on screen, since a 1024 point
    // fft doesn't fit wwell into 32 or 64 divisions.  the lin/log_x_axis
    // arrays determine how the fft data is broken up for a linear or log 
    // based frequency axis

    j=0;
    k=0;
    for(i=0;i<bands;i++)
    {
	val=0;
	// sum it then average it so the scaling doesn't go haywire
	for(j=0;j<(log_x_axis[i]);j++)
	{val+=auval[k++];}
	val/=(double)log_x_axis[i];
	if (slide)
	{
	    if (val<=(levels[i]-4))
	    {
		levels[i]-=decay_val;
		if (levels[i]<0)
		    levels[i]=0;
	    }
	    else 
		levels[i]=val;
	}
	else
	    levels[i]=val;
    }

    switch (mode)
    {
	case 0:
	    gdk_window_copy_area(win,gc,0,0,win,x3d_scroll,z3d_scroll,width-x3d_scroll,height-z3d_scroll);
	    gdk_draw_rectangle( win,
				disp->style->black_gc,
				TRUE, width-x3d_scroll,0,
				x3d_scroll,height);
	    gdk_draw_rectangle( win,
				disp->style->black_gc,
				TRUE, 0,height-z3d_scroll,
				width,z3d_scroll);
	    // pixel shift for scalability

	    // need to recalculate these, for borders so we don't get "black
	    // spots" on the display from running offscreen
	    // x_draw_width = width - "some magic amount"

	    // Smoothing routine.  Adjusts border slightly to avoid 2 pixel
	    // breakups in the display... (woohoo, it actually works!!)

	    // pix_per_block = floating point!!!


	    x_draw_width = width - abs(x3d_scroll)-2*x_border;
	    y_draw_height = height - abs(z3d_scroll)-2*y_border;

	    x_shift = ((x_draw_width*(1-x3d_start))-(x_draw_width*(1-x3d_end)));
	    x_shift_per_block = x_shift/bands;

	    x_offset = (width-x_draw_width)/2;
	    y_offset = (height-y_draw_height)/2;
	    if(x3d_start < x3d_end)
		x_fudge = 1;
	    else x_fudge = -1;

	    y_shift_per_block = z3d_scroll/2;

	    for( i=0; i < bands; i++) 
	    {   
		pt[0].x=width-((i*x_draw_width)*(1-x3d_start))/(bands)-(((bands-i)*x_draw_width)*(1-x3d_end))/(bands)-(x3d_scroll/2)-x_offset+x_fudge-(x3d_scroll%2);
		pt[0].y=height-(((i)*y_draw_height*y3d_start)/(bands))-(((bands-i)*y_draw_height*y3d_end)/(bands))-(gint)plevels[i]-(2*y_shift_per_block)-y_offset-(z3d_scroll%2);
		pt[1].x=width-((i*x_draw_width)*(1-x3d_start))/(bands)-(((bands-i)*x_draw_width)*(1-x3d_end))/(bands)+(x3d_scroll/2)-x_offset+x_fudge;
		pt[1].y=height-(((i)*y_draw_height*y3d_start)/(bands))-(((bands-i)*y_draw_height*y3d_end)/(bands))-(gint)levels[i]-y_offset; 
		pt[2].x=width-((i*x_draw_width)*(1-x3d_start))/(bands)-(((bands-i)*x_draw_width)*(1-x3d_end))/(bands)-(x_shift_per_block)+(x3d_scroll/2)-x_offset;
		if(i == (bands-1))
		    pt[2].y=height-(((i+1)*y_draw_height*y3d_start)/(bands))-(((bands-i-1)*y_draw_height*y3d_end)/(bands))-y_offset;
		else
		    pt[2].y=height-(((i+1)*y_draw_height*y3d_start)/(bands))-(((bands-i-1)*y_draw_height*y3d_end)/(bands))-(gint)levels[i+1]-y_offset; 
		pt[3].x=width-((i*x_draw_width)*(1-x3d_start))/(bands)-(((bands-i)*x_draw_width)*(1-x3d_end))/(bands)-(x_shift_per_block)-(x3d_scroll/2)-x_offset-(x3d_scroll%2);
		if(i == (bands-1))
		    pt[3].y=height-(((i+1)*y_draw_height*y3d_start)/(bands))-(((bands-i-1)*y_draw_height*y3d_end)/(bands))-(2*y_shift_per_block)-y_offset-(z3d_scroll%2);
		else
		    pt[3].y=height-(((i+1)*y_draw_height*y3d_start)/(bands))-(((bands-i-1)*y_draw_height*y3d_end)/(bands))-(gint)plevels[i+1]-(2*y_shift_per_block)-y_offset-(z3d_scroll%2);
		lvl=(gint)levels[i]/2;
		if (lvl>63) lvl=63;
		br=16+(gint)(levels[i]-plevels[i+1]);

		if (br<0) br=0;
		else if (br>31) br=31;
		cl.pixel=colortab[br][lvl];
		gdk_gc_set_foreground(gc,&cl);
		gdk_draw_polygon(win,gc,TRUE,pt,4);
		gdk_draw_line(win,disp->style->black_gc,
			      pt[0].x,pt[0].y,pt[1].x,pt[1].y);
		gdk_draw_line(win,disp->style->black_gc,
			      pt[0].x,pt[0].y,pt[3].x,pt[3].y);

		if(show_leader)
		{
		    // uses previous points instead of calculating twice...
		    pt[0].x=pt[1].x; //-z3d_scroll/2;
		    pt[0].y=pt[1].y+(gint)levels[i];
		    pt[1].x=pt[0].x; //+z3d_scroll/2;
		    pt[1].y=pt[1].y;
		    pt[2].x=pt[2].x; //-z3d_scroll/2;
		    pt[2].y=pt[2].y;
		    pt[3].x=pt[2].x; //+z3d_scroll/2;
		    if(i == (bands - 1))
			pt[3].y=pt[2].y;
		    else
			pt[3].y=pt[2].y+levels[i+1];

		    gdk_draw_polygon(win,gc,TRUE,pt,4);
		    gdk_draw_line(win,disp->style->black_gc,
				  pt[2].x,pt[2].y,pt[1].x,pt[1].y);
		}
	    }
	    break;
	case 1:
	    maxlevel = prevlevel = 0;

	    for (i=0;i < bands; i++)
	    {
		if (plevels[i] > prevlevel)
		    prevlevel = (gint)plevels[i];
		if (levels[i] > maxlevel)
		    maxlevel = (gint)levels[i];
	    }
	    if (prevlevel > maxlevel)
		maxlevel = prevlevel;

	    maxlevel = ((maxlevel*height)/128);

	    gdk_draw_rectangle( pixmap,
				disp->style->black_gc,
				TRUE, 0,height-maxlevel,
				width,maxlevel);

	    for( i=0; i < bands; i++ ) 
	    {   
		lvl=(gint)levels[i]/2;
		if (lvl>63) 
		    lvl=63;

		br=16+(gint)(levels[i]-plevels[i+1]);

		if (br<0) br=0;
		else if (br>31) br=31;
		//cl.pixel=colortab[br][lvl];
		cl.pixel=colortab[16][i];
		gdk_gc_set_foreground(gc,&cl);
		bar_start = height - (((gint)levels[i]*height)/128);
		gdk_draw_rectangle( pixmap,gc,
				    TRUE, 
				    (i*lwidth)+1,bar_start,
				    lwidth-2,(((gint)levels[i]*height)/128));
	    }
	    for(i=0;i<maxlevel;i+=seg_height)
	    {
		gdk_draw_rectangle( pixmap,disp->style->black_gc,
				    TRUE, 
				    0,height-i,
				    width,seg_space);
	    }

	    gdk_window_clear_area(win,0,height-maxlevel,width,maxlevel);
	    break;
	case 2:
	    top = height/2 -128;
	    bottom = top + 256;
	    lo_width = (512 < width) ? 512 : width;
	    	    gdk_draw_rectangle( pixmap,
	    				disp->style->black_gc,
	    				TRUE, 0,top,
	    				lo_width,bottom);
	    for(i=0;i<NSAMP;i+=2)
	    {
		j=(gint)(audata_backup[i]*127)+127;
		if (j<0) j=0;
		else if (j>255) j=255;
		if (j<127)
		    gdk_draw_pixmap(pixmap,gc,grad[j],0,0,
				    i/2,((height/2)-127)+j,1,127-j);
		else if (j>127)
		    gdk_draw_pixmap(pixmap,gc,grad[j],0,0,
				    i/2,height/2,1,j-127);
	    }
	    gdk_window_clear_area(win,0,top,lo_width,bottom);
	    break;
	case 3:
	    // "det" == detailed
	    gdk_window_copy_area(win,gc,0,0,win,xdet_scroll,zdet_scroll,width-xdet_scroll,height-zdet_scroll);
	    gdk_draw_rectangle( win,
				disp->style->black_gc,
				TRUE, width-xdet_scroll,0,
				xdet_scroll,height);
	    gdk_draw_rectangle( win,
				disp->style->black_gc,
				TRUE, 0,height-zdet_scroll,
				width,zdet_scroll);

	    x_draw_width = width - abs(xdet_scroll)-2*x_border;
	    y_draw_height = height - abs(zdet_scroll)-2*y_border;

	    x_offset = (width-x_draw_width)/2;
	    y_offset = (height-y_draw_height)/2;

	    for( i=0; i < NSAMP/2 ; i++ ) 
	    {   
		pt[0].x=width-(((i*x_draw_width)*(1-xdet_start))/(NSAMP/2))-(((((NSAMP/2)-i)*x_draw_width)*(1-xdet_end))/(NSAMP/2))-x_offset;
		pt[0].y=height-((i*y_draw_height*ydet_start)/(NSAMP/2))-((((NSAMP/2)-i)*y_draw_height*ydet_end)/(NSAMP/2))-y_offset;
		pt[1].x=width-(((i*x_draw_width)*(1-xdet_start))/(NSAMP/2))-(((((NSAMP/2)-i)*x_draw_width)*(1-xdet_end))/(NSAMP/2))-x_offset;
		pt[1].y=height-((i*y_draw_height*ydet_start)/(NSAMP/2))-((((NSAMP/2)-i)*y_draw_height*ydet_end)/(NSAMP/2))-(gint)auval[i]-y_offset;
		lvl=(gint)auval[i];
		if (lvl>63) 
		    lvl=63;

		cl.pixel=colortab[10][lvl];
		gdk_gc_set_foreground(gc,&cl);
		gdk_draw_line(win,gc,pt[0].x,pt[0].y,pt[1].x,pt[1].y);
	    }
	    break;
	default:
	    break;
    }
    gdk_flush();
    // shift current levels into the previous levels array,
    for( i=0; i < bands; i++ ) 
	plevels[i]=levels[i];

}

