/*
 * main.c source file for extace
 * 
 * /GDK/GNOME sound (esd) system output display program
 * 
 * Copyright (C) 1998 by Michael Fullbright
 * Re-hacked to look good by The Rasterman
 * 
 * Hacked some more by Dave J. Andruczyk <dave@techdev.buffalostate.edu>
 * to be fully scalable with lots of new options, for tilting the axis's
 * and various other cool stuff.
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include <config.h>
#include <globals.h>
#include <protos.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <dirent.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <esd.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <gdk_imlib.h>

// See globals.h for variable declarations and DEFINES

// External Prototypes

void kt_stars_stop(GtkWidget *s);
void kt_stars_start(GtkWidget *s, gint width, gint height);
GtkWidget * kt_stars_new(GtkWidget *area, GdkPixmap *pixmap);
void kt_stars_set_logo_pixmp(GtkWidget *s, GdkPixmap *pmap, GdkPixmap *mask);

// End External Prototypes

int main(int argc, char **argv)
{
    GtkWidget *vbox;
    GtkWidget *hbox;
    GtkWidget *window;
    GtkWidget *button;
    GtkWidget *sep;
    GtkWidget *label;
    GtkWidget *table;
    GtkWidget *scale;
    GtkObject *adj1;

    init();		// initialize all global variables
    make_extace_dir();
    read_config();
    gtk_init(&argc, &argv);
    gdk_imlib_init();
    gtk_widget_push_visual(gdk_imlib_get_visual());
    gtk_widget_push_colormap(gdk_imlib_get_colormap());

    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    main_win_ptr = window;
    gtk_window_set_title(GTK_WINDOW(window),"ExtacE");
    gtk_container_border_width(GTK_CONTAINER(window),0);
    gtk_signal_connect(GTK_OBJECT(window),"delete_event",
		       GTK_SIGNAL_FUNC(leave),NULL);
    gtk_signal_connect (GTK_OBJECT (window), "button_press_event",
			(GtkSignalFunc) button_notify_event, NULL);
    gtk_signal_connect (GTK_OBJECT (window), "button_release_event",
			(GtkSignalFunc) button_notify_event, NULL);

    gtk_signal_connect (GTK_OBJECT (window), "motion_notify_event",
			(GtkSignalFunc) motion_notify_event, NULL);

    gtk_widget_set_events (window, GDK_EXPOSURE_MASK
			   | GDK_BUTTON_PRESS_MASK
			   | GDK_BUTTON_RELEASE_MASK
			   | GDK_POINTER_MOTION_HINT_MASK
			   | GDK_POINTER_MOTION_MASK);
    gtk_window_set_policy(GTK_WINDOW(window), TRUE,	// allow shrink
			  TRUE, 	// allow grow
			  FALSE);	// auto shrink

    gtk_widget_show(window);

    init_gc(window);

    vbox=gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(window), vbox);
    gtk_widget_show(vbox);
    hbox=gtk_hbox_new(TRUE,0);
    gtk_widget_show(hbox);

    button=gtk_button_new_with_label("3D");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button),"clicked",
		       GTK_SIGNAL_FUNC(button_3d_fft),NULL);

    button=gtk_button_new_with_label("Flat");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button),"clicked",
		       GTK_SIGNAL_FUNC(button_2d_fft),NULL);

    button=gtk_button_new_with_label("Oscill");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button),"clicked",
		       GTK_SIGNAL_FUNC(button_oscilloscope),NULL);

    button=gtk_button_new_with_label("Spikes");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button),"clicked",
		       GTK_SIGNAL_FUNC(button_3d_detailed),NULL);

    button=gtk_button_new_with_label("About");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button),"clicked",
		       GTK_SIGNAL_FUNC(button_about),NULL);

    optionsbut=gtk_toggle_button_new_with_label("Options");
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(optionsbut), FALSE);
    gtk_box_pack_start(GTK_BOX(hbox),optionsbut,TRUE,TRUE,0);
    gtk_widget_show(optionsbut);
    gtk_signal_connect(GTK_OBJECT(optionsbut),"toggled",
		       GTK_SIGNAL_FUNC(button_options),NULL);

    button=gtk_button_new_with_label("Close");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button),"clicked",
		       GTK_SIGNAL_FUNC(leave),NULL);

    gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,0);

    disp=gtk_drawing_area_new();
    gtk_widget_show(disp);
    gtk_signal_connect( GTK_OBJECT(disp),"configure_event",
			(GtkSignalFunc)disp_configure, NULL);
    gtk_box_pack_start(GTK_BOX(vbox),disp,TRUE,TRUE,0);
    gtk_widget_realize(disp);
    gtk_widget_set_usize(disp,width,height);
    win=disp->window;
    gc=gdk_gc_new(win);
    gdk_gc_copy(gc,disp->style->white_gc);
    pixmap=gdk_pixmap_new(win,width,height,
			  gtk_widget_get_visual(disp)->depth);
    gdk_draw_rectangle( pixmap,
			disp->style->black_gc,
			TRUE, 0,0,
			width,height);
    gdk_window_set_back_pixmap(win,pixmap,0);

    // options window
    options = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(options),"Extace Options");
    gtk_container_border_width(GTK_CONTAINER(options),2);
    gtk_signal_connect(GTK_OBJECT(options),"delete_event",
		       GTK_SIGNAL_FUNC(close_options),optionsbut);

    vbox = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(options), vbox);

    label = gtk_label_new("Lag factor");
    gtk_box_pack_start(GTK_BOX(vbox),label,TRUE,TRUE,0);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new(7.0,0.0,7.0,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
    GTK_ADJUSTMENT (adj1)->value = lag;
    gtk_box_pack_start(GTK_BOX(vbox),scale,TRUE,TRUE,0);
    gtk_range_set_update_policy(GTK_RANGE (scale),
				GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
			GTK_SIGNAL_FUNC (slider_changed), (gpointer)LAG);
    gtk_widget_show(scale);

    label = gtk_label_new("Number of points");
    gtk_box_pack_start(GTK_BOX(vbox),label,TRUE,TRUE,0);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new(32.0,16.0,(float)MAXBANDS+1,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
    GTK_ADJUSTMENT (adj1)->value = bands;
    gtk_box_pack_start(GTK_BOX(vbox),scale,TRUE,TRUE,0);
    gtk_range_set_update_policy(GTK_RANGE (scale),
				GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
			GTK_SIGNAL_FUNC (slider_changed), (gpointer)BANDS);
    gtk_widget_show(scale);

    label = gtk_label_new("Sensitivity");
    gtk_box_pack_start(GTK_BOX(vbox),label,TRUE,TRUE,0);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new(200.0,0.0,800.0,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
    GTK_ADJUSTMENT (adj1)->value = sensitivity;
    gtk_box_pack_start(GTK_BOX(vbox),scale,TRUE,TRUE,0);
    gtk_range_set_update_policy(GTK_RANGE (scale),
				GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
			GTK_SIGNAL_FUNC (slider_changed), (gpointer)SENSE);
    gtk_widget_show(scale);

    table = gtk_table_new(16,2,FALSE);
    gtk_box_pack_start(GTK_BOX(vbox),table,TRUE,TRUE,0);

    sep = gtk_hseparator_new();
    gtk_table_attach_defaults(GTK_TABLE(table),sep,0,2,1,2);
    gtk_widget_show(sep);

    label = gtk_label_new("Hi-Res Options");
    gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,0,1);
    gtk_widget_show(label);

    label = gtk_label_new("Hi-res X-Axis Start Pt. (% width)");
    gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,2,3);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new(66.0,0.0,101.0,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
    GTK_ADJUSTMENT (adj1)->value = xdet_start*100;
    xdet_start_ptr = adj1;
    gtk_table_attach_defaults(GTK_TABLE(table),scale,1,2,3,4);
    gtk_range_set_update_policy(GTK_RANGE (scale),
				GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
			GTK_SIGNAL_FUNC (slider_changed), (gpointer)SPIKE_X_START);
    gtk_widget_show(scale);

    label = gtk_label_new("Hi-res X-Axis End Pt. (% width)");
    gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,4,5);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new(66.0,0.0,101.0,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
    GTK_ADJUSTMENT (adj1)->value = xdet_end*100;
    xdet_end_ptr = adj1;
    gtk_table_attach_defaults(GTK_TABLE(table),scale,1,2,5,6);
    gtk_range_set_update_policy(GTK_RANGE (scale),
				GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
			GTK_SIGNAL_FUNC (slider_changed), (gpointer)SPIKE_X_END);
    gtk_widget_show(scale);

    label = gtk_label_new("Hi-res Y-Axis Start Pt. (% height)");
    gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,6,7);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new(66.0,0.0,101.0,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
    GTK_ADJUSTMENT (adj1)->value = ydet_start*100;
    ydet_start_ptr = adj1;
    gtk_table_attach_defaults(GTK_TABLE(table),scale,1,2,7,8);
    gtk_range_set_update_policy(GTK_RANGE (scale),
				GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
			GTK_SIGNAL_FUNC (slider_changed), (gpointer)SPIKE_Y_START);
    gtk_widget_show(scale);

    label = gtk_label_new("Hi-res Y-Axis End Pt. (% height)");
    gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,8,9);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new(66.0,0.0,101.0,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
    GTK_ADJUSTMENT (adj1)->value = ydet_end*100;
    ydet_end_ptr = adj1;
    gtk_table_attach_defaults(GTK_TABLE(table),scale,1,2,9,10);
    gtk_range_set_update_policy(GTK_RANGE (scale),
				GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
			GTK_SIGNAL_FUNC (slider_changed), (gpointer)SPIKE_Y_END);
    gtk_widget_show(scale);

    label = gtk_label_new("Spikes X-Axis Scroll Speed");
    gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,10,11);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new(1.0,-20.0,20.0,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
    GTK_ADJUSTMENT (adj1)->value = xdet_scroll;
    xdet_scroll_ptr = adj1;
    gtk_table_attach_defaults(GTK_TABLE(table),scale,1,2,11,12);
    gtk_range_set_update_policy(GTK_RANGE (scale),
				GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
			GTK_SIGNAL_FUNC (slider_changed), (gpointer)SPIKE_X_SCROLL);
    gtk_widget_show(scale);

    label = gtk_label_new("Spikes Z-Axis Scroll Speed");
    gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,12,13);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new(0.0,-20.0,20.0,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
    GTK_ADJUSTMENT (adj1)->value = zdet_scroll;
    zdet_scroll_ptr = adj1;
    gtk_table_attach_defaults(GTK_TABLE(table),scale,1,2,13,14);
    gtk_range_set_update_policy(GTK_RANGE (scale),
				GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
			GTK_SIGNAL_FUNC (slider_changed), (gpointer)SPIKE_Z_SCROLL);
    gtk_widget_show(scale);

    label = gtk_label_new("3D Landscape Options");
    gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,0,1);
    gtk_widget_show(label);

    label = gtk_label_new("3D X-Axis Start Pt. (% width)");
    gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,2,3);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new(66.0,0.0,101.0,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
    GTK_ADJUSTMENT (adj1)->value = x3d_start*100;
    x3d_start_ptr = adj1;
    gtk_table_attach_defaults(GTK_TABLE(table),scale,0,1,3,4);
    gtk_range_set_update_policy(GTK_RANGE (scale),
				GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
			GTK_SIGNAL_FUNC (slider_changed), (gpointer)X3D_START);
    gtk_widget_show(scale);

    label = gtk_label_new("3D X-Axis End Pt. (% width)");
    gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,4,5);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new(66.0,0.0,101.0,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
    GTK_ADJUSTMENT (adj1)->value = x3d_end*100;
    x3d_end_ptr = adj1;
    gtk_table_attach_defaults(GTK_TABLE(table),scale,0,1,5,6);
    gtk_range_set_update_policy(GTK_RANGE (scale),
				GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
			GTK_SIGNAL_FUNC (slider_changed), (gpointer)X3D_END);
    gtk_widget_show(scale);

    label = gtk_label_new("3D Y-Axis Start Pt. (% height)");
    gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,6,7);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new(66.0,0.0,101.0,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
    GTK_ADJUSTMENT (adj1)->value = y3d_start*100;
    y3d_start_ptr = adj1;
    gtk_table_attach_defaults(GTK_TABLE(table),scale,0,1,7,8);
    gtk_range_set_update_policy(GTK_RANGE (scale),
				GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
			GTK_SIGNAL_FUNC (slider_changed), (gpointer)Y3D_START);
    gtk_widget_show(scale);

    label = gtk_label_new("3D Y-Axis End Pt. (% height)");
    gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,8,9);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new(66.0,0.0,101.0,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
    GTK_ADJUSTMENT (adj1)->value = y3d_end*100;
    y3d_end_ptr = adj1;
    gtk_table_attach_defaults(GTK_TABLE(table),scale,0,1,9,10);
    gtk_range_set_update_policy(GTK_RANGE (scale),
				GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
			GTK_SIGNAL_FUNC (slider_changed), (gpointer)Y3D_END);
    gtk_widget_show(scale);

    label = gtk_label_new("3D X-Axis Scroll Speed");
    gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,10,11);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new(0.0,-50.0,50.0,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
    GTK_ADJUSTMENT (adj1)->value = x3d_scroll;
    x3d_scroll_ptr = adj1;
    gtk_table_attach_defaults(GTK_TABLE(table),scale,0,1,11,12);
    gtk_range_set_update_policy(GTK_RANGE (scale),
				GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
			GTK_SIGNAL_FUNC (slider_changed), (gpointer)X3D_SCROLL);
    gtk_widget_show(scale);

    label = gtk_label_new("3D Z-Axis Scroll Speed");
    gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,12,13);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new(8.0,-50.0,50.0,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
    GTK_ADJUSTMENT (adj1)->value = z3d_scroll;
    z3d_scroll_ptr = adj1;
    gtk_table_attach_defaults(GTK_TABLE(table),scale,0,1,13,14);
    gtk_range_set_update_policy(GTK_RANGE (scale),
				GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
			GTK_SIGNAL_FUNC (slider_changed), (gpointer)Z3D_SCROLL);
    gtk_widget_show(scale);

    gtk_widget_show(table);

    label = gtk_label_new("Decay Amount");
    gtk_box_pack_start(GTK_BOX(vbox),label,TRUE,TRUE,0);
    gtk_widget_show(label);

    adj1 = gtk_adjustment_new(1.0,1.0,11.0,1.0,1.0,1.0);
    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj1));
    gtk_scale_set_digits(GTK_SCALE(scale),0);
    GTK_ADJUSTMENT (adj1)->value = decay_val;
    gtk_box_pack_start(GTK_BOX(vbox),scale,TRUE,TRUE,0);
    gtk_range_set_update_policy(GTK_RANGE (scale),
				GTK_UPDATE_CONTINUOUS);
    gtk_signal_connect (GTK_OBJECT (adj1), "value_changed",
			GTK_SIGNAL_FUNC (slider_changed), (gpointer)DECAY);
    gtk_widget_show(scale);

    hbox = gtk_hbox_new(TRUE,0);
    gtk_box_pack_start(GTK_BOX(vbox),hbox,TRUE,FALSE,0);

    button = gtk_toggle_button_new_with_label("Hide Leading Edge");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
			GTK_SIGNAL_FUNC (button_handle), (gpointer)SUPPRESS);
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), !show_leader);
    gtk_widget_show(button);

    button = gtk_toggle_button_new_with_label("Enable Decay");
    gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
    gtk_signal_connect (GTK_OBJECT (button), "toggled",
			GTK_SIGNAL_FUNC (button_handle), (gpointer)DECAY);
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), slide);
    gtk_widget_show(button);

    gtk_widget_show(hbox);
    gtk_widget_show(vbox);

    dir_win = gtk_window_new(GTK_WINDOW_DIALOG);
    gtk_window_set_title(GTK_WINDOW(dir_win),"Trace direction");
    gtk_container_border_width(GTK_CONTAINER(dir_win),2);
    gtk_widget_set_usize(dir_win,150,150);
    gtk_signal_connect(GTK_OBJECT(dir_win),"delete_event",
		       GTK_SIGNAL_FUNC(close_dir_win),NULL);

    gtk_signal_connect (GTK_OBJECT (dir_win), "motion_notify_event",
			(GtkSignalFunc) dir_motion, NULL);

    gtk_widget_set_events (dir_win, GDK_EXPOSURE_MASK
			   | GDK_BUTTON_PRESS_MASK
			   | GDK_BUTTON_RELEASE_MASK
			   | GDK_POINTER_MOTION_HINT_MASK
			   | GDK_POINTER_MOTION_MASK);

    dir_area = gtk_drawing_area_new();
    gtk_drawing_area_size (GTK_DRAWING_AREA (dir_area), 
			   dir_win->allocation.width, 
			   dir_win->allocation.height);
    gtk_signal_connect( GTK_OBJECT(dir_area),"configure_event",
			(GtkSignalFunc)dir_configure, NULL);
    gtk_signal_connect( GTK_OBJECT(dir_area),"configure_event",
			(GtkSignalFunc)update_pointer, NULL);
    gtk_signal_connect( GTK_OBJECT(dir_area),"expose_event",
			(GtkSignalFunc)dir_expose, NULL);

    gtk_container_add(GTK_CONTAINER(dir_win), dir_area);

    gtk_widget_show(dir_area);
    if (dir_win_present)
    {
	gtk_widget_show(dir_win);
	update_pointer();
    }

    gtk_window_set_policy(GTK_WINDOW(dir_win), TRUE,	// allow shrink
			  TRUE, 	// allow grow
			  FALSE);	// auto shrink

    gtk_window_set_policy(GTK_WINDOW(options), FALSE,	// allow shrink
			  TRUE, 	// allow grow
			  FALSE);	// auto shrink

    setup_datawindow();
    init_colortab();
    open_sound();
    fcntl(sound, F_SETFL, O_NONBLOCK);
    gdk_input_add(sound, GDK_INPUT_READ | GDK_INPUT_EXCEPTION,
		  handle_read, NULL);
    gtk_main();
    return 0;
}
