/*
 * MISC.C extace source file
 * 
 /GDK/GNOME sound (esd) system output display program
 * 
 * Copyright (C) 1999 by Dave J. Andruczyk 
 * 
 * Based upon Code written by The Rasterman and DrMike from redhat.com
 * Re-hacked to look good by The Rasterman (Michael Fullbright)
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include <config.h>
#include <globals.h>
#include <protos.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <dirent.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <gdk_imlib.h>
#include <esd.h>
#ifdef HAVE_GSL_GSL_ERRNO_H
#include <gsl/gsl_errno.h>
#endif
#ifdef HAVE_GSL_ERRNO_H
#include <gsl_errno.h>
#endif 
#ifdef HAVE_GSL_GSL_FFT_REAL_H
#include <gsl/gsl_fft_real.h>
#endif
#ifdef HAVE_GSL_FFT_REAL_H
#include <gsl_fft_real.h>
#endif
#ifdef HAVE_LIBFFTW
#include <rfftw.h>
#endif 


// Globals
#ifdef HAVE_LIBFFTW
fftw_plan plan;                 // fft plan for fftw library
#endif


void open_sound(void)
{
    sound=esd_monitor_stream(ESD_BITS16|ESD_MONO|ESD_STREAM|ESD_PLAY,RATE,NULL,"extace");
    if (sound<0)
    {
	g_error("Cannot connect to EsounD\n");
	exit(1);
    }
#ifdef HAVE_LIBFFTW
    plan = rfftw_create_plan(NSAMP, FFTW_FORWARD, FFTW_ESTIMATE);
#endif
}

void setup_datawindow(void)
{
    gint i;
    double *w;
    double sumw;

    w = g_malloc(NSAMP*sizeof(double));

    /* add in i=N for window function first */
    sumw = 1.0 - fabs((NSAMP - 0.5*(NSAMP-1.0))/(0.5*(NSAMP+1.0)));
    sumw *= sumw;

#ifdef WINDOW_DEBUG
    f = fopen("/tmp/window.in", "w");
#endif
    for (i=0; i<NSAMP; i++) {
	w[i] = 1.0 - fabs((i - 0.5*(NSAMP-1.0))/(0.5*(NSAMP+1.0)));
	sumw += w[i]*w[i];
#ifdef WINDOW_DEBUG
	fprintf(f, "%d %f %f\n", i, w[i], sumw);
#endif
    }
#ifdef WINDOW_DEBUG
    fclose(f);
#endif

    sumw *= NSAMP;
#ifdef WINDOW_DEBUG
    fprintf(stderr, "window sum is %f\n",sumw);
#endif
    datawindowsum = sumw;
    datawindow    = w;
}
int GetFFT(void)
{
    gint buf;
    short *ptr;
    double *w,*aup,*aup2,*fftp,*aupend;

    buf=((BUFS*2)+curbuf-lag)%BUFS;

    if ((curbuf%2)>0) return 0; 
    if (mode!=2)
    {
	ptr=aubuf[buf];
	w=datawindow;
	aup=audata;
	aupend=aup+NSAMP;
	while(aup<aupend)
	    *aup++=((*w++)*(double)(*ptr++))/32768.0;
    }
    ptr=aubuf[buf];
    aup=audata_backup;
    aupend=aup+NSAMP;
    while(aup<aupend)
	*aup++=((double)(*ptr++))/32768.0;

    if (mode==2) return 1;
#ifdef HAVE_LIBFFTW
#ifdef DEBUG
    g_print("Using FFTW routine\n");
#endif
    rfftw_one(plan, audata, auout);
    aufft[0] = (auout[0]*auout[0])/(NSAMP*NSAMP);
    aufft[NSAMP/2] = (auout[NSAMP/2]*auout[NSAMP/2])/(NSAMP*NSAMP);
    /* take the log */
    aup=auout;
    aup2=auout+NSAMP-1;

#endif
#ifdef HAVE_LIBGSLFFT
#ifdef DEBUG
    g_print("Using GSL fft routine\n");
#endif
    gsl_fft_real_radix2(audata, NSAMP);

    aufft[0] = (audata[0]*audata[0])/(NSAMP*NSAMP);
    aufft[NSAMP/2] = (audata[NSAMP/2]*audata[NSAMP/2])/(NSAMP*NSAMP);
    /* take the log */
    aup=audata;
    aup2=audata+NSAMP-1;
#endif

    aupend=aup+NSAMP;
    fftp=aufft;
    while(aup<aupend)
    {
	*fftp=8+log10((((*aup * *aup)+(*aup2 * *aup2)))/(NSAMP*NSAMP));
	if (*fftp<0) *fftp=0;
	fftp++;
	aup++;aup2--;
    }
    return 1;
}

gint disp_configure(GtkWidget *widget, GdkEventConfigure *event)
{
    if (win)
    {
	if ((width!=event->width)||(height!=event->height))
	{
	    width=event->width;
	    height=event->height;
	    if (pixmap)
		gdk_pixmap_unref(pixmap);
	    pixmap=gdk_pixmap_new(win,width,height,
				  gtk_widget_get_visual(disp)->depth);
	    gdk_draw_rectangle( pixmap,
				disp->style->black_gc,
				TRUE, 0,0,
				width,height);
	    gdk_window_set_back_pixmap(win,pixmap,0);
	    gdk_window_clear(win);
	}
    }
    return TRUE;
}

void init_colortab(void)
{
    gint i,j;
    gint r,g,b;
    gint rr,gg,bb;
    gint cr[64],cg[64],cb[64];
    unsigned char *data;

    j=0;
    r=30;g=0;b=160;
    rr=r;gg=g;bb=b;
    r=160;g=40;b=140;
    for(i=0;i<16;i++)
    {
	cr[j]=(((15-i)*rr)+(i*r))/15;
	cg[j]=(((15-i)*gg)+(i*g))/15;
	cb[j]=(((15-i)*bb)+(i*b))/15;
	j++;
    }
    rr=r;gg=g;bb=b;
    r=210;g=130;b=20;
    for(i=0;i<16;i++)
    {
	cr[j]=(((15-i)*rr)+(i*r))/15;
	cg[j]=(((15-i)*gg)+(i*g))/15;
	cb[j]=(((15-i)*bb)+(i*b))/15;
	j++;
    }
    rr=r;gg=g;bb=b;
    r=240;g=200;b=20;
    for(i=0;i<16;i++)
    {
	cr[j]=(((15-i)*rr)+(i*r))/15;
	cg[j]=(((15-i)*gg)+(i*g))/15;
	cb[j]=(((15-i)*bb)+(i*b))/15;
	j++;
    }
    rr=r;gg=g;bb=b;
    r=255;g=240;b=80;
    for(i=0;i<16;i++)
    {
	cr[j]=(((15-i)*rr)+(i*r))/15;
	cg[j]=(((15-i)*gg)+(i*g))/15;
	cb[j]=(((15-i)*bb)+(i*b))/15;
	j++;
    }
    for(i=0;i<bands;i++)
    {
	for(j=0;j<64;j++)
	{
	    r=cr[j]+((i-16)*4);
	    g=cg[j]+((i-16)*4);
	    b=cb[j]+((i-16)*4);
	    if (r<0) r=0;
	    else if (r>255) r=255;
	    if (g<0) g=0;
	    else if (g>255) g=255;
	    if (b<0) b=0;
	    else if (b>255) b=255;
	    colortab[i][j]=gdk_imlib_best_color_match(&r,&g,&b);
	}
    }
    data=malloc(64*3);
    for(i=0;i<64;i++)
    {
	data[(i*3)]=cr[63-i];
	data[(i*3)+1]=cg[63-i];
	data[(i*3)+2]=cb[63-i];
    }
    im=gdk_imlib_create_image_from_data(data,NULL,1,64);
    for(i=128;i<256;i++)
    {
	gdk_imlib_render(im,1,i-127);
	grad[i]=gdk_imlib_move_image(im);
    }
    gdk_imlib_flip_image_vertical(im);
    for(i=0;i<127;i++)
    {
	gdk_imlib_render(im,1,127-i);
	grad[i]=gdk_imlib_move_image(im);
    }
    gdk_imlib_kill_image(im);
}

void handle_read(gpointer data, gint source, GdkInputCondition condition)
{   
    gint count;
    if (last_is_full)
    {
	curbuf++;
	if(curbuf>=BUFS) 
	    curbuf = 0;
    }
    //    g_print("reading into buffer %i to get is: %i\n",curbuf,to_get);
    count = read(sound, aubuf[curbuf] + pos, to_get);
    if (count < 0)
    {
	//      g_print("Couldn't get any data!!!!\n");

	exit(1);
    }
    else
    {
	//      g_print("Got %i bytes\n",count); 
	pos += count;
	to_get -= count;
    }
    if (to_get == 0)
    {
	to_get = NSAMP; //*2;
	//      g_print("Buffer full, resetting pos to 0 and to_get to %i\n",to_get);
	pos = 0;
	last_is_full = 1;       // buffer full can crunch it now...
	draw();
    }
    else
	last_is_full = 0;       // not full yet
}

