/* im.c - (C) 1998 Tomas Berndtsson (tomas@nocrew.org)
 *
 *  This piece of software is released under the terms of the GNU General
 *  Public Licence version 2 or, at your choice, any higher version.
 *  The GNU GPL is included with this package in the file COPYING.
 */

#include <stdio.h>
#include <stdlib.h>
#include <magick/magick.h>

#include "magick.h"
#include "fb.h"

#define MAXCOLOUR(a) ((int)((a)>65535?65535:(a)))

static void runlength_decode(fb_picture *pic, Image *image, j_settings *js)
{
  unsigned char *dataptr = (unsigned char *) pic->chunkdata;
  RunlengthPacket *image_data = image->pixels;
  int x, y, i=0;
  unsigned char chunk = image_data->index;

  for(y=0 ; y<pic->height ; y++) {
    for(x=0 ; x<pic->width ; x++) {
	i--;
	if(i < 0) {
	  chunk = image_data->index;
	  i = image_data->length;
	  image_data++;
	}
	*dataptr++ = chunk;
    }
  
    if(js->verbose)
      fprintf(stderr,"\r%d/%d",y,pic->height);
  }

  if(js->verbose)
    fprintf(stderr,"\r%d/%d\n",y,pic->height);
}

fb_picture *im_convert(char *filename, j_settings *js)
{
  int col;
  Image *image;
  ImageInfo image_info;
  QuantizeInfo quantize_info;
  fb_picture *pic;

  if((pic=malloc(sizeof(fb_picture))) == NULL) {
    return NULL;
  }

  GetImageInfo(&image_info);
  strcpy(image_info.filename,filename);
  image_info.colorspace = RGBColorspace;
  if(js->verbose)
    fprintf(stderr,"Reading %s...\n", filename);

  image=ReadImage(&image_info);
  if(image == NULL) {
    exit(1);
  }

  if(js->verbose)
    fprintf(stderr,"Width: %d  Height: %d\n", image->columns, image->rows);

  if(image->colormap == NULL) {
    if(js->verbose)
      fprintf(stderr,"Quantize to 256 colours...\n");
    quantize_info.number_colors = 256;
    quantize_info.tree_depth = js->tree_depth;
    quantize_info.dither = 1;
    quantize_info.colorspace = RGBColorspace;
    QuantizeImage(&quantize_info,image);
  }

  /*
  if(js->verbose)
    fprintf(stderr,"Runlength decoding......\n");
  if(RunlengthDecodeImage(image) == False) {
    fprintf(stderr,"Runlength decoding failed.\n");
    DestroyImage(image);
    return NULL;
  }
  */

  if((pic = fb_alloc_picture(image->columns, image->rows,
			     js->bitplanes)) == NULL) {
    DestroyImage(image);
    return NULL;
  }

  runlength_decode(pic, image, js);

  for(col=0 ; col < image->colors ; col++) {
    pic->pal_red[col]   = MAXCOLOUR(((int)image->colormap[col].red   << 8)*js->brightness);
    pic->pal_green[col] = MAXCOLOUR(((int)image->colormap[col].green << 8)*js->brightness);
    pic->pal_blue[col]  = MAXCOLOUR(((int)image->colormap[col].blue  << 8)*js->brightness);
  }
  for( ; col < (1<<pic->bitplanes) ; col++) {
    pic->pal_red[col]   = 0;
    pic->pal_green[col] = 0;
    pic->pal_blue[col]  = 0;
  }

  DestroyImage(image);

  return pic;
}






