/*
 * browse.h: definitions, variables and func prototypes specific to mode -b.
 * 
 * This file is part of F-CULT.
 * 
 * Copyright (C) 1999 Lauri Hstholm
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * To contact Lauri Hstholm, send e-mail to laurijh@iname.com
 */

#include <signal.h>
#include <sys/wait.h>


/* Commands/status reports from the downloader: */
#define CMD_ABORT 1 /* kill the child */
#define CMD_ERRMSG 2 /* after this a 2-char length of the error message, then the error message itself */
#define STAT_FINISHED 5 /* download completed */
#define STAT_FILECMPLT 6 /* file downloaded */
#define STAT_KBDLED 7 /* KiloBytes DownLoadED, not KeyBoarD LED :) */

#define INFOTIME 2
#define BOTTOMHEIGHT 2
#define LISTWINHEIGHT LINES-BOTTOMHEIGHT-1

#define BOFMARK "*** BEGINNING OF FILE ***"
#define EOFMARK "*** END OF FILE ***"
#define SEARCHPROMPT "Enter search string: "
#define CONTPROMPT "Press PgDn to see more"

#define PAGENAME0 "Contents"
#define PAGENAME1 "Keystroke summary"
#define PAGENAME2 "Main screen"
#define PAGENAME3 "File tagging"
#define PAGENAME4 "Pattern searching"
#define PAGENAME5 "File downloading"
#define PAGENAME6 "Help system"
#define PAGENAME7 "[empty]"
#define PAGENAME8 "Software license"

#define HP0 "\
 Welcome to the interactive help system of F-CULT.\n\
 \n\
 Use the number keys or the arrow keys to change the page.\n\
 \n\
 Help pages:\n\
 \n\
 1 ..... " PAGENAME0 "\n\
 2 ..... " PAGENAME1 "\n\
 3 ..... " PAGENAME2 "\n\
 4 ..... " PAGENAME3 "\n\
 5 ..... " PAGENAME4 "\n\
 6 ..... " PAGENAME5 "\n\
 7 ..... " PAGENAME6 "\n\
 8 ..... " PAGENAME7 "\n\
 9 ..... " PAGENAME8 "\n\
 \n\
 Press 'q' to get back to the main screen."

#define HP1 "\
 In main screen:\n\
 \n\
 d                 Enter download screen\n\
 h                 Enter the interactive help system\n\
 t, ' ' / T        Tag for download (not recursive/recursive)\n\
 s, /              Search for a pattern\n\
 q                 Quit the browser\n\
 enter             'Press' currently highlighted button\n\
 left / right      Move to next / previous button\n\
 up / down         Move cursor one line up / down the list\n\
 PgUp / PgDn       Move cursor one screenful up / down the list\n\
 Home, g / End, G  Move cursor to the first / last line of the list\n\
 CTRL+L            Clear the screen\n\
 \n\
 In search screen (after entering initial search string):\n\
 \n\
 r                 Revise search string\n\
 n, up / p, down   Search for next / previous match\n\
 \n\
 In download screen (buttons work as in main screen):\n\
 \n\
 h                 See the download page of the help system\n\
 n / p             Next / previous site\n\
 e                 Edit site properties\n\
 r                 Remove site\n\
 d                 Begin downloading from current site\n\
 a                 Abort current download\n\
 s                 Save site list to config file (default ~/.fcultrc)\n\
 q, m              Back to main screen\n\
 up / down         Edit last / first field of site properties\n\
 \n\
 In help screen:\n\
 \n\
 1 - 9             Change to the page of the number pressed\n\
 q                 Back to main screen\n\
 left / right      Next / previous page\n\
 up / down         Move one line up / down the page\n\
 PgUp / PgDn       Move one pageful up / down the page"

#define HP2 "\
 The main screen consists of two parts, separated by a horizontal line;\n\
 the file list you're browsing and a menu/info bar.\n\
 \n\
 In the file list you see names of files and directories on the left side\n\
 and the sizes of the files on the right side. When you tag a file or the\n\
 contents of a directory for download (more of this later), it gets a star\n\
 sign ('*') on its left side. When the file has been downloaded, the star\n\
 turns into a dash ('-').
 \n\
 There're several ways to move on the list, by single lines with the arrow\n\
 keys, by screenfuls with PgUp / PgDn, and to the beginning / end of the\n\
 list with Home / End or 'g' / 'G'. Tagging also moves the cursor, see the\n\
 fourth page, '" PAGENAME3 "', for more information. Searching for a file\n\
 pattern moves the cursor to the next/previous match.\n\
 \n\
 The lower part of the main screen, the menu / info bar, consists of five\n\
 buttons on the first line and information about list position and the\n\
 state of a possible download on the second line. To activate the action\n\
 of a button, press the first letter of its name or move over it and press\n\
 enter. The case of the letter you press doesn't matter, except with the\n\
 file tagging button (see page 4). There are also alternate keypresses for\n\
 some of the buttons. They exist to conform to the usual Unix conventions.\n\
 For more information, see page number 2, '" PAGENAME1 "'.\n\
 \n\
 If you resize your screen during an F-CULT session, or for some other\n\
 reason want to refresh the screen, you can press ctrl-l to re-initialize\n\
 ncurses and to re-read the screen width and height variables."

#define HP3 "\
 With the '(T)ag file' action you can select files to download later. The\n\
 download-mark is a togglable flag, i.e. if you try to tag a file that's\n\
 already tagged, it'll be untagged. Tagged file has a '*' sign on its left\n\
 side.\n\
 \n\
 Directory tagging works differently depending of whether you use upper-\n\
 case 'T' or something else (the space bar, lower-case 't' or 'press' the\n\
 button with the enter key); upper-case 'T' makes the tag recursive, i.e.\n\
 the files in every subdirectory of the tagged dir will also be tagged.\n\
 \n\
 Tagging a file moves the cursor down one line, tagging a directory moves\n\
 the cursor to the next directory ('T' moves the cursor to the first dir\n\
 that isn't a subdirectory of the tagged directory, so with complicated\n\
 directory structures the cursor may move thousands of lines with just one\n\
 press of the upper-case 'T').\n\
 \n\
 A file that's already been downloaded has a '-' sign on its left side,\n\
 and cannot be tagged or downloaded again during the current session. You\n\
 will have to quit the browser and invoke it again to redownload the file."

#define HP4 "\
 As you enter the pattern searcher, you'll be prompted for a search string\n\
 in the bottom window. The string can be any part of any file entry (but\n\
 a file entry only). Write the string, then press the down key or enter to\n\
 search forward for the first match. To search backwards, press the up key\n\
 instead.\n\
 \n\
 After the first match has been found, you can use 'n' or the down key to\n\
 find the next entry and 'p' or the up key to find the previous one. To\n\
 change the search pattern in the middle of a search, press 'r'. Any other\n\
 key returns to the main menu.\n\
 \n\
 Notice that since the first entry on the screen is the first file to be\n\
 matched against, the searcher won't find anything if all the matches are\n\
 above the current screen, so you might want to move to the beginning of\n\
 the list before searching."

#define HP5 "\
 The download system has a menu/info bar similar to the main screen at the\n\
 bottom window and a site properties list at the top window.\n\
 \n\
 You can have up to 1024 sites stored in your site list file, but only one\n\
 will be shown at a time. To see the next or previous site, press 'n' or\n\
 'p', respectively. To edit the site properties, press 'e' or down arrow\n\
 to move the cursor to the first field, or up arrow for the last field. To\n\
 add a new site to the list, edit the empty site found as the last site on\n\
 the list. A new empty site form will be automatically added to the site\n\
 list. Press 's' to save the site list in the config file (~/.fcultrc by\n\
 default, change it with the -f command line parameter). Even if you don't\n\
 save the list, it will be preserved during the current browser session.\n\
 \n\
 For every site, there're 5 fields of properties. They all must be present\n\
 if you want to download from the site or save it to the site list file.\n\
 The first field (URL) is the Internet address of the remote site, the 2nd\n\
 (Port) is the FTP port to connect to (21 by default), the 3rd (Login) is\n\
 the user as whom to log on the site, the 4th (Password) is obvious, the\n\
 5th is the local directory where the downloaded files will be stored in.\n\
 \n\
 When you're happy with the properties, press 'd' to start downloading the\n\
 tagged files from the current site. To abort the download in progress,\n\
 press 'a'. You can go back to the main window during the download and do\n\
 whatever you want, e.g. tag more files for download. The files you tag\n\
 during a download won't be downloaded by the current session though, you\n\
 will have to wait for the current download to finish or abort it first.\n\
 If you don't need a site anymore, you can get rid of it by pressing 'r'\n\
 when it's being displayed. The rest of the sites are shifted one position\n\
 backwards.\n\
 \n\
 During a download, you see the number of downloaded bytes, total bytes,\n\
 percentage of downloaded bytes of total bytes, and the same for files.\n\
 \n\
 If something goes wrong during the download, you'll see an error message\n\
 in the bottom window telling you what happened. A similar message also\n\
 appears when the download is finished.\n\
 \n\
 The error/finish messages are also shown in the main listing, and you can\n\
 see the tagged-signs changing into downloaded-signs on the list as you\n\
 receive the files."
 
#define HP6 "\
 The interactive help system of the browser part of F-CULT offers you some\n\
 information about how the browser works and what you should do to get the\n\
 most out of it. It doesn't help you on any other working mode of F-CULT\n\
 but the browser, and neither does it tell you how to invoke the browser.\n\
 To get help on these subjects, read the manual page.\n\
 \n\
 In the help system, the top window contains the actual help texts and the\n\
 bottom window tells you which page you're on. To move between the pages,\n\
 either press the number of the page to go to, or use the left and right\n\
 arrow keys. If the page doesn't fit on your screen, you can scroll it up\n\
 and down with the up / down arrow keys and PgUp / PgDn. The help pages\n\
 work with screen width 80, just like every other part of the browser. To\n\
 get back to the main screen from the help system, press 'q'.\n\
 \n\
 The first page, '" PAGENAME0 "', tells you which page to go to for a certain\n\
 subject. The second page, '" PAGENAME1 "' lists all the keys you can\n\
 use in different parts of the browser. For an overview of the actual list\n\
 browser, see the third page, '" PAGENAME2 "'. To find out the details of\n\
 tagging files for download, see the fourth page, '" PAGENAME3 "'. Page\n\
 number 5, '" PAGENAME4 "', tells you how to find a string among the\n\
 file entries on the list. The sixth page, '" PAGENAME5 "', introduces\n\
 the downloading system. The seventh page (this page), '" PAGENAME6 "',\n\
 helps you in navigating the help system. The eighth page is reserved for\n\
 future additions. The ninth page '" PAGENAME8 "', contains the GPL\n\
 copyright notice."

#define HP7 "\
 Reserved for future additions"

#define HP8 LICENSESTR


int currentbutton;	/* currently highlighted button */
int currentline;	/* currently highlighted line, 0 < currentline < LINES-BOTTOMHEIGHT */
int dlpipe[2];		/* pipe from the downloader to the main prog */
int dlsite;		/* number of site dl'ing from + 1, 0 if none */
int showsizes;		/* show file sizes (0 only if they're all zeros */

long bytesdled;		/* # of bytes downloaded */
long bytestodl;		/* tagged bytes at the time of download start */
long currentfile;	/* # of the fptr at the topmost visible line */
long filesdled;		/* # of files downloaded */
long filestodl;		/* tagged files at the time of download start */
long taggedfiles;	/* total # of files to download */
long taggedbytes;	/* total size of tagged files */

WINDOW *bottom;		/* menu/info window, on bottom */
WINDOW *listwin;	/* the main window, on top */
WINDOW *separator;	/* just a line between bottom and listwin */


int checksubs(long dirtocheck, int add);
int countlines(char *str);
int getchildstatus(void);
int loadrcfile(char ***site);
int mymvwgetnstr(WINDOW *win, int y, int x, char *str, int n);
int savercfile(char ***site, int sites);
int strsrch(long fn, char *srchpat);

char *itoa(int i);

void bottominfo(char *infostring1, char *infostring2);
void boxbottom(void);
void changebutton(int newbut);
void chdlbutt(int buttnum);
void deccursor(WINDOW *win, int limit);
void dlscreen(void);
void downloadem(char *url, char *port, char *login, char *passwd, char *dldir);
void drawbutton(int buttonnumber, int mode);
void drawseparator(void);
void gethelp(int initpage, int dontrefresh);
void goend(void);
void gohome(void);
void inccursor(WINDOW *win, int limit);
void killdl(int quietkill);
void linedown(void);
void lineup(void);
void mainbottominfo(int searching);
void parenterrmsg(char *msg1, char *msg2);
void pressbutton(void);
void printlist(void);
void printsite(int sitenum, char **prompt, char ***site);
void redrawbottom(int hilitebutton);
void searchpat(void);
void skiptofile(int entrynum);
void tagentry(int recursive);
void tagfile(int fptrblock, long fptrfile);
