/*
 * main.h: general definitions, global variables and function prototypes.
 * 
 * This file is part of F-CULT
 * 
 * Copyright (C) 1999 Lauri Hstholm.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * To contact Lauri Hstholm, send e-mail to laurijh@iname.com
 */

#ifdef HAVE_CONFIG_H
 #include <config.h>
#else
 #include "dosconf.h"
#endif

#ifdef HAVE_DIRENT_H
 #include <dirent.h>
#else
 #define dirent direct
 #ifdef HAVE_NDIR_H
  #include <ndir.h>
 #endif
 #ifdef HAVE_SYS_DIR_H
  #include <sys/dir.h>
 #endif
 #ifdef HAVE_SYS_NDIR_H
  #include <sys/ndir.h>
 #endif
#endif

#ifdef NEED_MEMORY_H
 #include <memory.h>
#endif

#ifdef TM_IN_SYS_TIME
 #include <sys/time.h>
#else
 #include <time.h>
#endif

#ifdef HAVE_UNISTD_H
 #include <unistd.h>
#endif

#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>

#ifdef HAVE_GETOPT_H
 #include <getopt.h>
#endif


/* There actually is no limit for number of directories for the whole fs.
 * Max for subdirs in one dir is EXT2_LINK_MAX (=32000), but more than 16k
 * would make some malloc()'s > 64k. 16k ought to be enough anyway. */
#ifndef MAXDIRS
 #define MAXDIRS 16384
#endif

#ifndef MAXLINELEN
 #define MAXLINELEN 1536
#endif

#ifndef CATEGORIES
 #define CATEGORIES 54
#endif

#ifndef MAXFILESINCATEGORY
 #define MAXFILESINCATEGORY 16384
#endif

#ifndef INITFILESINCATEGORY
 #define INITFILESINCATEGORY 16
#endif

#ifndef FPTRBLOCKSIZE
 #define FPTRBLOCKSIZE 4096
#endif

#ifndef NAME_MAX
 #ifdef MAXNAMELEN
  #define NAME_MAX MAXNAMELEN
 #else
  #define NAME_MAX 255
 #endif
#endif

/* Compression programs: */
#define GZIP 1
#define BZIP2 2
#define ZIP 3

/* Modes of operation: */
#define OR 1
#define XOR 2
#define NOT 3
#define AND 4

/* Supported list types: */
#define SLD 1
#define SLX 2
#define DDD 3
#define DEFAULTTYPE DDD

/* Error message codes to errmsg(): */
#define FCULT_INV_TYPE 1
#define FCULT_EXTRA_MODE 2
#define FCULT_BROWSE_STDIN 3
#define FCULT_STDIN_CONF 4

/* GPL notice, for --license and the browser help. */
#define LICENSESTR "\
 F-CULT version " VERSION "\n\
 Copyright (C) 1999 Lauri Hstholm\n\
 \n\
 This program is free software; you can redistribute it and/or modify\n\
 it under the terms of the GNU General Public License as published by\n\
 the Free Software Foundation; either version 2 of the License, or\n\
 (at your option) any later version.\n\
 \n\
 This program is distributed in the hope that it will be useful,\n\
 but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
 GNU General Public License for more details.\n\
 \n\
 You should have received a copy of the GNU General Public License\n\
 along with this program; if not, write to the Free Software\n\
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n"

 
#ifdef HAVE_LIBNCURSES
 #include <ncurses.h>
 #include "ftplib.h"

 bool plaingfx;		/* don't use 8-bit chars for screen graphics? */
 long entries;		/* files+dirs+(BOF & EOF)+1 */
 char *configfile;	/* site list file */

 struct filepointer
 {
	char type;	/* 0=BOF/EOF, 1=dir, 2=file, 3=tagged file, 4=downloaded file */
	short dirnum;
	char category;
	short filenum;
 }**fptr;

 void browse_indexize_files(void);
 void browse_main_loop(void);
#else
 typedef char bool;
#endif

bool allinroot;		/* make everything look like they're in the root of the list? */
bool allowtrailing;	/* allow one alphabetic letter after the number (-N)? */
bool browse;		/* --browse used? */
bool dosdirs;		/* '\' as dir separator instead of '/'? */
bool foverwrite;	/* force overwrite of an existing file? */
bool ignorecase;	/* case doesn't matter when comparing and filtering? */
bool noemptydirs;	/* don't output empty dirs? */
bool nolinks;		/* read symlinks? */
bool nonrecursive;	/* don't read any subdirs? */
bool quiet;		/* output nothing but critical error messages on screen? */
bool stripdirs;		/* remove leading dir names? */
bool verbose;		/* echo each read dir on screen? */

int cd;			/* current dir, a 'pointer' to first free dircode */
int compressor;		/* compression method; one of GZIP, BZIP2, ZIP */
int dirs;		/* # of dirs in the list file */
int filters;		/* # of file suffix filters */
int inlt;		/* list type from gettype() */
int outlt;		/* list type provided by user */
int mode;		/* mode of operation, one of: NOT, AND, XOR, OR */
int startfrom;		/* first file in series to be output (-N) */

short *activefiles;	/* file count for each dir of list # 1*/
short *alphadir;	/* #'s of alphabetically sorted dircodes */
short **csf;		/* current sfile (first free one in category/dir) */
short inputfiles;	/* # of input file names */

long files;		/* # of files in the list */

char **dircode;		/* all directories, in the order of reading */
char *dirprefix;	/* string to add in front of dir names */
char **inputname;	/* list of input file names */
char *outputname;	/* name of the file to be written to */
char **suffilter;	/* list of file suffixes to include in the listing */

typedef struct
{
	short dirnum;
	char listnum;
	long filetime;
	long unsigned filesize;
       	char *filename;
}filestruct;
filestruct ****sfile;	/* one for every file of every category of every dir */


/* main.c: */
int check_access(int quiet, int force_overwrite, int mode, char *filename);
int check_files(int argc, char **argv);
int has_suffix(int caseinsensitive, char *string, char *suffix);
int main(int argc, char **argv);
int parse_options(int argc, char **argv);
int read_write_files(void);

void errmsg(int errormode, char *name);
void parse_filters(char *optarg);
void show_help(char *name);
void *xcalloc(size_t nmemb, size_t size);
void *xmalloc(size_t size);
void *xrealloc(void *ptr, size_t size);

/* filein.c: */
int read_input(int quieterrors, short listnum, char *infilename);
int store_dir(char *dirname);
int store_file(char *filename, int dirnum, short listnum, long unsigned filesize, long filetime);

/* fileout.c: */
int any_active_files(int dirnum);
int write_output(char *filename);

/* not.c: */
int findseries(int dir, int category);
int getdigit(char *string);
int is_series(char *file1, char *file2);
int not_main_loop(void);
char *firstpart(char *string);
char *itoa_pad(int i, int width);
char *secondpart(char *string);
