/*
 * browse.c: browse a list (ncurses-based browser).
 * 
 * This file is part of F-CULT.
 * 
 * Copyright (C) 1999 Lauri Hstholm
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * To contact Lauri Hstholm, send e-mail to laurijh@iname.com
 */

#include "main.h"

#ifdef HAVE_LIBNCURSES

#include "browse.h"

/* Get a keypress from the user and call the right function. */

void browse_main_loop()
{
	int i;
	char buttonid[] = "tsdhq";

	currentline = 1;
	currentfile = 0;
	goto initialize;
	for (;;) {
		if (dlsite) {
			if (getchildstatus()) {
				redrawbottom(currentbutton);
				printlist();
			}
		}

		switch (wgetch(bottom)) {
			case 12: /* ctrl-l */
				delwin(listwin);
				delwin(separator);
				delwin(bottom);
				endwin();
initialize:
				initscr();
				halfdelay(1);
				noecho();
				nonl();
				curs_set(0);
				bottom = newwin(BOTTOMHEIGHT, COLS, LISTWINHEIGHT+1, 0);
				redrawbottom(currentbutton);
/* Can't get this work without repeating the newwin and
 * redrawbottom calls (and if we leave out the extra delwin call,
 * every ctrl-l increases memory usage by 4k). Strange, huh? */
				delwin(bottom);
				listwin = newwin(LISTWINHEIGHT, COLS, 0, 0);
				separator = newwin(3, COLS, LISTWINHEIGHT, 0);
				bottom = newwin(BOTTOMHEIGHT, COLS, LISTWINHEIGHT+1, 0);
				keypad(bottom, TRUE);
				intrflush(bottom, FALSE);
				skiptofile(currentfile+currentline);
				drawseparator();
				redrawbottom(currentbutton);
				break;

			case 13: /* enter */
				ungetch(buttonid[currentbutton]);
				break;

			case 'D':
			case 'd':
				dlscreen();
				break;

			case 'H':
			case 'h':
				gethelp(1, 0);
				break;

			case ' ':
			case 't':
				tagentry(0);
				break;

			case 'T':
				tagentry(1);
				break;

			case 'Q':
			case 'q':
				redrawbottom(4);
				endwin();
				printf("\n");
				return;

			case '/':
			case 'S':
			case 's':
				searchpat();
				break;

			case KEY_LEFT:
				changebutton((currentbutton > 0) ? currentbutton-1 : 4);
				break;

			case KEY_RIGHT:
				changebutton((currentbutton < 4) ? currentbutton+1 : 0);
				break;

			case KEY_UP:
				lineup();
				printlist();
				mainbottominfo(0);
				break;

			case KEY_DOWN:
				linedown();
				printlist();
				mainbottominfo(0);
				break;

			case KEY_PPAGE:
				for (i=0; i<LISTWINHEIGHT-1; i++)
					lineup();
				printlist();
				mainbottominfo(0);
				break;

			case KEY_NPAGE:
				for (i=0; i<LISTWINHEIGHT-1; i++)
					linedown();
				printlist();
				mainbottominfo(0);
				break;

			case 'g':
			case KEY_HOME:
				gohome();
				break;

			case 'G':
			case KEY_END:
				goend();
				break;
		}
	}
}

/* Draw a line between listwin and bottom. */

void drawseparator()
{
	if (plaingfx)
		mvwhline(separator, 0, 0, '-', COLS);
	else
		mvwhline(separator, 0, 0, ACS_HLINE, COLS);
	wrefresh(separator);
}

/* Print an informative message at the 2nd line (ok, 3rd actually, but we
 * never use lines 1 and 4) of bottom screen, but unlike bottominfo(),
 * doesn't clear it after a pause. */

#define NEXTSRCH "(N)ext"
#define PREVSRCH "(P)revious"
#define RESRCH "(R)evise pattern"

void mainbottominfo(int searching)
{
	wmove(bottom, 1, 2);
	wclrtoeol(bottom);
	if (searching)
		wprintw(bottom, "%s  %s  %s  On line %ld of %ld total lines (%hd%%).", NEXTSRCH, PREVSRCH, RESRCH, currentline+currentfile, entries-3, (100*(currentline+currentfile))/(entries-3));
	else if (dlsite)
		wprintw(bottom, "Line %ld/%ld (%hd%%). Download at %ld/%ld (%hd%%) bytes, %ld/%ld (%hd%%) files.", currentline+currentfile, entries-3, (100*(currentline+currentfile))/(entries-3), bytesdled, bytestodl, (100*bytesdled)/bytestodl, filesdled, filestodl, (100*filesdled)/filestodl);
	else
		wprintw(bottom, "On line %ld of %ld total lines (%hd%%). Bytes in tagged files: %ld", currentline+currentfile, entries-3, (100*(currentline+currentfile))/(entries-3), taggedbytes);
	wrefresh(bottom);
}

/* Change button attributes. */

void changebutton(int newbut)
{
	drawbutton(currentbutton, A_NORMAL);
	currentbutton = newbut;
	drawbutton(currentbutton, A_STANDOUT);
}

/* Draw given button in given mode. */

void drawbutton(int buttonnumber, int mode)
{
	int i;
	int starthere = 2;
	char *buttonstr[5] = {"(T)ag", "(S)earch", "(D)ownload", "(H)elp", "(Q)uit"};

	for (i=0; i<buttonnumber; i++)
		starthere += strlen(buttonstr[i])+2;
	wmove(bottom, 0, starthere);
	for (i=0; i<strlen(buttonstr[buttonnumber]); i++)
		waddch(bottom, buttonstr[buttonnumber][i] | mode);
}


/* Redraw the bottom window and the buttons inside it. */

void redrawbottom(int hilitebutton)
{
	int c;

	noecho();
	werase(bottom);
	for (c=0; c<5; c++)
		drawbutton(c, A_NORMAL);
	drawbutton(hilitebutton, A_STANDOUT);
	mainbottominfo(0);
	wrefresh(bottom);
}

/* Convert an integer to a char *. Contents of the
 * returned array change with every call to this function. */

char *itoa(int i)
{
	int j;
	static char *a;
	char digit[11] = "0123456789";

	if (a == NULL)
		a = xmalloc(11);
	for (j=9; j>=0; j--) {
		a[j] = digit[i%10];
		i /= 10;
	}
	a[10] = '\0';
	for (j=0; j<10; j++) {
		if (a[j] != '0')
			return &a[j];
	}
	return &a[10];
}

/* Arrange directory and file entries in a continuous array. */

void browse_indexize_files()
{
	int d;
	int dir;
	int category;
	int fptrb = 0;
	int fptrfile = 0;
	int sfilenum;

	fptr[fptrb][fptrfile].type = 0; /* BOF */
	fptr[fptrb][fptrfile].filenum = -1;
	fptr[fptrb][fptrfile].dirnum = -1;
	fptr[fptrb][fptrfile++].category = -1;
	for (d=0; d<cd; d++) {
		dir = alphadir[d];
		if (!any_active_files(dir)) {
			entries--;
			continue;
		}
		if (csf != NULL) {
			if (csf[dir] == NULL && noemptydirs) {
				entries--;
				continue;
			}
		}
		fptr[fptrb][fptrfile].type = 1;
		fptr[fptrb][fptrfile].filenum = -1;
		fptr[fptrb][fptrfile].dirnum = dir;
		fptr[fptrb][fptrfile++].category = -1;
		if (fptrfile == FPTRBLOCKSIZE) {
			fptrb++;
			fptrfile = 0;
		} else if (csf == NULL)
			continue;
		else if (csf[dir] == NULL)
			continue;
		for (category=0; category<CATEGORIES; category++) {
			if (sfile[dir][category] == NULL)
				continue;
			for (sfilenum=0; sfilenum<csf[dir][category]; sfilenum++) {
				if ((mode == NOT || mode == XOR) && sfile[dir][category][sfilenum]->listnum)
					continue;
				fptr[fptrb][fptrfile].type = 2;
				fptr[fptrb][fptrfile].filenum = sfilenum;
				fptr[fptrb][fptrfile].dirnum = dir;
				fptr[fptrb][fptrfile++].category = category;
				if (sfile[dir][category][sfilenum]->filesize)
					showsizes = 1;
				if (fptrfile == FPTRBLOCKSIZE) {
					fptrb++;
					fptrfile = 0;
				}
			}
		}
	}
	fptr[fptrb][fptrfile].type = 0; /* EOF */
	fptr[fptrb][fptrfile].filenum = -1;
	fptr[fptrb][fptrfile].dirnum = -1;
	fptr[fptrb][fptrfile].category = -1;
}

/* Print LINES files in listwin(), starting at fptr[currentfile]. */

void printlist()
{
	int fnlength;
	int fptrb = 0;
	int fptrfile;
	int i;
	int j;
	char sizestr[13];

	werase(listwin);
	fptrb = currentfile/FPTRBLOCKSIZE;
	fptrfile = currentfile-(fptrb*FPTRBLOCKSIZE);
	for (i=0; i<LINES-BOTTOMHEIGHT; i++) {
		if (i+fptrfile >= FPTRBLOCKSIZE) {
			fptrfile -= FPTRBLOCKSIZE;
			fptrb++;
		}
		if (!fptr[fptrb][fptrfile+i].type) {
			if (fptrfile || i) {
				mvwaddstr(listwin, i, (COLS/2)-(strlen(EOFMARK)/2), EOFMARK);
				wrefresh(listwin);
				return;
			} else
				mvwaddstr(listwin, 0, (COLS/2)-(strlen(BOFMARK)/2), BOFMARK);
		} else {
			if (fptr[fptrb][fptrfile+i].filenum < 0) {
				fnlength = strlen(dircode[fptr[fptrb][fptrfile+i].dirnum]);
				if (i == currentline) {
					wmove(listwin, i, (COLS/2)-(fnlength/2));
					for (j=0; j<fnlength; j++)
						waddch(listwin, dircode[fptr[fptrb][fptrfile+i].dirnum][j] | A_STANDOUT);
				} else
					mvwaddstr(listwin, i, (COLS/2)-(fnlength/2), dircode[fptr[fptrb][fptrfile+i].dirnum]);
				i += fnlength/COLS;
			} else {
				fnlength = strlen(sfile[fptr[fptrb][fptrfile+i].dirnum][(int)fptr[fptrb][fptrfile+i].category][fptr[fptrb][fptrfile+i].filenum]->filename);
				if (fptr[fptrb][fptrfile+i].type == 3)
					mvwaddch(listwin, i, 0, '*');
				else if (fptr[fptrb][fptrfile+i].type == 4)
					mvwaddch(listwin, i, 0, '-');
				if (i == currentline) {
					wmove(listwin, i, 2);
					for (j=0; j<fnlength; j++)
						waddch(listwin, sfile[fptr[fptrb][fptrfile+i].dirnum][(int)fptr[fptrb][fptrfile+i].category][fptr[fptrb][fptrfile+i].filenum]->filename[j] | A_STANDOUT);
				} else
					mvwaddstr(listwin, i, 2, sfile[fptr[fptrb][fptrfile+i].dirnum][(int)fptr[fptrb][fptrfile+i].category][fptr[fptrb][fptrfile+i].filenum]->filename);
				i += fnlength/COLS;
				if (showsizes) {
					sprintf(sizestr, "%13lu", sfile[fptr[fptrb][fptrfile+i].dirnum][(int)fptr[fptrb][fptrfile+i].category][fptr[fptrb][fptrfile+i].filenum]->filesize);
					sizestr[0] = sizestr[3];
					for (j=2; j<5; j++)
						sizestr[j] = sizestr[j+2];
					for (j=6; j<9; j++)
						sizestr[j] = sizestr[j+1];
					if (sizestr[0] != ' ')
						sizestr[1] = ',';
					if (sizestr[4] != ' ')
						sizestr[5] = ',';
					if (sizestr[8] != ' ')
						sizestr[9] = ',';
					if ((i-(fnlength/COLS)) == currentline) {
						for (j=0; j<13; j++) {
							if (sizestr[j] != ' ')
								mvwaddch(listwin, i, COLS-(13-j), sizestr[j] | A_STANDOUT);
						}
					} else
						mvwaddstr(listwin, i, COLS-13, sizestr);
				}
			}
		}
	}
	wrefresh(listwin);
}

/* Search for given pattern among file entries. */

void searchpat()
{
	int c = 'r';
	int srchfieldlen;
	long i = (currentfile ? currentfile-1 : 0);
	char *searchpat;

	werase(bottom);
	srchfieldlen = COLS-strlen(SEARCHPROMPT)-3;
	searchpat = xmalloc(srchfieldlen+1);
	mvwaddstr(bottom, 0, 2, SEARCHPROMPT);
	mainbottominfo(1);
	cbreak();
	do {
		switch (c) {
			case 'r':
			case 'R':
				if (mymvwgetnstr(bottom, 0, strlen(SEARCHPROMPT)+2, searchpat, srchfieldlen) == KEY_UP)
					ungetch('p');
				else
					ungetch('n');
				if (!strlen(searchpat)) {
					free(searchpat);
					halfdelay(1);
					redrawbottom(currentbutton);
					return;
				}
				break;

			case KEY_DOWN:
			case 'N':
			case 'n':
				do {
					c = strsrch(++i, searchpat);
				} while (i < entries-3 && !c);
				break;

			case KEY_UP:
			case 'P':
			case 'p':
				do {
					c = strsrch(--i, searchpat);
				} while (i > 0 && !c);
				break;

			default:
				c = -1;
		}
		if (c > 0) {
			mainbottominfo(1);
			c = wgetch(bottom);
		}
	} while (c > 0);
	halfdelay(1);
	if (!c)
		bottominfo("No more matches.", "");
	free(searchpat);
	redrawbottom(currentbutton);
}


/* Search file entry for the search pattern.
 * Yeah, I know about the existance of strstr(),
 * but I wrote this before I read about it and
 * I couldn't get searchpat() to work with it. */

int strsrch(long fn, char *srchpat)
{
	int fptrb;
	int fptrfile = 0;
	int i;
	int j;
	size_t patlength = strlen(srchpat);

	fptrb = fn/FPTRBLOCKSIZE;
	fptrfile = fn-(fptrb*FPTRBLOCKSIZE);
	if (fptr[fptrb][fptrfile].type > 1) {
		for (i=0; i<strlen(sfile[fptr[fptrb][fptrfile].dirnum][(int)fptr[fptrb][fptrfile].category][fptr[fptrb][fptrfile].filenum]->filename); i++) {
			j = 0;
			while (j < patlength && srchpat[j] == sfile[fptr[fptrb][fptrfile].dirnum][(int)fptr[fptrb][fptrfile].category][fptr[fptrb][fptrfile].filenum]->filename[i+j])
				j++;
			if (j == patlength) {
				skiptofile(fn);
				return 1;
			}
		}
	}
	return 0;
}

/* Move cursor to given entry number. */

void skiptofile(int entrynum)
{
	if (entrynum < (LINES-BOTTOMHEIGHT)/2) {
		currentfile = 0;
		currentline = entrynum;
	} else if (entrynum > entries-((LINES-BOTTOMHEIGHT)/2)-2) {
		if (entries < LINES-2)
			currentline = entrynum;
		else {
			currentfile = entries-(LINES-BOTTOMHEIGHT+1);
			currentline = (LINES-BOTTOMHEIGHT+1)-(entries-entrynum);
		}
	} else {
		currentfile = entrynum-(LINES-BOTTOMHEIGHT)/2;
		currentline = (LINES-BOTTOMHEIGHT)/2;
	}
	printlist();
}

/* An mvwgentstr() replacement that can append to a previously
 * written string. Enables cursor during input, turns on keypad
 * for win, doesn't read further than COLS. Returns the key that
 * made it return (enter, up/down arrow). */

int mymvwgetnstr(WINDOW *win, int y, int x, char *str, int n)
{
	int c;
	int i;
	int lastch; /* lastch-x = number of typed characters */
	int posx;

	if (x+n > COLS)
		n = COLS-x;
	wmove(win, y, x);
	for (lastch=x+n; lastch>x; lastch--) {
		if ((mvwinch(win, y, lastch) & A_CHARTEXT) != ' ') {
			lastch++;
			break;
		}
	}
	wmove(win, y, lastch);
	curs_set(1);
	keypad(win, TRUE);
	do {
		c = wgetch(win);
		switch (c) {
			case KEY_HOME:
				wmove(win, y, x);
				break;

			case KEY_END:
				wmove(win, y, lastch);
				break;

			case KEY_LEFT:
				deccursor(win, x);
				break;

			case KEY_RIGHT:
				inccursor(win, lastch);
				break;

			case KEY_DC:
				getyx(win, i, posx);
				if (posx < lastch) {
					wdelch(win);
					mvwinsch(win, y, x+n, ' ');
					wmove(win, i, posx);
					lastch--;
				}
				break;

			case KEY_BACKSPACE:
				getyx(win, i, posx);
				if (posx > x) {
					deccursor(win, x);
					wdelch(win);
					mvwinsch(win, y, x+n, ' ');
					wmove(win, i, posx-1);
					lastch--;
				}
				break;

			default:
				if (lastch < x+n) {
					if (isprint(c)) {
						getyx(win, i, posx);
						mvwdelch(win, y, x+n);
						wmove(win, i, posx);
						winsch(win, c);
						inccursor(win, x+n);
						lastch++;
					}
				}
		}
	} while (c != 13 && c != KEY_UP && c != KEY_DOWN);
	curs_set(0);
	str = xrealloc(str, lastch-x+1);
	mvwinnstr(win, y, x, str, lastch-x);
	return c;
}

/* Move cursor one position to the right unless it already is at limit. */

void inccursor(WINDOW *win, int limit)
{
	int y;
	int x;

	getyx(win, y, x);
	if (x+1 <= limit)
		wmove(win, y, x+1);
}

/* Same as previous, but move left. */

void deccursor(WINDOW *win, int limit)
{
	int y;
	int x;

	getyx(win, y, x);
	if (x-1 >= limit)
		wmove(win, y, x-1);
}

/* Show an info message in the bottom window for INFOTIME seconds.
 * If length of infostring2 != 0, it will be included in the format. */

void bottominfo(char *infostring1, char *infostring2)
{
	wmove(bottom, 1, 2);
	wclrtoeol(bottom);
	if (strlen(infostring2))
		wprintw(bottom, infostring1, infostring2);
	else
		wprintw(bottom, infostring1);
	wrefresh(bottom);
	sleep(INFOTIME);
	wmove(bottom, 1, 2);
	wclrtoeol(bottom);
}

/* The interactive help system. */

void gethelp(int page, int dontrefresh)
{
	int c = *itoa(page);
	int hplines = 0;
	int i;
	int topline = 0;
	char *helppage[9] = {HP0, HP1, HP2, HP3, HP4, HP5, HP6, HP7, HP8};
	char *pagename[9] = {PAGENAME0, PAGENAME1, PAGENAME2, PAGENAME3, PAGENAME4, PAGENAME5, PAGENAME6, PAGENAME7, PAGENAME8};

	for (i=0; i<9; i++) {
		if (countlines(helppage[i]) > hplines)
			hplines = countlines(helppage[i]);
	}
	werase(bottom);
	werase(listwin);
	wrefresh(listwin);
	delwin(listwin);
	listwin = newpad(hplines+1, COLS);
	ungetch(c);
	while (c != 'q' && c != 'Q') {
		switch (c = wgetch(bottom)) {
			case KEY_LEFT:
				ungetch((page > 1) ? (--page)+'0' : (page = 9)+'0');
				break;

			case KEY_RIGHT:
				ungetch((page < 9) ? (++page)+'0' : (page = 1)+'0');
				break;

			case KEY_UP:
				if (topline)
					topline--;
				prefresh(listwin, topline, 0, 0, 0, LISTWINHEIGHT-1, COLS-1);
				break;

			case KEY_PPAGE:
				for (i=0; i<LISTWINHEIGHT; i++) {
					if (topline)
						topline--;
				}
				prefresh(listwin, topline, 0, 0, 0, LISTWINHEIGHT-1, COLS-1);
				break;

			case KEY_DOWN:
				if (topline+LISTWINHEIGHT-1 < hplines)
					topline++;
				prefresh(listwin, topline, 0, 0, 0, LISTWINHEIGHT-1, COLS-1);
				break;

			case KEY_NPAGE:
				for (i=0; i<LISTWINHEIGHT; i++) {
					if (topline+LISTWINHEIGHT-1 < hplines)
						topline++;
				}
				prefresh(listwin, topline, 0, 0, 0, LISTWINHEIGHT-1, COLS-1);
				break;

			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
				topline = 0;
				hplines = countlines(helppage[c-'0'-1]);
				werase(listwin);
				waddstr(listwin, helppage[c-'0'-1]);
				prefresh(listwin, topline, 0, 0, 0, LISTWINHEIGHT-1, COLS-1);
				page = c-'0';
				werase(bottom);
				mvwprintw(bottom, 0, 1, "Page: ");
				for (i=1; i<10; i++) {
					waddch(bottom, '[');
					if (i == page)
						waddch(bottom, c | A_STANDOUT);
					else
						waddch(bottom, i+'0');
					waddch(bottom, ']');
				}
				mvwprintw(bottom, 1, 1, "Help subject: %s", pagename[page-1]);
				wrefresh(bottom);
		}
	}
	delwin(listwin);
	listwin = newwin(LISTWINHEIGHT, COLS, 0, 0);
	werase(bottom);
	redrawbottom(currentbutton);
	scrollok(listwin, FALSE);
	werase(listwin);
	if (!dontrefresh)
		printlist();
}

/* Return number of lines in str. */

int countlines(char *str)
{
	int i;
	int lines = 0;

	for (i=0; i<strlen(str); i++) {
		if (str[i] == '\n')
			lines++;
	}
	return lines;
}

/* Tag a file or a directory (with its subdirs) to be downloaded. */

void tagentry(int recursive)
{
	int add = 0;
	int blocksub; /* subtraction, 0 or 64k */
	int category;
	int fptradd;
	int fptrb;
	int i;

	long basetag = currentfile+currentline;
	long fptrfile;

	fptrb = basetag/FPTRBLOCKSIZE;
	basetag -= fptrb*FPTRBLOCKSIZE;
	fptrfile = basetag;
	if (fptr[fptrb][basetag].type == 1) {
		do {
			fptradd = 0;
			if (csf[fptr[fptrb][fptrfile].dirnum]) {
				for (category=0; category<CATEGORIES; category++) {
					for (i=0; i<csf[fptr[fptrb][fptrfile].dirnum][category]; i++) {
						blocksub = 0;
						if (fptrfile+fptradd+i+1 >= FPTRBLOCKSIZE) {
							fptrb++;
							blocksub = FPTRBLOCKSIZE;
						}
						tagfile(fptrb, fptrfile+fptradd+i+1-blocksub);
						if (blocksub)
							fptrb--;
					}
					fptradd += csf[fptr[fptrb][fptrfile].dirnum][category];
				}
			}
			fptrfile += fptradd+1;
			if (fptrfile >= FPTRBLOCKSIZE) {
				fptrb++;
				fptrfile -= FPTRBLOCKSIZE;
			}
			linedown();
		} while (recursive && checksubs(basetag, ++add));
	}
	else
		tagfile(fptrb, basetag);
	printlist();
	mainbottominfo(0);
}

/* Tag a single file. */

void tagfile(int fptrblock, long fptrfile)
{
	switch (fptr[fptrblock][fptrfile].type) {
		case 2:
			fptr[fptrblock][fptrfile].type = 3;
			taggedfiles++;
			taggedbytes += sfile[fptr[fptrblock][fptrfile].dirnum][(int)fptr[fptrblock][fptrfile].category][fptr[fptrblock][fptrfile].filenum]->filesize;
			break;

		case 3:
			fptr[fptrblock][fptrfile].type = 2;
			taggedfiles--;
			taggedbytes -= sfile[fptr[fptrblock][fptrfile].dirnum][(int)fptr[fptrblock][fptrfile].category][fptr[fptrblock][fptrfile].filenum]->filesize;
	}
	linedown();
}

/* Return 1 if given dir has subdirs, 0 otherwise. */

int checksubs(long dirtocheck, int add)
{
	int fptrb;
	int i;

	fptrb = dirtocheck/FPTRBLOCKSIZE;
	dirtocheck -= fptrb*FPTRBLOCKSIZE;
	if (fptr[fptrb][dirtocheck].dirnum+add >= cd)
		return 0;
	for (i=0; i<strlen(dircode[fptr[fptrb][dirtocheck].dirnum]); i++) {
		if (dircode[fptr[fptrb][dirtocheck].dirnum][i] != dircode[fptr[fptrb][dirtocheck].dirnum+add][i])
			return 0;
	}
	if (dircode[fptr[fptrb][dirtocheck].dirnum+add][i] == '\\' || dircode[fptr[fptrb][dirtocheck].dirnum+add][i] == '/' || (dircode[fptr[fptrb][dirtocheck].dirnum][i-1] == '\\' && dircode[fptr[fptrb][dirtocheck].dirnum+add][i-1] == '\\') || (dircode[fptr[fptrb][dirtocheck].dirnum][i-1] == '/' && dircode[fptr[fptrb][dirtocheck].dirnum+add][i-1] == '/'))
		return 1;
	return 0;
}

/* Move cursor one line up. */

void lineup()
{
	if ((!currentfile && currentline > 1) || ((currentline > (LISTWINHEIGHT+1)/2) && currentfile == entries-LISTWINHEIGHT+5))
		currentline--;
	else if (currentfile)
		currentfile--;
}

/* Move cursor one line down. */

void linedown()
{
	if (currentline+currentfile < entries-3 && (currentline < (LISTWINHEIGHT+1)/2 || currentfile >= entries-(LISTWINHEIGHT/2)))
		currentline++;
	else if (currentfile+LISTWINHEIGHT+1 < entries)
		currentfile++;
}

/* Go to the first directory entry (first line after BOF). */

void gohome()
{
	currentline = 1;
	currentfile = 0;
	printlist();
	mainbottominfo(0);
}

/* Go to the last file/dir entry (last line before EOF). */

void goend()
{
	if (entries >= LISTWINHEIGHT) {
		currentline = LISTWINHEIGHT-2;
		currentfile = entries-(LISTWINHEIGHT)-1;
	} else
		currentline = entries-3;
	printlist();
	mainbottominfo(0);
}

#endif
