/*
 * download.c: download files tagged in the browser.
 * 
 * This file is part of F-CULT.
 * 
 * Copyright (C) 1999 Lauri Hstholm
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * To contact Lauri Hstholm, send e-mail to laurijh@iname.com
 */

#include "main.h"

#ifdef HAVE_LIBNCURSES

#include "browse.h"

#define DEFAULTPORT "21"
#define DLBUTTS 8
#define INITFIELDLEN 16
#define MAXSITES 1024
#define PROMPTS 5
#define SITEINFO " Site # "

/* The download screen. */

void dlscreen()
{
	int curdlbutt = 0;
	int curfield = -1; /* one for each prompt (0-4) */
	int i;
	int j;
	int sites = 0;
	static int cursite;
	char dlbuttid[] = "nperdasm";
	char *prompt[PROMPTS] = {"URL: ", "Port: ", "Login: ", "Password: ", "Dl directory: "};
	char ***site; /* site[sites] is always an empty site form */

	site = xmalloc(MAXSITES*sizeof (char *));
	sites = loadrcfile(site);
	if (sites == -1) {
		sites = 0;
		bottominfo("Can't read %s, using empty site list.", configfile);
	}
	site[sites] = xmalloc(PROMPTS*sizeof (char *));
	for (i=0; i<PROMPTS; i++) {
		site[sites][i] = xmalloc(INITFIELDLEN);
		if (i == 1)
			strcpy(site[sites][i], DEFAULTPORT);
		else
			site[sites][i][0] = '\0';
	}
	werase(bottom);
	werase(listwin);
	chdlbutt(curdlbutt);
	do {
		if (dlsite)
			getchildstatus();
		printsite(cursite, prompt, site);
		switch (wgetch(bottom)) {
			case 13:
				ungetch(dlbuttid[curdlbutt]);
				break;

			case KEY_DOWN:
				curfield = 0;
				break;

			case KEY_UP:
				curfield = PROMPTS-1;
				break;

			case KEY_LEFT:
				chdlbutt(!curdlbutt ? curdlbutt = DLBUTTS-1 : --curdlbutt);
				break;

			case KEY_RIGHT:
				chdlbutt(curdlbutt == DLBUTTS-1 ? curdlbutt = 0 : ++curdlbutt);
				break;

			case 'n':
			case 'N':
				chdlbutt(curdlbutt = 0);
				printsite((cursite == sites ? cursite = 0 : ++cursite), prompt, site);
				break;

			case 'p':
			case 'P':
				chdlbutt(curdlbutt = 1);
				printsite((cursite == 0 ? cursite = sites : --cursite), prompt, site);
				break;

			case 'e':
			case 'E':
				chdlbutt(curdlbutt = 2);
				curfield = 0;
				break;

			case 'r':
			case 'R':
				chdlbutt(curdlbutt = 3);
				if (cursite == sites) {
					bottominfo("Can't remove an empty site.", "");
					break;
				}
				for (i=0; i<PROMPTS; i++)
					free(site[cursite][i]);
				free(site[cursite]);
				for (i=cursite; i<sites; i++)
					site[i] = site[i+1];
				sites--;
				printsite(cursite, prompt, site);
				break;

			case 'd':
			case 'D':
				chdlbutt(curdlbutt = 4);
				if (strlen(site[cursite][0]) && strlen(site[cursite][1]) && strlen(site[cursite][2]) && strlen(site[cursite][3]) && strlen(site[cursite][4])) {
					downloadem(site[cursite][0], site[cursite][1], site[cursite][2], site[cursite][3], site[cursite][4]);
					if (dlsite)
						dlsite = cursite+1;
				}
				else
					bottominfo("All fields must be present!", "");
				break;

			case 'a':
			case 'A':
				chdlbutt(curdlbutt = 5);
				killdl(0);
				break;

			case 's':
			case 'S':
				chdlbutt(curdlbutt = 6);
				if (!savercfile(site, sites))
					bottominfo("Saved site list to %s.", configfile);
				break;

			case 'h':
			case 'H':
				gethelp(6, 1);
				break;

			case 'm':
			case 'M':
			case 'q':
			case 'Q':
				chdlbutt(curdlbutt = 7);
				curdlbutt = -1;
		}

		while (curfield > -1) {
			if (mymvwgetnstr(listwin, 3+curfield*2, strlen(prompt[curfield])+1, site[cursite][curfield], COLS-(strlen(prompt[curfield])+4)) == KEY_UP)
				curfield == 0 ? curfield = -1 : curfield--;
			else
				curfield == PROMPTS-1 ? curfield = -1 : curfield++;
		}
		if (strlen(site[sites][0]) || strlen(site[sites][2]) || strlen(site[sites][3]) || strlen(site[sites][4])) {
			site[++sites] = xmalloc(PROMPTS*sizeof (char *));
			for (i=0; i<PROMPTS; i++) {
				site[sites][i] = xmalloc(INITFIELDLEN);
				site[sites][i][0] = '\0';
			}
			strcpy(site[sites][1], DEFAULTPORT);
		}
	} while (curdlbutt > -1);
	for (i=0; i<=sites; i++) {
		for (j=0; j<PROMPTS; j++)
			free(site[i][j]);
		free(site[i]);
	}
	free(site);
	redrawbottom(currentbutton);
	werase(listwin);
	printlist();
}

/* Print site info. */

void printsite(int sitenum, char **prompt, char ***site)
{
	int i;

	mvwaddstr(listwin, 1, 0, SITEINFO);
	wclrtoeol(listwin);
	if (sitenum == dlsite-1)
		wprintw(listwin, "%d: %ld/%ld files (%hd%%), %ld/%ld bytes (%hd%%) downloaded", sitenum, filesdled, taggedfiles, (short)((100*filesdled)/taggedfiles), bytesdled, taggedbytes, (short)((100*bytesdled)/taggedbytes));
	else
		wprintw(listwin, "%d", sitenum);
	for (i=0; i<PROMPTS; i++) {
		mvwaddstr(listwin, 3+i*2, 1, prompt[i]);
		wclrtoeol(listwin);
		waddstr(listwin, site[sitenum][i]);
	}
	wrefresh(listwin);
}

/* Change download screen button. */

void chdlbutt(int buttnum)
{
	int i;
	int j;
	int horzpos;
	char *dlbutton[DLBUTTS] = {"(N)ext", "(P)revious", "(E)dit", "(R)emove", "(D)ownload", "(A)bort", "(S)ave", "(M)ain"};

	wmove(bottom, 0, 2);
	for (i=0; i<DLBUTTS; i++) {
		if (i == buttnum) {
			for (j=0; j<strlen(dlbutton[i]); j++)
				waddch(bottom, dlbutton[i][j] | A_STANDOUT);
		}
		else
			waddstr(bottom, dlbutton[i]);
		getyx(bottom, j, horzpos);
		wmove(bottom, 0, horzpos+2);
	}
	wrefresh(bottom);
}

/* Read from the child and print status or execute a command.
 * Returns 1 if windows need an update (i.e. they have been altered). */

int getchildstatus()
{
	int fptrb;
	int fptrfile;
	char childmsg[2][128] = {"\0", "\0"};
	char filenumlen[2];
	char *fptrfilenum;
	char readlen[3];

	if (read(dlpipe[0], childmsg[0], 1) == -1) {
		killdl(1); /* if we can't read from the child, we'd better get rid of it */
		return 0;
	}
	switch (atoi(childmsg[0])) {
		case CMD_ABORT:
			killdl(1);
			break;

		case CMD_ERRMSG:
			read(dlpipe[0], readlen, 2);
			readlen[2] = '\0';
			read(dlpipe[0], childmsg[0], atoi(readlen));
			childmsg[0][atoi(readlen)] = '\0';
			if (childmsg[0][strlen(childmsg[0])-1] == '\n') {
				childmsg[0][strlen(childmsg[0])-1] = '\0';
				read(dlpipe[0], readlen, 2);
				read(dlpipe[0], childmsg[1], atoi(readlen));
				childmsg[1][atoi(readlen)] = '\0';
			}
			else
				childmsg[1][0] = '\0';
			bottominfo(childmsg[0], childmsg[1]);
			return 1;

		case STAT_FINISHED:
			wait(NULL);
			bottominfo("Download finished", "");
			dlsite = filesdled = filestodl = bytesdled = bytestodl = 0;
			return 1;

		case STAT_FILECMPLT:
			read(dlpipe[0], filenumlen, 1);
			fptrfilenum = xmalloc(atoi(filenumlen)+1);
			read(dlpipe[0], fptrfilenum, atoi(filenumlen));
			fptrb = atoi(fptrfilenum)/FPTRBLOCKSIZE;
			fptrfile = atoi(fptrfilenum)-fptrb*FPTRBLOCKSIZE;
			fptr[fptrb][fptrfile].type = 4;
			filesdled++;
			bytesdled += sfile[fptr[fptrb][fptrfile].dirnum][(int)fptr[fptrb][fptrfile].category][fptr[fptrb][fptrfile].filenum]->filesize;
			free(fptrfilenum);
			return 1;
	}
	return 0;
}

/* Fork; parent returns, child does the work in the background. */

#define PARENTBYTE(byte) write(dlpipe[1], itoa(byte), 1)

void downloadem(char *url, char *port, char *login, char *passwd, char *dldir)
{
	int fptrb;
	int fptrfile;
	int i;
	int previousdirnum = -1;
	char *urlandport;
	static pid_t childpid;
	pid_t ppid;
	netbuf *server = NULL;

	if (dlsite) {
		bottominfo("Already downloading - wait or abort before starting a new one.", "");
		return;
	}
	if (!taggedfiles) {
		bottominfo("No files tagged for download.", "");
		return;
	}
	if (pipe(dlpipe)) {
		bottominfo("pipe(): ", strerror(errno));
		return;
	}
	if ((childpid = fork()) == -1) {
		bottominfo("fork(): ", strerror(errno));
		return;
	}
	if (childpid) {
		close(dlpipe[1]);
		bytestodl = taggedbytes;
		filestodl = taggedfiles;
		dlsite = -1;
		return;
	}
/* We're the child from here on. */
	close(dlpipe[0]);
	close(2);
	ppid = getppid();
	if (chdir(dldir)) {
		urlandport = xmalloc(strlen(dldir)+7);
		sprintf(urlandport, "%s: %%s\n", dldir);
		parenterrmsg(urlandport, strerror(errno));
		free(urlandport);
		PARENTBYTE(CMD_ABORT);
		exit(1);
	}
	urlandport = xmalloc(strlen(url)+strlen(port)+2);
	sprintf(urlandport, "%s:%s", url, port);
	FtpInit();
	if (!FtpConnect(urlandport, &server)) {
		parenterrmsg("Can't connect to %s\n", urlandport);
		free(urlandport);
		PARENTBYTE(CMD_ABORT);
		exit(1);
	}
	if (!FtpLogin(login, passwd, server)) {
		parenterrmsg("Login failed!", "");
		PARENTBYTE(CMD_ABORT);
		exit(1);
	}
	for (i=1; i<entries-3; i++) {
		fptrb = i/FPTRBLOCKSIZE;
		fptrfile = i-fptrb*FPTRBLOCKSIZE;
		if (fptr[fptrb][fptrfile].type == 3) {
			if (!access(sfile[fptr[fptrb][fptrfile].dirnum][(int)fptr[fptrb][fptrfile].category][fptr[fptrb][fptrfile].filenum]->filename, F_OK))
				parenterrmsg("Won't overwrite existing file %s\n", sfile[fptr[fptrb][fptrfile].dirnum][(int)fptr[fptrb][fptrfile].category][fptr[fptrb][fptrfile].filenum]->filename);
			else {
				if (fptr[fptrb][fptrfile].dirnum != previousdirnum) {
					previousdirnum = fptr[fptrb][fptrfile].dirnum;
					if (!FtpChdir(dircode[fptr[fptrb][fptrfile].dirnum], server)) {
						parenterrmsg("Can't change to remote directory %s\n", dircode[fptr[fptrb][fptrfile].dirnum]);
						PARENTBYTE(CMD_ABORT);
						exit(1);
					}
				}
				if (!FtpGet(sfile[fptr[fptrb][fptrfile].dirnum][(int)fptr[fptrb][fptrfile].category][fptr[fptrb][fptrfile].filenum]->filename, sfile[fptr[fptrb][fptrfile].dirnum][(int)fptr[fptrb][fptrfile].category][fptr[fptrb][fptrfile].filenum]->filename, FTPLIB_IMAGE, server)) {
					parenterrmsg("Can't download remote file %s\n", sfile[fptr[fptrb][fptrfile].dirnum][(int)fptr[fptrb][fptrfile].category][fptr[fptrb][fptrfile].filenum]->filename);
					PARENTBYTE(CMD_ABORT);
					exit(1);
				}
			}
			PARENTBYTE(STAT_FILECMPLT);
			PARENTBYTE(strlen(itoa(i)));
			write(dlpipe[1], itoa(i), strlen(itoa(i)));
		}
	}
	FtpQuit(server);
	PARENTBYTE(STAT_FINISHED);
	exit(0);
}

/* First command code, then message lengths, then the messages.
 * If you want to include another msg in the format of the first
 * one, the last character of the 1st one has to be a '\n'. An
 * abort is also sent, so it needn't be sent separately. */

void parenterrmsg(char *msg1, char *msg2)
{
	PARENTBYTE(CMD_ERRMSG);
	write(dlpipe[1], itoa(strlen(msg1)), 2);
	write(dlpipe[1], msg1, strlen(msg1));
	if (strlen(msg2)) {
		write(dlpipe[1], itoa(strlen(msg2)), 2);
		write(dlpipe[1], msg2, strlen(msg2));
	}
}

/* Kill everyone in the process group except this process. */

void killdl(int quietkill)
{
	if (!dlsite) {
		if (!quietkill)
			bottominfo("No download in progress.", "");
		return;
	}
	signal(SIGTERM, SIG_IGN);
	killpg(0, SIGTERM);
	signal(SIGTERM, SIG_DFL);
	dlsite = filesdled = filestodl = bytesdled = bytestodl = 0;
	wait(NULL);
	if (!quietkill)
		bottominfo("Download aborted.", "");
}

/* Read site list from user-provided conf file (default ~/.fcultrc). */

int loadrcfile(char ***site)
{
	int i;
	int sites = 0;
	char line[MAXLINELEN+1] = "\0";
	FILE *rcfile;

	rcfile = fopen(configfile, "r");
	if (rcfile == NULL)
		return -1;
	do {
		fgets(line, MAXLINELEN, rcfile);
		
		if (*line != '#' && strlen(line) >= 2*PROMPTS-1) { /* commented or too short to be valid */
			site[sites] = xmalloc(PROMPTS*sizeof (char *));
			for (i=0; i<PROMPTS; i++)
				site[sites][i] = xmalloc((MAXLINELEN/4)+1);
			if (sscanf(line, "%s %s %s %s %s\n", site[sites][0], site[sites][1], site[sites][2], site[sites][3], site[sites][4]) == PROMPTS)
				sites++;
			else {
				for (i=0; i<PROMPTS; i++)
					free(site[sites][i]);
				free(site[sites]);
			}
		}
		line[0] = '\0';
	} while (!feof(rcfile) && sites < MAXSITES);
	fclose(rcfile);
	return sites;
}

/* Save site list. Overwrites possibly existing file. */

#define RCFILEHEADER "\
# Configuration file for F-CULT.\n\
\n\
# You can edit this file by hand, but follow these rules; one site per
# line; separate fields with spaces or tabs; empty lines and lines\n\
# starting with '#' ignored. Everything after the last ignored line\n\
# will be recreated every time this file is saved from the browser.\n\
# An example site:\n\
# 208.47.101.9 justme  mypass  /usr/local/download\n\
\n"

int savercfile(char ***site, int sites)
{
	int i;
	int j;
	long curpos;
	char line[MAXLINELEN+1] = "\0";
	FILE *rcfile;

	for (i=0; i<sites; i++) {
		for (j=0; j<PROMPTS; j++) {
			if (!strlen(site[i][j])) {
				bottominfo("Empty field in site %s, can't save", itoa(i));
				return 1;
			}
		}
	}
	if (!access(configfile, F_OK)) { /* exists already? */
		rcfile = fopen(configfile, "r+");
		if (rcfile == NULL) {
			bottominfo(strerror(errno), "");
			return 1;
		}
		curpos = ftell(rcfile);
		while (!feof(rcfile) && (line[0] == '#' || strlen(line) < 2*PROMPTS-1)) {
			curpos = ftell(rcfile);
			fgets(line, MAXLINELEN, rcfile);
		}
		fseek(rcfile, curpos, SEEK_SET);
	}
	else {
		rcfile = fopen(configfile, "w");
		if (rcfile == NULL) {
			bottominfo(strerror(errno), "");
			return 1;
		}
		fputs(RCFILEHEADER, rcfile);
	}
	for (i=0; i<sites; i++)
		fprintf(rcfile, "%s %s %s %s %s\n", site[i][0], site[i][1], site[i][2], site[i][3], site[i][4]);
	fclose(rcfile);
	return 0;
}

#endif
