/*
 * guess.c: Guess which files are missing from a list.
 * 
 * This file is part of F-CULT.
 * 
 * Copyright (C) 1999 Lauri Hstholm
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * To contact Lauri Hstholm, send e-mail to laurijh@iname.com
 */

#include "main.h"

/* Call findseries for each category of each dir. */

int not_main_loop()
{
	int category;
	int d;

	for (d=0; d<cd; d++) {
		if (sfile == NULL)
			continue;
		if (sfile[alphadir[d]] == NULL)
			continue;
		for (category=0; category<CATEGORIES; category++) {
			if (findseries(alphadir[d], category))
				return 1;
		}
	}
	return 0;
}

/* Find files that seem to be of a 'series' and store a
 * complete series based on what's found, e.g. if there's
 * a file08.ext and file16.ext in a dir, files from
 * file09.ext to file15.ext are stored. */

int findseries(int dir, int category)
{
	int f;
	int first = 0;
	int firstdigit;
	int i;
	int last;
	char filename[NAME_MAX+1];
	filestruct *tmpptr;

	if (sfile[dir][category] == NULL)
		return 0;
/* XXX: Do we need this? */
/*	if (!sfile[dir][category][f]->listnum)
		continue;*/
nextseries:
	last = 0;
	f = first;
	if (f >= csf[dir][category]-1)
		return 0;
	while (strcmp(firstpart(sfile[dir][category][f]->filename), firstpart(sfile[dir][category][f+1]->filename)) && f<csf[dir][category]-1)
		f++;
	if (f >= csf[dir][category]-1)
		return 0;
	first = f;
	for (i=first; i<csf[dir][category]; i++) {
		if (sfile[dir][category][i]->listnum) {
			if (is_series(sfile[dir][category][first]->filename, sfile[dir][category][i]->filename))
				last = i;
		}
	}
	if (!last) {
		first++;
		goto nextseries;
	}
	firstdigit = getdigit(sfile[dir][category][first]->filename);
	if (startfrom > -1 && startfrom < firstdigit)
		firstdigit = startfrom;
	else
		firstdigit++;
	tmpptr = sfile[dir][category][last];
	for (i=firstdigit; i<getdigit(tmpptr->filename); i++) {
		strcpy(filename, firstpart(sfile[dir][category][first]->filename));
		strcat(filename, itoa_pad(i, strlen(sfile[dir][category][first]->filename)-(strlen(filename)+strlen(secondpart(sfile[dir][category][first]->filename)))));
		strcat(filename, secondpart(sfile[dir][category][first]->filename));
		if (store_file(filename, sfile[dir][category][first]->dirnum, 0, 0, 0))
			return 1;
	}
	if (first >= csf[dir][category])
		return 0;
	while (is_series(sfile[dir][category][first]->filename, sfile[dir][category][first+1]->filename) && first < csf[dir][category]-2)
		first++;
	first++;
	goto nextseries;
}

/* An itoa that pads the return string with 0's
 * up to width chars (max width=10). */

char *itoa_pad(int i, int width)
{
	int j;
	static char *a;
	char digit[11] = "0123456789";

	if (a == NULL)
		a = malloc(11);
	for (j=9; j>=0; j--) {
		a[j] = digit[i%10];
		i /= 10;
	}
	a[10] = '\0';
	for (j=0; j<10-width; j++) {
		if (a[j] != '0')
			return &a[j];
	}
	return &a[10-width];
}

/* Start from the end, get the last array of digits in
 * string and return it converted to an integer. */

int getdigit(char *string)
{
	int i = strlen(string)-1;
	int dstrptr = 0;
	int retval;
	char digitstr[10];

	while (!isdigit(string[i]))
		i--;
	while (isdigit(string[i]))
		i--;
	i++;
	while (isdigit(string[i]))
		digitstr[dstrptr++] = string[i++];
	digitstr[dstrptr] = '\0';
	retval = atoi(digitstr);
	return retval;
}

/* Return 1 if file f of given category of
 * directory dir is of the same series
 * as the next one (f + 1). */

int is_series(char *file1, char *file2)
{
	char tmpbuf1[NAME_MAX+1];
	char tmpbuf2[NAME_MAX+2];

	strcpy(tmpbuf1, firstpart(file1));
	strcat(tmpbuf1, secondpart(file1));
	strcpy(tmpbuf2, firstpart(file2));
	strcat(tmpbuf2, secondpart(file2));
	if (!strcmp(tmpbuf1, tmpbuf2))
		return 1;
	return 0;
}

/* Return the part of string before the last digit string.
 * The contents of where the returned pointer points to
 * change with every call to this function. */

char *firstpart(char *string)
{
	int i = strlen(string)-1;
	int lastdigit;
	int rsptr = 0;
	static char *retstr;

	if (!retstr)
		retstr = xmalloc(NAME_MAX+1);
	while (!isdigit(string[i]))
		i--;
	if (!i) {
		strcpy(retstr, string);
		return retstr;
	}
	while (isdigit(string[i]))
		i--;
	lastdigit = i+1;
	for (i=0; i<lastdigit; i++)
		retstr[rsptr++] = string[i];
	retstr[rsptr] = '\0';
	return retstr;
}

/* Return the part of string after the last digit string.
 * See the note for firstpart(). */

char *secondpart(char *string)
{
	int i = strlen(string)-1;
	static char *retstr;

	if (!retstr)
		retstr = xmalloc(NAME_MAX+1);
	while (!isdigit(string[i]) && i > -1)
		i--;
	strcpy(retstr, &string[i+1]);
	return retstr;
}
