
/*
  This will be a font displaying program some day 
  
   1998 Roberto Alameda <alameda@ibm.net>
  You may modify and distribute this file under the terms of the GNU
  General Public License, version 2, or any later version, at your
  convenience. See the file COPYING for details. 

*/


#include <config.h>

#if HAVE_UNISTD_H
# include <unistd.h>
# include <sys/types.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <locale.h>
#include <limits.h>
#include <errno.h>
#include <sys/stat.h>

#if HAVE_DIRENT_H
# include <dirent.h>
# define NAMLEN(dirent) strlen((dirent)->d_name)
#else
# define dirent direct
# define NAMLEN(dirent) (dirent)->d_namlen
# if HAVE_SYS_NDIR_H
#  include <sys/ndir.h>
# endif
# if HAVE_SYS_DIR_H
#  include <sys/dir.h>
# endif
# if HAVE_NDIR_H
#  include <ndir.h>
# endif
#endif

#include <gtk/gtk.h>
#include <gdk/gdkx.h>

#include "gfont.h"

#include "mini-ofolder.xpm"
#include "font.xpm"
#include "emptypixmap.xpm"
#include "stipple.xbm"



// Global variables
GdkVisual *visual;
GdkColormap *colormap;
GdkGC *bwGC, *defGC;
GdkColorContext* mCC;
GtkStyle *stylebold;
GdkColor white, black;
GdkCursor* watchcursor;
GdkPixmap* emptypixmap;

GtkWidget* mainwindow;  // The main window widget
GtkWidget* fontlist;    // Font list widget
GtkWidget* dirl;        // Directory name label
GtkWidget* stbar;       // Status bar below
GtkWidget* prbar;       // Progress bar below
GtkWidget* samplet;     // The sample text (a pixmap)
InputDialog *fontsizeD, *characterD, *stringD;

int antialiasing = 0;
int kerning = 0;
int xresolution, yresolution;  // Resolution of the display

char *fontdirectory = DEFAULT_FONTDIR; // Displayed directory

bool enable_antialiasing = TRUE;
GdkColor graypalette[5];  // Colors for antialiasing: white - light - medium - dark - black

TT_Engine ttengine;



// ************** Function prototypes (internally used in this module)
gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data);
Callback newfontdir, leave, showchar, showstring, fonttable, toggle_button;
void gfont_init(int *argc, char*** argv);
void select_fontlist(GtkWidget *widget, gint row, gint column, GdkEventButton *event);
GtkWidget* makefontlist(void);
static void destroyrow(gpointer data);
gint compareEntries(gconstpointer first, gconstpointer second);
void fillfontlist(GtkWidget* clist, char* directory);
void okfontdir(gpointer diag);





// ************** Menubar
static GtkItemFactoryEntry menu_items[] = {  
{ "/_File",          NULL,          NULL,                               0, "<Branch>" },  
{ "/File/Open",      "<control>O",  (GtkItemFactoryCallback)newfontdir,       0, NULL },  
{ "/File/Print",     "<control>P",  (GtkItemFactoryCallback)printfont,        0, NULL }, 
{ "/File/Properties","<control>R",  (GtkItemFactoryCallback)show_properties,  0, NULL }, 
{ "/File/sep1",      NULL,          NULL,                            0, "<Separator>" },  
{ "/File/Quit",      "<control>Q",  (GtkItemFactoryCallback)leave,            0, NULL },  
{ "/_View",          NULL,          NULL,                               0, "<Branch>" },  
{ "/View/Errors",    NULL,          (GtkItemFactoryCallback)show_message_window, 0, NULL },
{ "/_Help",          NULL,          NULL,                           0, "<LastBranch>" },  
{ "/Help/About",     NULL,          (GtkItemFactoryCallback)makeabout,        0, NULL }
};



// ********************* Begin of program functions

#if 0
void info()
{
  Display *dp = GDK_DISPLAY();
  int screen = gdk_screen;

  printf("X%dR%d revision %d\n", ProtocolVersion(dp), XlibSpecificationRelease,
         ProtocolRevision(dp));
  printf("%s, vendor release %d\n", ServerVendor(dp), VendorRelease(dp));

  printf("Display: %s, screen %d, visual id %#lx, colormap id %#lx\n", DisplayString(dp), screen,
         XVisualIDFromVisual(GDK_VISUAL_XVISUAL(visual)), GDK_COLORMAP_XCOLORMAP(colormap));
  printf("Colormap cells: %d\n", colormap->size);
  printf("Depth of visual: %d planes\n", visual->depth);

  printf("Width:%d Height:%d\n", DisplayWidth(dp,screen), DisplayHeight(dp,screen));
  printf("WidthMM:%d HeightMM:%d\n", DisplayWidthMM(dp,screen), DisplayHeightMM(dp,screen));
  printf("Default root window id of display: %#lx\n", DefaultRootWindow(dp));

  printf("X %s support locale %s\n", (XSupportsLocale()==True)?"does":"does not",
         setlocale(LC_ALL, NULL));
  printf("Locale modifiers:%s\n", XSetLocaleModifiers(NULL));
}



void dumpGC(GdkGC* gc)
{
  XGCValues values;
  GC xgc = GDK_GC_XGC(gc);
  
  XGetGCValues(GDK_DISPLAY(), xgc, GCForeground | GCBackground, &values);
  printf("Foreground: %ld, Background:%ld\n", values.foreground, values.background);
}
#endif




void gfont_init(int *argc, char*** argv)
{
  // Init GTK and X11 stuff
  gtk_init(argc, argv);
  visual = gdk_visual_get_system();
  colormap = gdk_colormap_get_system();
  mCC = gdk_color_context_new(visual, colormap);
  if (visual->depth<8 || visual->depth==15) enable_antialiasing = FALSE;

  gdk_color_black(colormap, &black);
  gdk_color_white(colormap, &white);

  // Calculate screen resolution in dots per inch
  xresolution = (int)(0.5 + gdk_screen_width() / (gdk_screen_width_mm()/25.4));
  yresolution = (int)(0.5 + gdk_screen_height() / (gdk_screen_height_mm()/25.4));

  GtkStyle *style = gtk_widget_get_default_style();
  gdk_font_unref(style->font);
  style->font = gdk_font_load("-adobe-helvetica-medium-r-*-*-*-120-*-*-p-*-*-*");
  if (!style->font) style->font = gdk_font_load("fixed");

  stylebold = gtk_style_new();
  gdk_font_unref(stylebold->font);
  stylebold->font = gdk_font_load("-adobe-helvetica-bold-r-*-*-*-120-*-*-p-*-*-*");
  if (!stylebold->font) stylebold->font = gdk_font_load("fixed");

  watchcursor = gdk_cursor_new(GDK_WATCH);

  // Fill graypalette[] for antialiasing
  if (enable_antialiasing) {
    char *list[] = {"white", "gray75", "gray50", "gray25", "black"};
    for (guint i=0; i<sizeof(list)/sizeof(char*); i++) {
      int failed = 1;
      if (gdk_color_parse(list[i], &graypalette[i]))
	graypalette[i].pixel = gdk_color_context_get_pixel(mCC, graypalette[i].red, graypalette[i].green, graypalette[i].blue, &failed);
      if (failed) graypalette[i].pixel = black.pixel;
    }
  }

  // Init T1lib
  T1_SetLogLevel(T1LOG_STATISTIC);
  T1_SetBitmapPad(16);
  T1_SetDeviceResolutions(xresolution, yresolution);
  if (T1_InitLib(NO_LOGFILE | IGNORE_FONTDATABASE | IGNORE_CONFIGFILE)==NULL){
    fprintf(stderr, "Initialization of T1-library failed\n");
    exit(1); 
  }
  if (enable_antialiasing) {
    if (T1_AASetBitsPerPixel(visual->depth) < 0) {
      fprintf(stderr, "Cannot set T1-library antialiasing depth to %d: %s\n", 
	      visual->depth, GetT1error(T1_errno));
      exit(1);
    }
    T1_AASetLevel(T1_AA_LOW);
    T1_AASetGrayValues(graypalette[0].pixel, graypalette[1].pixel, graypalette[2].pixel, graypalette[3].pixel, graypalette[4].pixel); 
  }
  
  // Init FreeType
  int error = TT_Init_FreeType(&ttengine);
  if (error) {
    fprintf(stderr, "Could not create TT engine instance\n");
    exit(1);
  }
  error = TT_Init_Kerning_Extension(ttengine);
  if (error) {
    fprintf(stderr, "Could not init the kerning extension\n");
    exit(1);
  }
  error = TT_Init_Post_Extension(ttengine);
  if (error) {
    fprintf(stderr, "Could not init the PostScript extension\n");
    exit(1);
  }
  if (enable_antialiasing) {
    // Gray palette: the values are indexes into graypalette[]
    guchar palette[5] = {0, 1, 2, 3, 4};  // white - light - medium - dark - black
    TT_Set_Raster_Gray_Palette(ttengine, palette);
  }
}




InputDialog::InputDialog(const char* labeltext, const char* entrytext)
{
  hbox = gtk_hbox_new(FALSE, 5);
  gtk_container_border_width(GTK_CONTAINER(hbox), 0);

  label = gtk_label_new(labeltext);
  gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
  gtk_widget_set_style(label, stylebold);

  entry = gtk_entry_new();
  gtk_widget_set_usize(entry, 50, -1);
  gtk_entry_set_text(GTK_ENTRY(entry), entrytext);
  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 0);
}


void delete_image(GtkWidget*, gpointer data)
{
  GdkImage *image = (GdkImage*)data;
  gdk_image_destroy(image);
}


void delete_pixmap(GtkWidget*, gpointer data)  // destroy callback
{
  GdkPixmap *pixmap = (GdkPixmap*)data;
  gdk_pixmap_unref(pixmap);
}



gint expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
  GdkPixmap *pixmap = (GdkPixmap*)data;
  //printf("In the expose event, pixmap:%lx\n", pixmap);
  gdk_draw_pixmap(widget->window, bwGC, pixmap,
		  event->area.x, event->area.y,
		  event->area.x, event->area.y,
		  event->area.width, event->area.height);
  return FALSE;
}




void showchar(GtkWidget *button, gpointer list)
{
  char *invl;

  GList *selection = GTK_CLIST(list)->selection;
  if (selection == NULL) {
    errormsg("No row selected");
    return;
  }
  double size = atof(fontsizeD->getText());
  if (size<1.0 || size>1000.0) {
    errormsg("Invalid size");
    return;
  }
  const char *chr = characterD->getText();
  long rc = strtol(chr, &invl, 0);
  if (*chr=='\0' || *invl!='\0' || rc==LONG_MIN || rc==LONG_MAX || rc<0 || rc>0xFFFF) {
    errormsg("Invalid character");
    return;
  }

  gdk_window_set_cursor(mainwindow->window, watchcursor);
  while (gtk_events_pending()) gtk_main_iteration();

  int row = GPOINTER_TO_INT(selection->data);
  FontData* fd = (FontData*)gtk_clist_get_row_data(GTK_CLIST(list), row);
  switch (fd->fontType) {
  case T1FONT:
    t1_showchar(fd, (guint)rc, size);
    break;
  case TTFONT: 
    tt_showchar(fd, (guint)rc, size);
    break;
  }
  gdk_window_set_cursor(mainwindow->window, NULL);
}



void fonttable(GtkWidget *button, gpointer list)
{
  GList *selection = GTK_CLIST(list)->selection;
  if (selection == NULL) {
    errormsg("No row selected");
    return;
  }
  double size = atof(fontsizeD->getText());
  if (size<1.0 || size>1000.0) {
    errormsg("Invalid size");
    return;
  }
  int row = GPOINTER_TO_INT(selection->data);
  FontData* fd = (FontData*)gtk_clist_get_row_data(GTK_CLIST(list), row);

  gdk_window_set_cursor(mainwindow->window, watchcursor);
  gtk_statusbar_push(GTK_STATUSBAR(stbar), 1, "Making font table...");
  while (gtk_events_pending()) gtk_main_iteration();
  switch (fd->fontType) {
  case T1FONT:
    t1_fonttable(fd, size);
    break;
  case TTFONT: 
    tt_fonttable(fd, size);
    break;
  }
  gtk_statusbar_pop(GTK_STATUSBAR(stbar), 1);
  gdk_window_set_cursor(mainwindow->window, NULL);
}




void showstring(GtkWidget *button, gpointer list)
{
  GList *selection = GTK_CLIST(list)->selection;
  if (selection == NULL) {
    errormsg("No row selected");
    return;
  }
  const char* string = stringD->getText();
  if (string==NULL || *string=='\0') string = "Test";
  double size = atof(fontsizeD->getText());
  if (size<1.0 || size>1000.0) {
    errormsg("Invalid size");
    return;
  }
  gdk_window_set_cursor(mainwindow->window, watchcursor);
  while (gtk_events_pending()) gtk_main_iteration();

  int row = GPOINTER_TO_INT(selection->data);
  FontData* fd = (FontData*)gtk_clist_get_row_data(GTK_CLIST(list), row);
  switch(fd->fontType) {
  case T1FONT:
    t1_showstring(fd, string, size);
    break;
  case TTFONT: 
    tt_showstring(fd, string, size);
    break;
  }

  gdk_window_set_cursor(mainwindow->window, NULL);
}



void toggle_button(GtkWidget*, gpointer data)
{
  int *variable =(int*)data;
  *variable = !*variable;  // Invert state of variable
}





void select_fontlist(GtkWidget *widget, gint row, gint column, GdkEventButton *event)
{
  //printf("GtkCList Selection: row %d column %d button %d\n", row, column, event?event->button:0);
  FontData* fd = (FontData*)gtk_clist_get_row_data(GTK_CLIST(widget), row);
  // fillfontlist will generate 1 select event before the data is there
  if (fd == NULL) return;  // So check for it
  switch(fd->fontType) {
  case T1FONT: 
    gdk_window_set_cursor(mainwindow->window, watchcursor);
    while (gtk_events_pending()) gtk_main_iteration();
    t1_drawsample(fd, "the quick brown fox jumps over the lazy dog", 18.0);
    gdk_window_set_cursor(mainwindow->window, NULL);
    break;
  case TTFONT:
    tt_drawsample(fd, "the quick brown fox jumps over the lazy dog", 18.0);
    break;
  }
  /* 
     The next 2 lines are a workaround for a clist bug present in GTK+ 1.2.2:
     if a modal window (an error message) is displayed within the 
     select_row event, the pointer is grabbed and the X-Server 
     does not respond to mouse events
   */
  gtk_grab_remove(widget);
  if (gdk_pointer_is_grabbed()) gdk_pointer_ungrab(GDK_CURRENT_TIME);
}




GtkWidget* makefontlist(void)
{
  char* titles[] = {"File name", "Glyphs", "Font name", "Kern pairs"};

  GtkWidget* scwindow = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scwindow), GTK_POLICY_ALWAYS, 
				 GTK_POLICY_AUTOMATIC);

  fontlist = gtk_clist_new_with_titles(sizeof(titles)/sizeof(char*), titles);
  gtk_clist_column_titles_passive(GTK_CLIST(fontlist));
  gtk_clist_set_column_width(GTK_CLIST(fontlist), 0, 100);
  gtk_clist_set_column_width(GTK_CLIST(fontlist), 1, 50);
  gtk_clist_set_column_width(GTK_CLIST(fontlist), 2, 180);
  gtk_clist_set_selection_mode(GTK_CLIST(fontlist), GTK_SELECTION_BROWSE);
  gtk_clist_set_shadow_type(GTK_CLIST(fontlist), GTK_SHADOW_ETCHED_OUT);
  gtk_container_border_width(GTK_CONTAINER(fontlist), 0);
  gtk_signal_connect(GTK_OBJECT(fontlist), "select_row", 
		     (GtkSignalFunc)select_fontlist, NULL);

  gtk_widget_set_usize(fontlist, -1, 200);
  gtk_container_add(GTK_CONTAINER(scwindow), fontlist);
  return scwindow;
}



// Destroy row in clist fontlist; called when displaying a new directory
// to free all structures related to fonts in the old directory
static void destroyrow(gpointer data)
{
  FontData *fd = (FontData*)data;
  fd->visible = FALSE; // No longer being displayed in list
  destroyfont(fd);
}


// Try to destroy a font if unused: free all memory related to its FontData
void destroyfont(FontData *fd)
{
  if (fd->refcount > 0) return;  // Still in use

  switch(fd->fontType) {
  case T1FONT: 
    T1_DeleteFont(fd->t1data.fontID);
    // If encoding is 'Custom x', free all components of encoding vector
    if (fd->t1data.encoding && 
	strncmp(fd->t1data.encoding[256], "Custom", 6) == 0) {
      for (int i=0; i<=256; i++) g_free(fd->t1data.encoding[i]);
      g_free(fd->t1data.encoding);
    }
    break;
  case TTFONT:
    TT_Close_Face(fd->ttdata.face);  
    break;
  }
  g_free(fd->fontName);
  g_free(fd->fontFile);
  g_free(fd);
}




gint compareEntries(gconstpointer first, gconstpointer second)
{
  FontData* fd1 = (FontData*)first;
  FontData* fd2 = (FontData*)second;

  return(strcmp(fd1->fontName, fd2->fontName));
}



void fillfontlist(GtkWidget* clist, char* directory)
{
  GSList *rowlist=NULL;
  struct dirent *entry;
  struct stat buf;
  int nfiles=0, readfiles=0;

  GString* fullname = g_string_new("");
  gtk_clist_clear(GTK_CLIST(clist));
  if (T1_AddToFileSearchPath(T1_PFAB_PATH | T1_AFM_PATH, 
			     T1_PREPEND_PATH, directory)==-1) {
    errormsg("Cannot add directory %s to T1 path: %s", directory, 
	     GetT1error(T1_errno));
    g_string_free(fullname, TRUE);
    return;
  }
  g_string_sprintf(fullname, "%s/afm", directory);
  if (T1_AddToFileSearchPath(T1_AFM_PATH, T1_PREPEND_PATH, fullname->str)==-1) {
    errormsg("Cannot add directory %s to T1 afm path: %s", fullname->str, 
	     GetT1error(T1_errno));
    g_string_free(fullname, TRUE);
    return;
  }
  // Calculate number of entries in directory
  DIR* dir = opendir(directory);
  if (dir == NULL) {
    errormsg("Cannot open dir %s: %s", directory, g_strerror(errno));
    return;
  }
  gdk_window_set_cursor(mainwindow->window, watchcursor);
  gtk_statusbar_push(GTK_STATUSBAR(stbar), 1, "Reading files...");
  while (gtk_events_pending()) gtk_main_iteration();
  while((entry=readdir(dir))) nfiles++;
  rewinddir(dir);

  // Read all files in directory and create a sorted linked list with the valid ones
  while((entry=readdir(dir))) {
    int fontID;
    char *fontname;

    gtk_progress_bar_update(GTK_PROGRESS_BAR(prbar), double(++readfiles)/nfiles);
    while (gtk_events_pending()) gtk_main_iteration();

    g_string_sprintf(fullname, "%s/%s", directory, entry->d_name);
    if (stat(fullname->str, &buf)==0 && S_ISREG(buf.st_mode)) {
      FontData *fontdesc = g_new(FontData, 1);
      fontdesc->refcount = 0;
      fontdesc->visible = TRUE;
      fontdesc->fontFile = g_strdup(entry->d_name);  // Must be there for the error messages
      fontdesc->fontName = NULL;
      // Either T1
      if (is_t1font(fullname->str, fontdesc)) {
	rowlist = g_slist_insert_sorted(rowlist, fontdesc, compareEntries);
	continue;
      }
      // Or TT
      if (is_ttfont(fullname->str, fontdesc)) {
	rowlist = g_slist_insert_sorted(rowlist, fontdesc, compareEntries);
	continue;
      }
      // Or TT font collection
      if (is_ttfontcollection(fullname->str, fontdesc)) {
	int num = fontdesc->ttdata.properties.num_Faces;  // Number of fonts in collection
	TT_Close_Face(fontdesc->ttdata.face);
	g_free(fontdesc);
	for (int i=0; i<num; i++) {  // Make a list entry for each font in collection
	  FontData *fd = g_new(FontData, 1);
	  fd->fontFile = g_strdup(entry->d_name);
	  int error = TT_Open_Collection(ttengine, fullname->str, i, &fd->ttdata.face);
	  if (error) {
	    add_error(fd, "Cannot open font number %d in collection", i);
	    g_free(fd->fontFile);
	    g_free(fd);
	    continue;
	  }
	  fill_ttfontdata(fd);
	  rowlist = g_slist_insert_sorted(rowlist, (gpointer)fd, compareEntries);
	}
	continue;
      }
      // Or unknown; so forget it
      g_free(fontdesc->fontFile);
      g_free(fontdesc);
    }
  }
  g_string_free(fullname, TRUE);
  closedir(dir);
  gtk_progress_bar_update(GTK_PROGRESS_BAR(prbar), 0.0);
  gtk_statusbar_pop(GTK_STATUSBAR(stbar), 1);
  while (gtk_events_pending()) gtk_main_iteration();

  // Put the linked list in the list widget
  gtk_clist_freeze(GTK_CLIST(clist));
  gtk_statusbar_push(GTK_STATUSBAR(stbar), 1, "Making entries...");
  while (gtk_events_pending()) gtk_main_iteration();
  int row = 0;
  for (GSList* p=rowlist; p; p=p->next, row++) {
    gchar *rowentries[4];
    char buf[24], buf2[24];
    FontData* fd = (FontData*)p->data;
    rowentries[0] = fd->fontFile;
    rowentries[2] = fd->fontName;
    if (fd->num_glyphs >= 0)
      sprintf(buf, "%u", fd->num_glyphs);
    else 
      strcpy(buf, "---");
    rowentries[1] = buf;
    if (fd->fontType == TTFONT) 
      sprintf(buf2, "%u", fd->ttdata.kernpairs);
    else
      strcpy(buf2, "---");
    rowentries[3] = buf2;

    gtk_clist_append(GTK_CLIST(clist), rowentries);
    gtk_clist_set_row_data_full(GTK_CLIST(clist), row, p->data, destroyrow);  // Each row has a FontData*
  }
  gtk_statusbar_pop(GTK_STATUSBAR(stbar), 1);
  gtk_clist_thaw(GTK_CLIST(clist));
  if (row > 0) gtk_clist_select_row(GTK_CLIST(clist), 0, -1); // Select first row

  g_slist_free(rowlist);
  gtk_label_set(GTK_LABEL(dirl), directory);
  gdk_window_set_cursor(mainwindow->window, NULL);
}


// The user selected to display a new directory
void okfontdir(gpointer diag)
{
  GtkWidget* filesel = GTK_WIDGET(diag);
  char *newdir = gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel));
  fontdirectory = g_strdup(newdir);  // Not freed; not a big deal, however
  gtk_widget_destroy(filesel);
  gdk_window_clear(samplet->window);  // Clear old font sample
  gtk_pixmap_set(GTK_PIXMAP(samplet), emptypixmap, NULL);  

  fillfontlist(fontlist, fontdirectory);
}



// Selection window to choose a new directory
void newfontdir(GtkWidget*, gpointer)
{
  gdk_window_set_cursor(mainwindow->window, watchcursor);
  while (gtk_events_pending()) gtk_main_iteration();

  GtkWidget* filesel = gtk_file_selection_new("Font Viewer directory selection");
  gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(filesel));
  gtk_widget_set_sensitive(GTK_FILE_SELECTION(filesel)->file_list, FALSE);
  gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filesel)->cancel_button),
			    "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), 
			    GTK_OBJECT(filesel));
  gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filesel)->ok_button),
			    "clicked", GTK_SIGNAL_FUNC(okfontdir), GTK_OBJECT(filesel));
  gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel), fontdirectory);
  gtk_window_position(GTK_WINDOW(filesel), GTK_WIN_POS_CENTER);
  gtk_window_set_modal(GTK_WINDOW(filesel), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(filesel), GTK_WINDOW(mainwindow));
  gtk_widget_show(filesel);
  gdk_window_set_cursor(mainwindow->window, NULL);
}




gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  gtk_statusbar_push(GTK_STATUSBAR(stbar), 1, "Exiting...");
  while (gtk_events_pending()) gtk_main_iteration();
  return FALSE; // Send a destroy event
}


// Exit the program
void leave(GtkWidget*, gpointer data)
{
  T1_CloseLib();
  TT_Done_FreeType(ttengine);
  gtk_exit(0);
}




int main(int argc, char** argv)
{
  gfont_init(&argc, &argv);

  // Check for filename or directory as rest argument
  if (argc > 1) {
    struct stat buf;
    char *givenarg = argv[1];        
    if (stat(givenarg, &buf) != 0) {
      fprintf(stderr, "Cannot find %s: %s\n", givenarg, g_strerror(errno));
      exit(1);
    }
    if (S_ISDIR(buf.st_mode)) {  // If a directory, display it
      fontdirectory = givenarg;
    }
    if (S_ISREG(buf.st_mode)) {  // If a file, display its directory
      fontdirectory = g_dirname(givenarg);
    }
  }

  //******** Main window
  mainwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_container_border_width(GTK_CONTAINER(mainwindow), 0);
  gtk_window_set_title(GTK_WINDOW(mainwindow), "Font Viewer");
  gtk_window_set_policy(GTK_WINDOW(mainwindow), FALSE, TRUE, FALSE); /* shrink, grow, auto_shrink */
  gtk_widget_realize(mainwindow);	 
					 
  gtk_signal_connect(GTK_OBJECT(mainwindow), "destroy", GTK_SIGNAL_FUNC(leave), NULL);
  gtk_signal_connect(GTK_OBJECT(mainwindow), "delete_event", GTK_SIGNAL_FUNC(delete_event), NULL);
  
  // Global vbox
  GtkWidget *vboxglobal = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(mainwindow), vboxglobal);

  // Menu vbox
  GtkWidget *vboxmenu = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vboxglobal), vboxmenu, FALSE, FALSE, 0);

  // Main vertical container
  GtkWidget* vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_border_width(GTK_CONTAINER(vbox), 10);
  gtk_box_pack_start(GTK_BOX(vboxglobal), vbox, TRUE, TRUE, 0);


  // Menubar
  GtkAccelGroup *accel_group = gtk_accel_group_new();
  GtkItemFactory *item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, 
						      "<main>", accel_group);
  gint nmenu_items = sizeof(menu_items) / sizeof (menu_items[0]);
  gtk_item_factory_create_items(item_factory, nmenu_items, menu_items, NULL);
  GtkWidget *menubar = gtk_item_factory_get_widget(item_factory, "<main>");
  gtk_accel_group_attach(accel_group, GTK_OBJECT(mainwindow));
  gtk_box_pack_start(GTK_BOX(vboxmenu), menubar, FALSE, FALSE, 0);


  //****** Current directory
  GtkWidget* hboxl = gtk_hbox_new(FALSE, 10);
  gtk_box_pack_start(GTK_BOX(vbox), hboxl, FALSE, FALSE, 5);

  GtkWidget* dirb = gtk_button_new();
  GtkWidget* icondir = makeicon(mainwindow, miniofolder);
  gtk_container_add(GTK_CONTAINER(dirb), icondir);
  gtk_signal_connect(GTK_OBJECT(dirb), "clicked", GTK_SIGNAL_FUNC(newfontdir), NULL);
  gtk_box_pack_start(GTK_BOX(hboxl), dirb, FALSE, FALSE, 0);

  dirl = gtk_label_new("");
  gtk_box_pack_start(GTK_BOX(hboxl), dirl, FALSE, FALSE, 0);

  GtkWidget* flist = makefontlist();
  gtk_box_pack_start(GTK_BOX(vbox), flist, TRUE, TRUE, 0);

  //******** Sample text
  // Use a viewport to clip pixmap
  GtkWidget* viewport = gtk_viewport_new(NULL, NULL);
  gtk_widget_set_usize(viewport, -1, 35);
  gtk_viewport_set_shadow_type(GTK_VIEWPORT(viewport), GTK_SHADOW_IN);
  gtk_box_pack_start(GTK_BOX(vbox), viewport, FALSE, FALSE, 0);
  // This prevents the size requests of the viewport from propagating up to the toplevel
  gtk_container_set_resize_mode(GTK_CONTAINER(viewport->parent), GTK_RESIZE_QUEUE);

  GdkBitmap* mask;
  GtkStyle* style = gtk_widget_get_style(mainwindow);
  emptypixmap = gdk_pixmap_create_from_xpm_d(mainwindow->window, &mask,
					     &style->bg[GTK_STATE_NORMAL], 
					     emptypixmap_d);
  samplet = gtk_pixmap_new(emptypixmap, mask);
  gtk_container_add(GTK_CONTAINER(viewport), samplet);
  
     
  //******** Input entries
  fontsizeD = new InputDialog("Font Size:", "25");
  gtk_box_pack_start(GTK_BOX(vbox), fontsizeD->getWidget(), FALSE, FALSE, 0);

  GtkWidget* hboxe = gtk_hbox_new(TRUE, 10);
  gtk_container_border_width(GTK_CONTAINER(hboxe), 0);
  gtk_box_pack_start(GTK_BOX(vbox), hboxe, FALSE, FALSE, 0);

  characterD = new InputDialog("Character:", "65");
  gtk_box_pack_start(GTK_BOX(hboxe), characterD->getWidget(), TRUE, TRUE, 0);

  stringD = new InputDialog("Test String:", "Test");
  stringD->setEntryWidth(100);
  gtk_box_pack_start(GTK_BOX(hboxe), stringD->getWidget(), TRUE, TRUE, 0);


  //********** Toggle buttons
  GtkWidget* hboxt = gtk_hbox_new(TRUE, 0);
  gtk_container_border_width(GTK_CONTAINER(hboxt), 3);
  gtk_box_pack_start(GTK_BOX(vbox), hboxt, FALSE, FALSE, 0);

  GtkWidget* aabutton = gtk_check_button_new_with_label("Antialias");
  gtk_box_pack_start(GTK_BOX(hboxt), aabutton, TRUE, FALSE, 0);
  gtk_signal_connect(GTK_OBJECT(aabutton), "toggled", GTK_SIGNAL_FUNC(toggle_button), &antialiasing);
  if (!enable_antialiasing) gtk_widget_set_sensitive(aabutton, FALSE);

  GtkWidget* kernbutton = gtk_check_button_new_with_label("Kerning");
  gtk_box_pack_start(GTK_BOX(hboxt), kernbutton, TRUE, FALSE, 0);
  gtk_signal_connect(GTK_OBJECT(kernbutton), "toggled", GTK_SIGNAL_FUNC(toggle_button), &kerning);


  //********** Separator
  GtkWidget* sep1 = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(vbox), sep1, FALSE, FALSE, 0);


  //******** Action buttons
  GtkWidget* hboxa = gtk_hbox_new(TRUE, 0);
  gtk_container_border_width(GTK_CONTAINER(hboxa), 0);
  gtk_box_pack_start(GTK_BOX(vbox), hboxa, FALSE, FALSE, 0);
  GtkWidget* charb = gtk_button_new_with_label("Make character");
  gtk_signal_connect(GTK_OBJECT(charb), "clicked", GTK_SIGNAL_FUNC(showchar), fontlist);
  gtk_box_pack_start(GTK_BOX(hboxa), charb, TRUE, TRUE, 0);
  GtkWidget* stringb = gtk_button_new_with_label("Make string");
  gtk_signal_connect(GTK_OBJECT(stringb), "clicked", GTK_SIGNAL_FUNC(showstring), fontlist);
  gtk_box_pack_start(GTK_BOX(hboxa), stringb, TRUE, TRUE, 0);
  GtkWidget* tableb = gtk_button_new_with_label("Make font table");
  gtk_signal_connect(GTK_OBJECT(tableb), "clicked", GTK_SIGNAL_FUNC(fonttable), fontlist);
  gtk_box_pack_start(GTK_BOX(hboxa), tableb, TRUE, TRUE, 0);
  

  //********** Message and progress area  -- Last in window
  GtkWidget* hboxm = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_end(GTK_BOX(vbox), hboxm, FALSE, FALSE, 0);
  stbar = gtk_statusbar_new();
  gtk_box_pack_start(GTK_BOX(hboxm), stbar, TRUE, TRUE, 0);
  prbar = gtk_progress_bar_new();
  gtk_box_pack_end(GTK_BOX(hboxm), prbar, FALSE, FALSE, 0);
  gtk_widget_set_usize(prbar, 100, -1);

  
  //********** Final steps
  gtk_widget_show_all(mainwindow);
 
  // define GCs 
  
  // bwGC: black & white GC, with stippled fill pattern
  GdkGCValues gcval;
  int gcmask = GDK_GC_FUNCTION | GDK_GC_FOREGROUND | GDK_GC_BACKGROUND | 
    GDK_GC_FILL | GDK_GC_STIPPLE;

  gcval.function = GDK_COPY;
  gcval.foreground.pixel = black.pixel;
  gcval.background.pixel = white.pixel;
  gcval.fill = GDK_STIPPLED;
  gcval.stipple = gdk_bitmap_create_from_data(mainwindow->window, stipple_bits, 
					      stipple_width, stipple_height);
  bwGC = gdk_gc_new_with_values(mainwindow->window, &gcval, (GdkGCValuesMask)gcmask);
   
  // defGC: fg is black, bg is the default window bg (possibly gray)
  defGC = gdk_gc_new(mainwindow->window); 
  gdk_gc_set_foreground(defGC, &black);
  gdk_gc_set_background(defGC, &style->bg[GTK_STATE_NORMAL]);
  // Set the bg colour of black_gc in the default style to white
  gdk_gc_set_background(style->black_gc, &white);

  GtkWidget *windowicon = makeicon(mainwindow, font_xpm);
  gdk_window_set_icon(mainwindow->window, NULL, GTK_PIXMAP(windowicon)->pixmap, NULL);

  make_message_window();
  while (gtk_events_pending()) gtk_main_iteration();
  fillfontlist(fontlist, fontdirectory);

  gtk_main();
  return 1;  // Should not reach
}
