/* Copyright (C) 1999 Southern Gold Development <tangomanrulz@geocities.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "gview.h"

gint nextSlide(gpointer data) {
	sgview *view;
	GtkWidget *label;

	view = (sgview *) data;
	gtk_timeout_remove(view->timeout);
	if (!(view->slide))
		return FALSE;
	view->currSlide = view->currSlide->next;
	if (view->currSlide) {
		loadPic(view, (gchar *) view->currSlide->data);
		view->timeout = gtk_timeout_add(view->slideTime, (GtkFunction) nextSlide, (gpointer) data);
	}
	else {
		view->statCh = (view->statCh | 0x02);
		label = GTK_BUTTON (view->slideBtn)->child;
		gtk_signal_disconnect(GTK_OBJECT (view->slideBtn), view->sigSlide);
		view->sigSlide = gtk_signal_connect (GTK_OBJECT (view->slideBtn), "clicked", (GtkSignalFunc) startSlide, (gpointer) view);
		gtk_label_set_text (GTK_LABEL (label), _("SlideShow"));
	}
	return FALSE;
}

void stopSlide(GtkWidget *widget, gpointer data) {
	sgview *view;
	GtkWidget *label;

	view = (sgview *) data;
	
	if (view->currSlide) {
		view->currSlide = NULL;
		gtk_timeout_remove(view->timeout);
		view->statCh = (view->statCh | 0x02);
		label = GTK_BUTTON (view->slideBtn)->child;
		if (gtk_signal_n_emissions_by_name(GTK_OBJECT (view->slideBtn), "clicked") != 0)
			gtk_signal_emit_stop_by_name(GTK_OBJECT (view->slideBtn), "clicked");
		gtk_signal_disconnect(GTK_OBJECT (view->slideBtn), view->sigSlide);
		view->sigSlide = gtk_signal_connect (GTK_OBJECT (view->slideBtn), "clicked", (GtkSignalFunc) startSlide, (gpointer) view);
		gtk_label_set_text (GTK_LABEL (label), _("SlideShow"));
	}
}

void pauseSlide(GtkWidget *widget, gpointer data) {
	sgview *view;
	GtkWidget *label;

	view = (sgview *) data;
	
	if (view->currSlide) {
		gtk_timeout_remove(view->timeout);
		view->statCh = (view->statCh | 0x02);
		label = GTK_BUTTON (view->slideBtn)->child;
		if (gtk_signal_n_emissions_by_name(GTK_OBJECT (view->slideBtn), "clicked") != 0)
			gtk_signal_emit_stop_by_name(GTK_OBJECT (view->slideBtn), "clicked");
		gtk_signal_disconnect(GTK_OBJECT (view->slideBtn), view->sigSlide);
		view->sigSlide = gtk_signal_connect (GTK_OBJECT (view->slideBtn), "clicked", (GtkSignalFunc) startSlide, (gpointer) view);
		gtk_label_set_text (GTK_LABEL (label), _("SlideShow"));
	}
}

void resumeSlide(GtkWidget *widget, gpointer data) {
	sgview *view;
	GtkWidget *label;

	view = (sgview *) data;
	if (!(view->slide))
		return;
	if (!(view->currSlide))
		return;
	view->statCh = (view->statCh & 0xfd);
	loadPic(view, (gchar *) view->currSlide->data);
	label = GTK_BUTTON (view->slideBtn)->child;
	gtk_label_set_text(GTK_LABEL (label), _("Stop"));
	if (gtk_signal_n_emissions_by_name(GTK_OBJECT (view->slideBtn), "clicked") != 0)
		gtk_signal_emit_stop_by_name(GTK_OBJECT (view->slideBtn), "clicked");
	gtk_signal_disconnect(GTK_OBJECT (view->slideBtn), view->sigSlide);
	view->sigSlide = gtk_signal_connect(GTK_OBJECT (view->slideBtn), "clicked", (GtkSignalFunc) stopSlide, (gpointer) view);
	view->timeout = gtk_timeout_add(view->slideTime, (GtkFunction) nextSlide, (gpointer) data);
}

void clearSlide(GtkWidget *widget, gpointer data) {
	sgview *view;
	GList *tmp;

	view = (sgview *) data;
	if (!(view->statCh & 0x02))
		return;
	if (view->slide) {
		tmp = view->slide;
		while (tmp) {
			g_free(tmp->data);
			tmp = tmp->next;
		}
		g_list_free(tmp);
		view->slide = NULL;
		view->currSlide = NULL;
	}
	view->currSlide = NULL;
}

void startSlide(GtkWidget *widget, gpointer data) {
	sgview *view;
	GtkWidget *label;

	view = (sgview *) data;
	if (!(view->slide))
		return;
	view->currSlide = view->slide;
	view->statCh = (view->statCh & 0xfd);
	loadPic(view, (gchar *) view->currSlide->data);
	label = GTK_BUTTON (view->slideBtn)->child;
	gtk_label_set_text(GTK_LABEL (label), _("Stop"));
	if (gtk_signal_n_emissions_by_name(GTK_OBJECT (view->slideBtn), "clicked") != 0)
		gtk_signal_emit_stop_by_name(GTK_OBJECT (view->slideBtn), "clicked");
	gtk_signal_disconnect(GTK_OBJECT (view->slideBtn), view->sigSlide);
	view->sigSlide = gtk_signal_connect(GTK_OBJECT (view->slideBtn), "clicked", (GtkSignalFunc) stopSlide, (gpointer) view);
	view->timeout = gtk_timeout_add(view->slideTime, (GtkFunction) nextSlide, (gpointer) data);
}

void enqueueSlide(GtkWidget *widget, gpointer data) {
	sgview *view;
	gchar *file, *fPath;
	GList *selList;
	
	view = (sgview *) data;
	if (GTK_CLIST (view->clist)->focus_row == -1)
		return;
	if (GTK_CLIST (view->clist)->selection) {
		selList = GTK_CLIST (view->clist)->selection;
		while (selList) {
			gtk_clist_get_text(GTK_CLIST (view->clist), (int) selList->data, 0, &file);
			if ((strcmp(file, _(".. (parent)")) != 0) && (file[strlen(file) - 1] != '/')) {
				fPath = g_strconcat(view->currPath, file, NULL);
				if (view->slide) {
					g_list_append(view->slide, (gpointer) fPath);
				}
				else {
					view->slide = g_list_alloc();
					view->slide->data = (gpointer) fPath;
				}
			}
			selList = selList->next;
		}
	}
	else {
		gtk_clist_get_text(GTK_CLIST (view->clist), GTK_CLIST(view->clist)->focus_row, 0, &file);
		if ((strcmp(file, _(".. (parent)")) != 0) && (file[strlen(file) - 1] != '/')) {
			fPath = g_strconcat(view->currPath, file, NULL);
			if (view->slide) {
				g_list_append(view->slide, (gpointer) fPath);
			}
			else {
				view->slide = g_list_alloc();
				view->slide->data = (gpointer) fPath;
			}
		}
	}
}

void viewSlideKill(GtkWidget *widget, gpointer data) {
	ren *viewD;
	
	viewD = (ren *) data;
	viewD->view = NULL;
	g_free(viewD);
}

void loadSlide(sgview *view, gint pos) {
	GList *list;
	gint cnt;
	
	cnt = 0;
	list = view->slide;
	while ((list) && (cnt != pos)) {
		list = list->next;
		cnt++;
	}
	view->currSlide = list;
	loadPic(view, (gchar *) list->data);
}

gint viewSlideMouse(GtkWidget *widget, GdkEventButton *event, gpointer data) {
	ren *viewD;
	
	viewD = (ren *) data;
	if (GTK_CLIST (viewD->ent)->focus_row == -1)
		return FALSE;
	switch (event->button) {
		case 1:
			if (event->type == GDK_2BUTTON_PRESS) 
				loadSlide(viewD->view, GTK_CLIST (viewD->ent)->focus_row);
			break;
		default:
			break;
	} 
	return FALSE;
}

gint viewSlideKey(GtkWidget *widget, GdkEventKey *event, gpointer data) {
	ren *viewD;
	
	viewD = (ren *) data;
	if (GTK_CLIST (viewD->ent)->focus_row == -1)
		return FALSE;

	switch (event->keyval) {
		case GDK_Return:
		case GDK_KP_Enter:
			loadSlide(viewD->view, GTK_CLIST (viewD->ent)->focus_row);
			break;
		default:
			break;
	}
	return FALSE;
}

void viewSlide(GtkWidget *widget, gpointer data) {
	ren *viewD;
	GtkWidget *rsp, *scr, *list;
	gchar *fName, *ins[1], *titles[] = {_("File")};
	GList *slides;
	gint res;
	
	viewD = g_malloc(sizeof(ren) * 1);
	viewD->view = (sgview *) data;
	
	rsp = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	viewD->rsp = rsp;
	gtk_window_set_title(GTK_WINDOW (rsp), "Files In Slide List");
	gtk_window_set_policy(GTK_WINDOW (rsp), FALSE, FALSE, FALSE);
	gtk_window_set_position(GTK_WINDOW (rsp), GTK_WIN_POS_MOUSE);
	gtk_widget_set_usize(GTK_WIDGET (rsp), 380, 360);
	gtk_signal_connect(GTK_OBJECT (rsp), "destroy", (GtkSignalFunc) viewSlideKill, (gpointer) viewD);
	gtk_container_border_width(GTK_CONTAINER (rsp), 5);
	
	scr = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scr), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER (rsp), scr);
	
	list = gtk_clist_new_with_titles(1, titles);
	viewD->ent = list;
	gtk_clist_set_column_width(GTK_CLIST (list), 0, 310);
	gtk_clist_set_column_justification(GTK_CLIST (list), 0, GTK_JUSTIFY_LEFT);
	gtk_clist_set_auto_sort(GTK_CLIST (list), FALSE);
	gtk_clist_set_selection_mode(GTK_CLIST (list), GTK_SELECTION_SINGLE);
	gtk_signal_connect_after(GTK_OBJECT (list), "button_press_event", (GtkSignalFunc) viewSlideMouse, (gpointer) viewD);
	gtk_signal_connect(GTK_OBJECT (list), "key_press_event", (GtkSignalFunc) viewSlideKey, (gpointer) viewD);
	gtk_container_add(GTK_CONTAINER (scr), list);
	gtk_widget_show(list);
	gtk_widget_show(scr);
	gtk_widget_realize(rsp);
	
	gtk_clist_freeze(GTK_CLIST (list));
	slides = viewD->view->slide;
	while (slides) {
		fName = (gchar *) slides->data;
		if (strlen(fName) < 60)
			ins[0] = fName;
		else {
			res = strlen(fName) - 63;
			fName += res;
			ins[0] = g_strconcat("...", fName, NULL);
		}
		gtk_clist_append(GTK_CLIST (list), ins);
		slides = slides->next;
	}
	gtk_clist_thaw(GTK_CLIST (list));
	gtk_widget_show(rsp);
}