#!/usr/bin/perl

# if you want to just run this on a particular partition, then make $DEBUG be 1
# and put approiate values in $fakerec, numTests is actually one less than
# the number of tests that will be run. 
my $DEBUG = 0;
my $fakerec = {
           dev => '/dev/sda7',
           dir => '/home/jpschewe',
           numTests => 2
          };

@table = runIOtest();
if(defined @table) {
  print <<HEADER;
  
<center><table BORDER=0 CELLPADDING=5 WIDTH="100%" BGCOLOR="#C1CDC1">
<tr>
<td BGCOLOR="#666688"><b><font face="ariel, helvetica"><font color="#FFFFFF">
Filesystem Performance as tested by <a href="ftp://ftp.cs.uni-sb.de/pub/others
">iozone</a> maintained by <a href="mailto:b_norcott\@xway.com">Bill
Norcott</a></font></font></b></td>
</tr>
  </table>
   <td BGCOLOR="#FFFFFF"><font color="#000000">Time Required for iozone to
read/write files of various sizes reported in Megabytes/Second (Bigger
Number is Better)</font></td>
<table BGCOLOR="#C0C0C0" NOSAVE >
<tr NOSAVE>
<td NOSAVE><div align="right"><font color="#000000">Device</font></div></td>

<td><div align="right"><font color="#000000">Process</font></div></td>

<td><div align="right"><font color="#000000">1M</font></div></td>

<td><div align="right"><font color="#000000">2M</font></div></td>

<td><div align="right"><font color="#000000">4M</font></div></td>

<td><div align="right"><font color="#000000">8M</font></div></td>

<td><div align="right"><font color="#000000">16M</font></div></td>

<td><div align="right"><font color="#000000">32M</font></div></td>

<td><div align="right"><font color="#000000">64M</font></div></td>

<td><div align="right"><font color="#000000">128M</font></div></td>

<td><div align="right"><font color="#000000">256M</font></div></td>

<td><div align="right"><font color="#000000">512M</font></div></td>

<td><div align="right"><font color="#000000">1G</font></div></td>
</tr> 
HEADER

  for $foo (@table) {
    my @row = @$foo;
    print "<tr>\n";
    for $value (@row) {
      print "<td><div align=\"right\"><font
color=\"#000000\">$value</font></dev></td>\n"
    }
    print "</tr>\n\n";
  }
  print "</table></center>\n";
}
else {
  print "Didn't get any results from the test!";
  exit 1;
}
exit;

sub runIOtest {
  my @table = ();
  my @mounts2 = parsedf();
  if(defined @mounts2) {
    for $value (@mounts2) {
      my @readrow = ();
      my @writerow = ();
      push @readrow, $value->{dev};
      push @readrow, 'read';
      push @writerow, $value->{dev};
      push @writerow, 'write';
      for $exp (0..$value->{numTests}) {
        my $size  = 2**$exp;
        my $results = runIozone($value->{dir}, $size);
        if(! (defined $results)) {
          push @readrow, "read-only";
          push @writerow, "read-only";
        }
        else {
          push @readrow, round($results->{read});
          push @writerow, round($results->{write});
        }
      }
      push @table, \@readrow;
      push @table, \@writerow;
    }
    
    return @table;
  }
  else {
    return undef;
  }
}


sub parsedf {
  my @mounts = ();
  my $output = `df -k`;
  if($output =~ /invalid/) {
    print STDERR "df doesn't accept df -k, don't know what to do here\n";
    return undef;
  }
  #print $output;
  my @lines = split(/\n/, $output);
  
  for $index (1..$#lines) {
    my($dev, $size, $used, $avail, $capacity, $mountPoint, $rest) = split(/\s+/,
$lines[$index]);
    
    
    # convert free space to Mb
    $free = $avail / 1024;
    
    # build list of sizes from 1 to 1Gb or 90% of free space
    my @sizes = ();             # list of sizes found
    my $curSize = 1;
    while($curSize <= 1024 && $curSize < 0.9 * $free) {
      push @sizes, $curSize;
      $curSize *= 2;
    }
    my $rec = {
               dev => $dev,
               dir => $mountPoint,
               numTests => $#sizes
              };
    push @mounts, $rec;
  }

  # for debugging
  if($DEBUG) {
    @mounts = ();
    push @mounts, $fakerec;
  }
  return @mounts;
}

# run iozone with the given directory and size and return an array with
# the output from iozone
sub runIozone {
  my($dir, $size) = @_;
  my $filename = $dir."/hwinfo2html-test";
  while(-e $filename) {
    $filename .= "1";
  }
  # check if we can write to this partition
  open TEST, ">$filename" || return undef;
  if(! -w TEST) {
    close TEST;
    return undef;
  }
  close TEST;
  
  my $output = `iozone $size $filename`;

  my $temp = $output;
  $temp =~ /(\d+) bytes\/second for reading the file/;
  my $read = $1 / 1024 / 1024;

  $temp =~ /(\d+) bytes\/second for writing the file/;
  my $write = $1 / 1024 / 1024;

  my $rec = {
          read => $read,
          write => $write
         };

  return $rec;
}

sub round {
  my $value = @_[0];
  return int($value + 0.5);
}
