//
// Copyright (C) 1999, Mark W J Redding <grawlfang@ukhackers.net>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <unistd.h>
#include <stdlib.h>

#include <unistd.h>
#include <kapp.h>

#include "KalIO.h"


KalIO::KalIO(QString* addr , int port)
{
	ioType = network;
	soc = new KalSocket(addr,port);
	if(soc->isActive())
	{
		fileExists = true;
		readOnly = false;
	}
	else	// force it into diskfile mode
	{
		ioType = diskfile;
		fileExists = false;
		readOnly = true;
	}
}

KalIO::KalIO()
{
	ioType = diskfile;
	fileExists = false;
	readOnly = true;
}

KalIO::KalIO(QString* inhost)
{
#ifdef USE_MYSQL
	ioType = database;
	fileExists = false;
	host = *inhost;
	readOnly = true;
	doInit = true;
#else
	ioType = diskfile;
	fileExists = false;
	readOnly = true;
#endif
}

KalIO::~KalIO()
{
	if(ioType == network)
	{
		delete soc;
	}

#ifdef USE_MYSQL
	if(ioType == database)
	{
		::mysql_close(sql);
	}
#endif
}

bool
KalIO::isNetwork()
{
	return (ioType == network) ? true : false;
}

bool
KalIO::isDatabase()
{
	return (ioType == database) ? true : false;
}

bool
KalIO::exists()
{
	return fileExists;
}

void
KalIO::unopen(QString& session)
{
	if(ioType == network)
	{
		soc->command(KalSocket::closeSession,session);
	}
	if(ioType == diskfile)
	{
		unlock();
	}
#ifdef USE_MYSQL
	if(ioType == database)
	{
		::mysql_close(sql);
	}
#endif
	fileExists = false;
	readOnly = true;
}

bool
KalIO::open(QString& user, QString& session, bool write, bool onlyRead)
{
	readOnly = true;

	if(ioType == network)
	{
		if(write)
		{
			soc->command(KalSocket::saveSession,session);
			readOnly = false;
		}
		else if(onlyRead)
		{
			soc->command(KalSocket::openRead,session);
			readOnly = true;
		}
		else
		{
			soc->command(KalSocket::openWrite,session);
			readOnly = false;
		}
		fileOnDisk = "";
	}

#ifdef USE_MYSQL
	if(ioType == database)
	{
		if(doInit)
		{
			::mysql_init(&mysql);
			doInit = false;
			sql = ::mysql_real_connect(	&mysql,
							host.data(),
							user.data(),
							NULL,
							"kalendar",
							0,
							NULL,
							0);
			printf("connected (%s)\n",mysql_error(sql));
		}
		fileOnDisk = "";	
		fileExists = true;
		if(!onlyRead) readOnly = false;	
	}
#endif
	
	if(ioType == diskfile)
	{
		fileOnDisk = session;
		ufil = KApplication::kde_datadir() + "/kalendar/" + fileOnDisk + ".kal";
		fil.setName(ufil);
		if(!write)
		{
			if(fil.exists())
			{
				fil.open(IO_ReadOnly);
				ds.setDevice(&fil);
				fileExists = true;
			}
		}
		else // opening for write
		{
			fileExists = fil.exists();
			fil.open(IO_WriteOnly);
			ds.setDevice(&fil);
			readOnly = false;
		}
		if(!onlyRead)	// not read only, so lock it
		{
			readOnly = lock();
		}
		else
		{
			readOnly = true;
		}
	}
	return readOnly;
}

void
KalIO::close()
{
	if(ioType == network)
	{
		soc->close();
	}
	
	if(ioType == diskfile)
	{
	// ensure that all users can access the calendar
		fil.close();
		QString modcmd = "chmod 777 " + ufil;
		::system(modcmd.data());
		unlock();
	}

	fileExists = false;
	readOnly = true;
}

bool
KalIO::lock()
{
	if(ioType == diskfile)
	{

		QString ulock = KApplication::kde_datadir() + "/kalendar/" + fileOnDisk + ".LCK";

		QFile lck(ulock);
		readOnly = lck.exists();
		if(!readOnly)
		{
			lck.open(IO_ReadWrite);
			lck.writeBlock(fileOnDisk.data(),fileOnDisk.length());
			lck.close();
		}
	}

	return readOnly;
}

void
KalIO::unlock()
{
	if(ioType == diskfile)
	{
		if(!readOnly)
		{
			QString ulock = KApplication::kde_datadir() + "/kalendar/" + fileOnDisk + ".LCK";
			if(QFile::exists(ulock.data()))
			{
				QString delcmd = "rm " + ulock;
				::system(delcmd.data());
			}
		}
		readOnly = true;
	}
}

bool
KalIO::isReadOnly()
{
	return readOnly;
}


void
KalIO::command(char txt1 , QString& txt2)
{
	if(ioType == network)
	{
	QString txt = txt1 + txt2;

		write(txt);
	}
}

// QString handler

void
KalIO::read(QString* buffer , int len)
{
	if(ioType == network)
	{
		soc->read(buffer,len);
	}
	if(ioType == diskfile)
	{
		ds.readRawBytes(buffer->data(),len);
	}
}

void
KalIO::write(QString& txt , int len)
{
	if(ioType == network)
	{
		soc->write(txt);
	}
	if(ioType == diskfile)
	{
		ds.writeRawBytes(txt.data(),len);
	}
}

// char handler

void
KalIO::write(char* txt)
{
	if(ioType == network)
	{
		soc->write(txt);
	}
	if(ioType == diskfile)
	{
		ds >> txt;
	}
}

// Integer handler

void
KalIO::read(int* num)
{
	if(ioType == network)
	{
		soc->read(num);
	}
	if(ioType == diskfile)
	{
	int r;

		ds >> r;
		*num = r;
	}
}

void
KalIO::write(int num)
{
	if(ioType == network)
	{
		soc->write(num);
	}
	if(ioType == diskfile)
	{
		ds << num;
	}
}

// QDateTime handler

void
KalIO::read(QDateTime* dt)
{
	if(ioType == network)
	{
		soc->read(dt);
	}
	if(ioType == diskfile)
	{
	QDateTime r;

		ds >> r;
		*dt = r;
	}
}

void
KalIO::write(QDateTime dt)
{
	if(ioType == network)
	{
		soc->write(dt);
	}
	if(ioType == diskfile)
	{
		ds << dt;
	}
}

// QDate handler

void
KalIO::read(QDate* dt)
{
	if(ioType == network)
	{
		soc->read(dt);
	}
	if(ioType == diskfile)
	{
	QDate r;

		ds >> r;
		*dt = r;
	}
}

void
KalIO::write(QDate dt)
{
	if(ioType == network)
	{
		soc->write(dt);
	}
	if(ioType == diskfile)
	{
		ds << dt;
	}
}

// Boolean handler

void
KalIO::read(bool* num)
{
int r;

	if(ioType == network)
	{
		soc->read(&r);
	}
	if(ioType == diskfile)
	{
		ds >> r;
	}
	*num = (bool)r;
}

void
KalIO::write(bool num)
{
int r = (int)num;

	if(ioType == network)
	{
		soc->write(r);
	}
	if(ioType == diskfile)
	{
		ds << r;
	}
}

int
KalIO::select(int unit, QString& query)
{
#ifdef USE_MYSQL

int sts;

printf("select (%s)\n",query.data());
	sts = ::mysql_query(sql,query.data());
	if(sts)
	{
		printf("error is %s\n",mysql_error(sql));
		return 0;
	}
	result[unit] = ::mysql_store_result(sql);
	if(!result[unit])
	{
		return 0;
	}
	num_cols = (int)mysql_num_fields(result[unit]);	// a mysql macro
	num_rows = (int)mysql_num_rows(result[unit]);	// a mysql macro
	if(!num_rows)
	{
		doneSelect(unit);
		return 0;
	}
	return num_rows;
#else
	return 0;
#endif
}

bool
KalIO::fetch(int unit)
{
#ifdef USE_MYSQL

	row[unit] = ::mysql_fetch_row(result[unit]);
	if(!row[unit])
	{
		return false;
	}
	lens[unit] = ::mysql_fetch_lengths(result[unit]);
	return true;
#else
	return false;
#endif
}

void
KalIO::doneSelect(int unit)
{
#ifdef USE_MYSQL
	::mysql_free_result(result[unit]);
#endif
}

int
KalIO::execute(QString& cmd)
{
#ifdef USE_MYSQL

int sts;

printf("executing query (%s)\n",cmd.data());
	sts = ::mysql_query(sql,cmd.data());
	if(sts)
	{
		printf("error is %s\n",mysql_error(sql));
		return 0;
	}
	return (int)mysql_insert_id(sql);	// a mysql macro
#else
	return 0;
#endif
}

char *
KalIO::getString(int unit, int ind)
{
#ifdef USE_MYSQL
	return (char*)row[unit][ind];
#else
	return (char*)0;
#endif
} 

int
KalIO::getNumber(int unit, int ind)
{
#ifdef USE_MYSQL
	return (int)atol(row[unit][ind]);
#else
	return 0;
#endif
}

QDate&
KalIO::getDate(int unit,int ind)
{
static QDate tdate;

#ifdef USE_MYSQL

QString sdate;
int y,m,d;

	sdate.resize(0);
	sdate.append(row[unit][ind]);
	y = atol(sdate.left(4).data());
	m = atol(sdate.mid(5,2).data());
	d = atol(sdate.right(2).data());

	tdate.setYMD(y,m,d);
#endif
	return tdate;
}

int
KalIO::getUserID()
{
	return userID;
}

void
KalIO::setUserID(int u)
{
	userID = u;
}

