//
// Copyright (C) 1999, Mark W J Redding <grawlfang@ukhackers.net>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include "KalSocket.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>

KalSocket::KalSocket(QString* addr , int port)
{
	sock = ::socket(AF_INET,SOCK_STREAM,0);
	if(sock >= 0)
	{
	struct sockaddr_in server;
	struct hostent *hp;

		::memset(&server,0,sizeof(server));
		server.sin_family = AF_INET;
		hp = gethostbyname(addr->data());
		::memcpy(&server.sin_addr,hp->h_addr,hp->h_length);
		server.sin_port = htons(port);

		if(::connect(sock,(sockaddr *)&server,sizeof(server)) < 0)
		{
			sock = -1;
		}
	}
}

KalSocket::~KalSocket()
{
	if(sock >= 0)
	{
		close();
	}
}

int
KalSocket::getSocket()
{
	return sock;
}

void
KalSocket::close()
{
	if(sock >= 0)
	{
		::close(sock);
		sock = -1;
	}
}

bool
KalSocket::isActive()
{
	return (sock == -1) ? false : true;
}

int
KalSocket::read(QString* buffer , int len)
{
int rlen,clen;
char buff[1024];

	clen = 0;
	if(sock >= 0)
	{
		rlen = (len == 0) ? sizeof(buff) : len;

		buffer->resize(0);
		clen = ::read(sock,buff,rlen);
		if(clen < 0)
		{
			return clen;
		}
		buffer->append(buff);
	}
	return clen;
}

bool
KalSocket::write(QString& txt)
{
	if(sock >= 0)
	{
		::write(sock,txt.data(),txt.length());
		return true;
	}
	else
	{
		return false;
	}
}

bool
KalSocket::write(char* txt)
{
	if(sock >= 0)
	{
		::write(sock,txt,::strlen(txt));
		return true;
	}
	else
	{
		return false;
	}
}

bool
KalSocket::command(char oper , QString& txt)
{
QString ltxt;
char rs;

	ltxt.sprintf("%c%s",oper,txt.data());
	write(ltxt);
	::read(sock,&rs,sizeof(char));
printf("return status = %c\n",rs);
	return (rs=='T') ? true : false;
}

bool
KalSocket::write(int num)
{
	if(sock >= 0)
	{
		::write(sock,&num,sizeof(int));
		return true;
	}
	else
	{
		return false;
	}
}

int
KalSocket::read(int* num)
{
	if(sock >= 0)
	{
		::read(sock,num,sizeof(int));
	}
	return *num;
}

int
KalSocket::read(QDateTime* dt)
{
QDate date;
QTime time;
int h,m,s;

	read(&date);
	read(&h);
	read(&m);
	read(&s);
	time.setHMS(h,m,s);

	dt->setDate(date);
	dt->setTime(time);
	return h * 10000 + m * 100 + s;
}

bool
KalSocket::write(QDateTime dt)
{
QDate date = dt.date();
QTime time = dt.time();

	write(date);
	write(time.hour());
	write(time.minute());
	write(time.second());
	return true;
}

int
KalSocket::read(QDate* dt)
{
int y,m,d;

	read(&y);
	read(&m);
	read(&d);
	dt->setYMD(y,m,d);
	return y * 10000 + m * 100 + d;
}

bool
KalSocket::write(QDate dt)
{
	write(dt.year());
	write(dt.month());
	write(dt.day());
	return true;
}


