//
// Copyright (C) 1998, Mark W J Redding <grawlfang@ukhackers.net>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include "TaskList.h"

#include "Ktodo.h"
#include "KalIO.h"
#include "Klook.h"

//
// create a blank list of todo items
//
TaskList::TaskList(Klook* w)
{
	mainW = w;
	tasks.setAutoDelete(true);
}

//
// remove all todo items from the list
//
TaskList::~TaskList()
{
	remove();
}

//
// save all todo items to a file
//
void
TaskList::save(bool hasChanged)
{
	if(!mainW->getStream()->isDatabase())
	{
		ToDo *curr;
		QListIterator<ToDo> it(tasks);
		int recs = it.count();
		if(hasChanged)
		{
			mainW->getStream()->write(recs);
		}
		for(; it.current(); ++it)
		{
			curr = it.current();
			curr->save(hasChanged);
		}
	}
}

//
// load all todo items from a file and add to this list
//
void
TaskList::load(int version)
{
ToDo *read;
int recs;

	if(mainW->getStream()->isDatabase())
	{
	QString query;

		query.sprintf("select todo_id,priority,subject,text,complete,startdate,duration,sensitivity from todo where user_id=%d",mainW->getStream()->getUserID());
		recs = mainW->getStream()->select(2,query);
	}
	else
	{
		mainW->getStream()->read(&recs);
	}

	for( int i = 0; i < recs; i++)
	{
		read = new ToDo(this);
		read->load(version);
		if(!read->isComplete())
		{
			tasks.append(read);
		}
	}

	if(mainW->getStream()->isDatabase()&&recs)
	{
		mainW->getStream()->doneSelect(2);
	}
}

//
// remove all todo items from this list
//
void
TaskList::remove()
{
	tasks.clear();
}

//
// cycle through this list and display all todo items
//
void
TaskList::viewTasks(Ktodo* v)
{
	v->clear();
	ToDo *curr;
	QListIterator<ToDo> it(tasks);
	for(; it.current(); ++it)
	{
		curr = it.current();
		v->showTask(curr);
	}
}

//
// search for a todo item
//
ToDo*
TaskList::findTask(int ind)
{
	ToDo *curr;
	QListIterator<ToDo> it(tasks);
	for(int i = 0; it.current(); ++it, i++)
	{
		curr = it.current();
		if(i == ind)
		{
			return curr;
		}
	}
	return (ToDo*)0;
}

//
// add a new todo item to this list
//
ToDo*
TaskList::addTask(QDate& sd)
{
	ToDo* curr = new ToDo(this,sd);
	tasks.append(curr);
	return curr;
}

//
// remove a single todo item from this list
//
void
TaskList::removeTask(ToDo* a)
{
	a->remove();
	tasks.remove(a);
}

KalIO*
TaskList::getStream()
{
	return mainW->getStream();
}

