//
// Copyright (C) 1998, Mark W J Redding <grawlfang@ukhackers.net>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <kapp.h>
#include <klocale.h>

#include "ToDo.h"
#include "TaskList.h"
#include "KalIO.h"

//
// create a blank todo item
//
ToDo::ToDo(TaskList* t)
{
	tList = t;
	subject = new QString("");
	text = new QString("");
	priority = 0;
	valid = false;
	complete = false;
	startDate = QDate::currentDate();
	duration = 0;
	sensitivity = 0;
	identifier = 0;
}

ToDo::ToDo(TaskList* t, QDate& sd)
{
	tList = t;
	subject = new QString("");
	text = new QString("");
	priority = 0;
	valid = false;
	complete = false;
	setStartDate(sd);
	duration = 0;
	sensitivity = 0;
	identifier = 0;
}

//
// create a todo item
//
ToDo::ToDo(TaskList* t, QString& sub, int pri , QDate& sd, int dur, QString& txt=0)
{
	tList = t;
	subject = new QString(sub);
	text = new QString(txt);
	setPriority(pri);
	setText(txt);
	setStartDate(sd);
	setDuration(dur);
	sensitivity = 0;
	valid = false;
	identifier = 0;
}

//
// tidy up
//
ToDo::~ToDo()
{
	delete subject;
	delete text;
}

//
// set the subject text of this todo item
//
void
ToDo::setSubject( QString& sub )
{
	subject->resize(0);
	subject->append(sub);
}

//
// return a reference to the subject text
//
QString&
ToDo::getSubject()
{
	return *subject;
}

//
// set the descriptive text of this todo item
//
void
ToDo::setText( QString& app )
{
	text->resize(0);
	text->append(app);
}

//
// return a reference to the descriptive text of this todo item
//
QString&
ToDo::getText()
{
	return *text;
}

//
// set the current priority of this todo item (0-2, high,med,low)
//
void
ToDo::setPriority(int p)
{
	priority = p;
}
//
// return the current priority of this todo item
//
int
ToDo::getPriority()
{
	return priority;
}

//
// set the current sensitivity of this todo item
//
void
ToDo::setSensitivity(int p)
{
	sensitivity = p;
}
//
// return the current priority of this todo item
//
int
ToDo::getSensitivity()
{
	return sensitivity;
}

//
// save this item to a file
//
void
ToDo::save(bool hasChanged)
{
int len;

	if(!tList->getStream()->isDatabase())
	{
		if(hasChanged)
		{
			tList->getStream()->write(priority);
			len = subject->length();
			tList->getStream()->write(len);
			if(len)
			{
				tList->getStream()->write(*subject,len);
			}
			len = text->length();
			tList->getStream()->write(len);
			if(len)
			{
				tList->getStream()->write(*text,len);
			}
			tList->getStream()->write((int)complete);
			tList->getStream()->write(startDate);
			tList->getStream()->write(duration);
			tList->getStream()->write(sensitivity);
		}
	}
}

//
// load this item from a file
//
void
ToDo::load(int version)
{
int len;

	if(tList->getStream()->isDatabase())
	{
		tList->getStream()->fetch(2);
		identifier = tList->getStream()->getNumber(2,0);
		priority = tList->getStream()->getNumber(2,1);
		subject->resize(0);
		subject->append(tList->getStream()->getString(2,2));
		text->resize(0);
		text->append(tList->getStream()->getString(2,3));
		complete = tList->getStream()->getNumber(2,4) ? true : false;	
		startDate = tList->getStream()->getDate(2,5);
		duration = tList->getStream()->getNumber(2,6);
		sensitivity = tList->getStream()->getNumber(2,7);
	}
	else
	{
		tList->getStream()->read(&priority);
		tList->getStream()->read(&len);
		subject->resize(len);
		if(len)
		{
			tList->getStream()->read(subject,len);
		}
		subject->truncate(len);

		tList->getStream()->read(&len);
		text->resize(len);
		if(len)
		{
			tList->getStream()->read(text,len);
		}
		text->truncate(len);
		tList->getStream()->read(&complete);
		if(version > 3)
		{
			tList->getStream()->read(&startDate);
			tList->getStream()->read(&duration);
		}
		if(version > 4)
		{
			tList->getStream()->read(&sensitivity);
		}
	}
	makeValid();
}

//
// return the status of this item.
// the EditTask class requires an existing instance of a ToDo class.
// if EditTask was creating a new ToDo item and quits, then this instance
// of the ToDo item is not required and should be deleted
//
bool
ToDo::isNew()
{
	return !valid;
}

//
// mark this todo item as valid (ie. EditTask exited rather than quit)
//
void
ToDo::makeValid()
{
	valid = true;
}

//
// mark this todo item as done
//
void
ToDo::makeComplete()
{
	complete = true;
}

//
// mark this todo item as not done
//
void
ToDo::makeInComplete()
{
	complete = false;
}

//
// return the status of this todo item (true - it is complete)
//
bool
ToDo::isComplete()
{
	return complete;
}


void
ToDo::setStartDate(QDate& d)
{
	startDate = d;
}


QDate&
ToDo::getStartDate()
{
	return startDate;
}


void
ToDo::setDuration(int d)
{
	duration = d;
}


int
ToDo::getDuration()
{
	return duration;
}

void
ToDo::update()
{
	if(tList->getStream()->isDatabase())
	{
	QString query;
	QString sdate;
			
		sdate.sprintf("%04d-%02d-%02d",startDate.year(),startDate.month(),startDate.day());

		if(identifier)
		{
			query.sprintf("update todo set priority=%d,subject='%s',text='%s',complete=%d,startdate='%s',duration='%d',sensitivity=%d where todo_id=%d",
				priority,
				subject->data(),
				text->data(),
				complete ? 1 : 0,
				sdate.data(),
				duration,
				sensitivity,
				identifier);
		}
		else
		{
			query.sprintf("insert into todo (user_id,priority,subject,text,complete,startdate,duration,sensitivity) values (%d,%d,'%s','%s','%s','%s',%d,%d)",
				tList->getStream()->getUserID(),
				priority,
				subject->data(),
				text->data(),
				complete ? 1 : 0,
				sdate.data(),
				duration,
				sensitivity);
		}
		int id = tList->getStream()->execute(query);
		if(!identifier)
		{
			identifier = id;
		}
	}
}

void
ToDo::remove()
{
	if(tList->getStream()->isDatabase())
	{
	QString query;

		query.sprintf("delete from todo where todo_id=%d",identifier);
		tList->getStream()->execute(query);
	}
}
