//
// Copyright (C) 1998, Mark W J Redding <grawlfang@ukhackers.net>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include "UserAccess.h"

#include "KalIO.h"
#include "UserList.h"

//
// create a user access item
//
UserAccess::UserAccess(UserList* u)
{
	uList = u;
	username = new QString(" ");
	add_public = false;
	del_public = false;
	view_business = false;
	add_business = false;
	del_business = false;
	view_personnal = false;
	add_personnal = false;
	del_personnal = false;
	identifier = 0;
}

UserAccess::UserAccess(UserList* u, QString& user, bool a, bool b, bool c, bool d, bool e, bool f, bool g, bool h)
{
	uList = u;
	username = new QString(user);
	add_public = a;
	del_public = b;
	view_business = c;
	add_business = d;
	del_business = e;
	view_personnal = f;
	add_personnal = g;
	del_personnal = h;
	identifier = 0;
}

//
// tidy up
//
UserAccess::~UserAccess()
{
	delete username;
}

//
// set the subject text of this UserAccess item
//
void
UserAccess::setUsername( QString& user )
{
	username->resize(0);
	username->append(user);
}

//
// save this item to a file
//
void
UserAccess::save(bool hasChanged)
{
int len;

	if(!uList->getStream())
	{
		if(hasChanged)
		{
			len = username->length();
			uList->getStream()->write(len);
			if(len)
			{
				uList->getStream()->write(*username,len);
			}
			uList->getStream()->write((int)add_public);
			uList->getStream()->write((int)del_public);
			uList->getStream()->write((int)view_business);
			uList->getStream()->write((int)add_business);
			uList->getStream()->write((int)del_business);
			uList->getStream()->write((int)view_personnal);
			uList->getStream()->write((int)add_personnal);
			uList->getStream()->write((int)del_personnal);
		}
	}
}

//
// load this item from a file
//
void
UserAccess::load(int version)
{
	if(uList->getStream()->isDatabase())
	{
		uList->getStream()->fetch(8);
		identifier = uList->getStream()->getNumber(8,0);
		username->resize(0);
		username->append(uList->getStream()->getString(8,1));
		add_public = (uList->getStream()->getNumber(8,2)) ? true : false;
		del_public = (uList->getStream()->getNumber(8,3)) ? true : false;
		view_business = (uList->getStream()->getNumber(8,4)) ? true : false;
		add_business = (uList->getStream()->getNumber(8,5)) ? true : false;
		del_business = (uList->getStream()->getNumber(8,6)) ? true : false;
		view_personnal = (uList->getStream()->getNumber(8,7)) ? true : false;
		add_personnal = (uList->getStream()->getNumber(8,8)) ? true : false;
		del_personnal = (uList->getStream()->getNumber(8,9)) ? true : false;
	}
	else
	{
	int len;

		uList->getStream()->read(&len);
		username->resize(len);
		if(len)
		{
			uList->getStream()->read(username,len);
		}
		username->truncate(len);

		uList->getStream()->read(&add_public);
		uList->getStream()->read(&del_public);
		uList->getStream()->read(&view_business);
		uList->getStream()->read(&add_business);
		uList->getStream()->read(&del_business);
		uList->getStream()->read(&view_personnal);
		uList->getStream()->read(&add_personnal);
		uList->getStream()->read(&del_personnal);
	}
}

//
// set the public access bits
//
void
UserAccess::setPublic(bool a, bool d)
{
	add_public = a;
	del_public = d;
}

//
// set the business access bits
//
void
UserAccess::setBusiness(bool v, bool a, bool d)
{
	view_business = v;
	add_business = a;
	del_business = d;
}

//
// set the personnal access bits
//
void
UserAccess::setPersonnal(bool v, bool a, bool d)
{
	view_personnal = v;
	add_personnal = a;
	del_personnal = d;
}

QString&
UserAccess::getUsername()
{
	return *username;
}


bool
UserAccess::canAddPublic()
{
	return add_public;
}

bool
UserAccess::canDelPublic()
{
	return del_public;
}

bool
UserAccess::canViewBusiness()
{
	return view_business;
}

bool
UserAccess::canAddBusiness()
{
	return add_business;
}

bool
UserAccess::canDelBusiness()
{
	return del_business;
}

bool
UserAccess::canViewPersonnal()
{
	return view_personnal;
}

bool
UserAccess::canAddPersonnal()
{
	return add_personnal;
}

bool
UserAccess::canDelPersonnal()
{
	return del_personnal;
}

bool
UserAccess::canAdd()
{
	return add_public | add_business | add_personnal;
}

bool
UserAccess::canDelete()
{
	return del_public | del_business | del_personnal;
}

void
UserAccess::update()
{
	if(uList->getStream()->isDatabase())
	{
	QString query;

		if(identifier)
		{
			query.sprintf("update security set alloweduser='%s',add_public=%d,del_public=%d,view_business=%d,add_business=%d,del_business=%d,view_personnal=%d,add_personnal=%d,del_personnal=%d where security_id=%d",
				username->data(),
				(add_public) ? 1 : 0,
				(del_public) ? 1 : 0,
				(view_business) ? 1 : 0,
				(add_business) ? 1 : 0,
				(del_business) ? 1 : 0,
				(view_personnal) ? 1 : 0,
				(add_personnal) ? 1 : 0,
				(del_personnal) ? 1 : 0,
				identifier);
		}
		else
		{
			query.sprintf("insert into security (user_id,alloweduser,add_public,del_public,view_business,add_business,del_business,view_personnal,add_personnal,del_personnal) values (%d,'%s',%d,%d,%d,%d,%d,%d,%d,%d)",
				uList->getStream()->getUserID(),
				username->data(),
				(add_public) ? 1 : 0,
				(del_public) ? 1 : 0,
				(view_business) ? 1 : 0,
				(add_business) ? 1 : 0,
				(del_business) ? 1 : 0,
				(view_personnal) ? 1 : 0,
				(add_personnal) ? 1 : 0,
				(del_personnal) ? 1 : 0);
		}
		int id = uList->getStream()->execute(query);
		if(!identifier)
		{
			identifier = id;
		}
	}
}

void
UserAccess::remove()
{
	if(uList->getStream()->isDatabase())
	{
	QString query;

		query.sprintf("delete from security where security_id=%d",identifier);
		uList->getStream()->execute(query);
	}
}

int
UserAccess::getID()
{
	return identifier;
}

