/* 
 *      XSitecopy, for managing remote web sites with a GNOME interface.
 *      Copyright (C) 1999, Lee Mallabone <lee0@callnetuk.com
 *                                                                        
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *     
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *     
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *  
 */

/* This file contains generic functions used in the creation of a gtk
 * front end for sitecopy.
*/

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>

#include "sites.h"
#include "common.h"
#include "rcfile.h"
#include "fe_gnome_common.h"
#include "fe_gtk_main.h"
#include "fe_gtk_site_ops.h"
#include "config.h"

extern struct site_t *selected_site;
extern GtkWidget *error_log_list, *error_log_window;

extern struct site_t *all_sites;

/* For the 'main' upload window */
GtkWidget *upload_window, *main_progressbar, *job_progressbar, *error_button;
GtkWidget *status_label, *op_label, *file_label, *dir_label, *info_label;
GtkWidget *begin_button, *close_button;


void close_main_update_window (GtkWidget *button, gpointer data) {
   extern GtkWidget *connection_label;
   gtk_window_set_modal (GTK_WINDOW (upload_window), false);
   /* FIXME: Kill the pthread, write the files list back,, and enable the
    * error log button. */
   gtk_widget_destroy (upload_window);
   connection_label = NULL;
}

/* All this pissing about with functions to make the fe, functions to
 * start the thing, functions to call the thread, then more to actually
 * DO the bloody thing, are in fact necessary.... I think. */

void *actual_main_update (void *a_site) {
   int ret;
   
   pthread_detach (pthread_self());
   gtk_widget_set_sensitive (begin_button, false);

   switch (ret = rc_verifysite (selected_site)) {
    case SITE_NONAME:
      fe_status ("No site name given.\n");
      return NULL;
      break;
    case SITE_NOSERVER:
      fe_status ("No server name given.\n");
      return NULL;
      break;
    case SITE_NOUSERNAME:
      fe_status ("No username given.\n");
      return NULL;
      break;
    case SITE_NOPASSWORD:
      fe_status ("No password given.\n");
      return NULL;
      break;
    case SITE_NOREMOTEDIR:
      fe_status ("No remote directory given.\n");
      return NULL;
      break;
    case SITE_NOLOCALDIR:
      fe_status ("No local directory given.\n");
      return NULL;
      break;
    case SITE_ACCESSLOCALDIR:
      fe_status ("The local dir couldn't be accessed.\n");
      return NULL;
      break;
    case 0:
      /* Site integrity is okay. */
      break;
    default:
      printf ("Site was verified okay I think, but control shouldn't be here.\n");
   }
   ret = site_update (selected_site, FALSE);
   if (ret == SITE_CONNECT) {
      gtk_label_set (GTK_LABEL (status_label), "Unable to establish connection.");
      gtk_label_set (GTK_LABEL (op_label), " ");
      gtk_label_set (GTK_LABEL (file_label), " ");
      gtk_label_set (GTK_LABEL (dir_label), " ");
   } else if (ret == SITE_ERRORS) {
      gtk_label_set (GTK_LABEL (status_label), "There was a problem with the file/directory transfer.");
      gtk_label_set (GTK_LABEL (op_label), " ");
      gtk_label_set (GTK_LABEL (file_label), " ");
      gtk_label_set (GTK_LABEL (dir_label), " ");
      site_writefiles (selected_site);
      site_destroy (selected_site);
      site_readfiles (selected_site);
      refresh_selected();
      gtk_widget_set_sensitive (error_button, true);
   } else if (ret == SITE_LOOKUP) {
      gtk_label_set (GTK_LABEL (status_label), "Unable to connect: Host name look-up failed.");
      gtk_label_set (GTK_LABEL (op_label), " ");
      gtk_label_set (GTK_LABEL (file_label), " ");
      gtk_label_set (GTK_LABEL (dir_label), " ");
   } else if (ret == SITE_OK) {
      site_writefiles (selected_site);
      site_destroy (selected_site);
      site_readfiles (selected_site);
      gtk_label_set (GTK_LABEL (status_label), "Update complete.");
      gtk_label_set (GTK_LABEL (op_label), " ");
      gtk_label_set (GTK_LABEL (file_label), " ");
      gtk_label_set (GTK_LABEL (dir_label), " ");
      gtk_label_set (GTK_LABEL (info_label), "There were no errors.");
      redraw_main_area();
      refresh_selected();
   } else {
      gtk_label_set (GTK_LABEL (status_label), "Unexpected Update Return Value! Contact Maintainer.");
   }
   return NULL;
}

int start_main_update (void) { 
   pthread_t update_thread_id = 0;
   if ( pthread_create (&update_thread_id, NULL, actual_main_update, 
			NULL) != 0) {
      fe_status ("There was a problem creating the thread to do an update.\n");
      return 0;
   }
   if (update_thread_id != 0)
     pthread_join (update_thread_id, NULL);
   gtk_window_set_modal (GTK_WINDOW (upload_window), FALSE);
   return 1;
}

int main_update_please (GtkWidget *update_button, gpointer data) {
   extern GtkWidget *connection_label;
   extern float uploaded_bytes;
   GtkWidget *vbox, *table, *hbox;
   GtkWidget *label, *button;

   uploaded_bytes = 0.0;
   if (selected_site == NULL) {
      fe_status ("You must select a site if you want to upload the changes!");
      return 0;
   }
   if (! selected_site->is_different ) {
      fe_status ("Remote and local sites are already synchronised.");
      return 1;
   }

   upload_window = gtk_window_new (GTK_WINDOW_TOPLEVEL); /* dialog? */
   gtk_widget_set_usize (GTK_WIDGET (upload_window), 390, -1);
   gtk_window_set_title (GTK_WINDOW (upload_window), "Transfer Progress");
   gtk_container_set_border_width (GTK_CONTAINER (upload_window), 5);
   gtk_window_set_modal (GTK_WINDOW (upload_window), true);
   
   vbox = gtk_vbox_new (FALSE, 4);
   gtk_widget_show (vbox);
   
   table = gtk_table_new (4, 2, FALSE);
   gtk_widget_show (table);
   
   label = gtk_label_new ("Status:");
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
   gtk_widget_show (label);
   gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);

   status_label = gtk_label_new ("Waiting for user input.");
   gtk_misc_set_alignment (GTK_MISC (status_label), 0, 0.5);
   gtk_widget_show (status_label);
   connection_label = status_label;
   gtk_table_attach_defaults (GTK_TABLE (table), status_label, 1, 2, 0, 1);

   label = gtk_label_new ("File:");
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
   gtk_widget_show (label);
   gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 1, 2);
   
   file_label = gtk_label_new (" ");
   gtk_misc_set_alignment (GTK_MISC (file_label), 0, 0.5);
   gtk_widget_show (file_label);
   gtk_table_attach_defaults (GTK_TABLE (table), file_label, 1, 2, 1, 2);
   
   label = gtk_label_new ("Operation:");
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
   gtk_widget_show (label);
   gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 2, 3);

   op_label = gtk_label_new (" ");
   gtk_misc_set_alignment (GTK_MISC (op_label), 0, 0.5);
   gtk_widget_show (op_label);
   gtk_table_attach_defaults (GTK_TABLE (table), op_label, 1, 2, 2, 3);

   label = gtk_label_new ("To Directory:");
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
   gtk_widget_show (label);
   gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 3, 4);
   
   dir_label = gtk_label_new (" ");
   gtk_misc_set_alignment (GTK_MISC (dir_label), 0, 0.5);
   gtk_widget_show (dir_label);
   gtk_table_attach_defaults (GTK_TABLE (table), dir_label, 1, 2, 3, 4);

   gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 2);
   
   label = gtk_label_new ("Current Progress:");
   gtk_widget_show (label);
   gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 1);
   
   main_progressbar = gtk_progress_bar_new ();
   gtk_widget_show (main_progressbar);
   gtk_box_pack_start (GTK_BOX (vbox), main_progressbar, FALSE, FALSE, 2);
   
   label = gtk_label_new ("Total Progress:");
   gtk_widget_show (label);
   gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 1);
   
   job_progressbar = gtk_progress_bar_new ();
   gtk_widget_show (job_progressbar);
   gtk_box_pack_start (GTK_BOX (vbox), job_progressbar, FALSE, FALSE, 2);
   
   hbox = gtk_hbox_new (FALSE, 3);
   gtk_widget_show (hbox);

   begin_button = gtk_button_new_with_label ("Begin");
   gtk_signal_connect (GTK_OBJECT (begin_button), "clicked", 
		       GTK_SIGNAL_FUNC (start_main_update), NULL);
   gtk_widget_show (begin_button);
   gtk_box_pack_end (GTK_BOX (hbox), begin_button, FALSE, FALSE, 2);

   close_button = gtk_button_new_with_label ("Close");
   gtk_signal_connect (GTK_OBJECT (close_button), "clicked",
		       GTK_SIGNAL_FUNC (close_main_update_window), NULL);
   gtk_widget_show (close_button);
   gtk_box_pack_end (GTK_BOX (hbox), close_button, FALSE, FALSE, 2);

   make_error_window ();
   error_button = gtk_button_new_with_label ("Error Log");
   gtk_signal_connect_object (GTK_OBJECT (error_button), "clicked",
			      GTK_SIGNAL_FUNC (gtk_widget_show), 
			      GTK_OBJECT (error_log_window));
   gtk_widget_set_sensitive (error_button, false);
   gtk_widget_show (error_button);
   gtk_box_pack_end (GTK_BOX (hbox), error_button, FALSE, FALSE, 2);
   
   info_label = gtk_label_new (" ");
   gtk_misc_set_alignment (GTK_MISC (info_label), 0, 0.5);
   gtk_box_pack_end (GTK_BOX (hbox), info_label, TRUE, TRUE, 2);
   gtk_widget_show (info_label);
   
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 2);
     
   gtk_container_add (GTK_CONTAINER (upload_window), vbox);
   gtk_widget_show (upload_window);
   return 1;
}
   
int fe_catchup_site (void) {
   if (selected_site == NULL) {
      fe_status ("No site has been selected to catch-up.");
      return 0;
   }
   fe_gtk_question ("This will mark ALL files as already updated.\nAre you sure you want to do this?",
		(GnomeReplyCallback) catchup_selected);
   return 0;
}

void catchup_selected (gint button_number, gpointer data) {
   gchar result[BUFSIZ];
   
   if (button_number == GNOME_YES) {
      site_catchup (selected_site);
      sprintf (result, "All files in %s of site %s\nhave been marked as updated.",
	       selected_site->local_root, selected_site->name);
      fe_status (result);
      /* On huge sites, this may take ages? Progress bars/locking of ctree
       * selection may be advisable... */
      site_writefiles (selected_site);
      redraw_main_area();
      refresh_selected();
   }
}

int fe_init_site (void) {
   if (selected_site == NULL) {
      fe_status ("No site has been selected to initialise.");
      return 0;
   }
   fe_gtk_question ("This will mark all files as *NOT* updated on the remote site.\nAre you sure you want to do this?",
		(GnomeReplyCallback) initialize_selected);
   return 0;
}

void initialize_selected (gint button_number, 
			  gpointer data) {
   gchar result[BUFSIZ];
   
   switch (button_number) {
    case GNOME_YES:
      site_initialize (selected_site);
      sprintf (result, "All files in %s of site '%s'\nhave been marked as *NOT* updated.",
	       selected_site->local_root, selected_site->name);
      fe_status (result);
      /* On huge sites, this may take ages? Progress bars/locking of ctree
       * selection may be advisable... */
      site_writefiles (selected_site);
      redraw_main_area();
      refresh_selected();
      break;
    case GNOME_NO:
      break;
   }
}

void save_default (void) {
   gchar tmp[BUFSIZ];
   extern char *rcfile;
   extern bool rcfile_saved;
   extern GtkWidget *sitecopy;
   
   if (rcfile == NULL) {
      gnome_app_error (GNOME_APP (sitecopy), "The current rcfile is set to NULL. Sites will not be saved. Contact the maintainer.");
      return;
   }
   if (rcfile_write (rcfile, all_sites) == 0) {
      sprintf (tmp, "Site definitions saved to %s.", rcfile);
      fe_status (tmp);
   } else {
      fe_status ("There was an error writing the site definitions.\n They may not have saved correctly.");
   }
   rcfile_saved = true;
}

void delete_a_site (GtkWidget *button_or_menu, gpointer data) {
   gchar tmp[BUFSIZ];
   
   if (selected_site == NULL) {
      fe_status ("Cannot perform deletion - no site appears to be selected");
   } else {
      sprintf (tmp, "Are you sure you wish to permanently delete the record of '%s'?",
	       selected_site->name);
      fe_gtk_question (tmp, (GnomeReplyCallback) (delete_selected));
   }
}

struct site_t *find_prev_site (struct site_t *a_site) {
   struct site_t *tmp;
   tmp = all_sites;
   while ( tmp->next != NULL) {
      if ( strcmp (tmp->next->name, selected_site->name) == 0 ) {
	 return tmp;
      } else {
	 tmp = tmp->next;
      }
   }
   return NULL; /* Bad if this ever happens */
}

void delete_selected (gint button_number) {
   struct site_t *tmp, *tmp2;
   extern GtkCTree *the_tree;
   extern GtkCTreeNode *current_site_node;
   extern bool rcfile_saved;
   
   if (button_number != GNOME_YES)
     return;
   if ( (selected_site != NULL) && (all_sites != NULL) ) {
      if (strcmp (all_sites->name, selected_site->name) == 0) {
	 all_sites = all_sites->next;
      } else {
	 tmp = find_prev_site (selected_site);
	 if (tmp->next != NULL) {
	    tmp2 = tmp->next;
	    tmp->next = tmp2->next;
	 }
      }
      gtk_ctree_remove_node (GTK_CTREE (the_tree), 
			     GTK_CTREE_NODE (current_site_node));
      selected_site = NULL;
      current_site_node = NULL;
   } else {
      fe_status ("I've detected no site is selected, but one should be selected in order to get here. Oh dear.");
   }
   clear_main_area();
   rcfile_saved = false;
}
